@@ -1,6 +1,6 @@
 ;;; installed-scm-file
 
-;;;; 	Copyright (C) 1996 Free Software Foundation, Inc.
+;;;; 	Copyright (C) 1996, 1998 Free Software Foundation, Inc.
 ;;;; 
 ;;;; This program is free software; you can redistribute it and/or modify
 ;;;; it under the terms of the GNU General Public License as published by
@@ -14,12 +14,13 @@
 ;;;; 
 ;;;; You should have received a copy of the GNU General Public License
 ;;;; along with this software; see the file COPYING.  If not, write to
-;;;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+;;;; the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+;;;; Boston, MA 02111-1307 USA
 ;;;; 
 
 
 
-(define-module #/ice-9/lineio)
+(define-module (ice-9 lineio))
 
 
 ;;; {Line Buffering Input Ports}
@@ -31,6 +32,11 @@
 ;;;
 ;;; 	read-string  	which returns the next line of input
 ;;;	unread-string 	which pushes a line back onto the stream
+;;; 
+;;; The implementation of unread-string is kind of limited; it doesn't
+;;; interact properly with unread-char, or any of the other port
+;;; reading functions.  Only read-string will get you back the things that
+;;; unread-string accepts.
 ;;;
 ;;; Normally a "line" is all characters up to and including a newline.
 ;;; If lines are put back using unread-string, they can be broken arbitrarily
@@ -89,9 +95,7 @@
 
 	 (unread-string (lambda (str)
 			  (and (< 0 (string-length str))
-			       (if (ungetc-char-ready? self)
-				   (set! buffers (append! (list str (string (read-char self))) buffers))
-				   (set! buffers (cons str buffers))))))
+				   (set! buffers (cons str buffers)))))
 
 	 (read-string (lambda ()
 		       (cond
@@ -99,10 +103,8 @@
 			 (let ((answer (car buffers)))
 			   (set! buffers (cdr buffers))
 			   answer))
-			((ungetc-char-ready? self)
-			 (read-line self 'include-newline))
 			(else
-			 (read-line underlying-port 'include-newline))))))
+			 (read-line underlying-port 'concat)))))) ;handle-newline->concat
 
     (set-object-property! self 'unread-string unread-string)
     (set-object-property! self 'read-string read-string)
