@@ -2,7 +2,7 @@
 
 #ifndef __SCMH
 #define __SCMH
-/*	Copyright (C) 1995, 1996, 1998 Free Software Foundation, Inc.
+/*	Copyright (C) 1995, 1996, 1998, 1999 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -285,11 +285,12 @@ extern unsigned int scm_async_clock;
 
    However, GCC's volatile asm feature forms a barrier over which code is
    never moved. So if you add...
-	   asm volatile ("");
-   ...to each of the DEFER_INTS and ALLOW_INTS macros, the critical code
-   will always remain in place.  */
+	   asm ("");
+   ...to each of the DEFER_INTS and ALLOW_INTS macros, the critical
+   code will always remain in place.  asm's without inputs or outputs
+   are implicitly volatile. */
 #ifdef __GNUC__
-#define SCM_FENCE asm volatile ("")
+#define SCM_FENCE asm /* volatile */ ("")
 #else
 #define SCM_FENCE
 #endif
@@ -414,6 +415,48 @@ extern unsigned int scm_async_clock;
           goto _label
 #endif
 
+/*
+ * SCM_WTA_DISPATCH
+ */
+
+extern SCM scm_call_generic_0 (SCM gf);
+
+#define SCM_WTA_DISPATCH_0(gf, arg, pos, subr) \
+  return ((gf) \
+          ? scm_call_generic_0 ((gf)) \
+          : scm_wta ((arg), (char *) (pos), (subr)))
+#define SCM_GASSERT0(cond, gf, arg, pos, subr) \
+  if (!(cond)) SCM_WTA_DISPATCH_0((gf), (arg), (pos), (subr))
+
+extern SCM scm_call_generic_1 (SCM gf, SCM a1);
+
+#define SCM_WTA_DISPATCH_1(gf, a1, pos, subr) \
+  return ((gf) \
+          ? scm_call_generic_1 ((gf), (a1)) \
+          : scm_wta ((a1), (char *) (pos), (subr)))
+#define SCM_GASSERT1(cond, gf, a1, pos, subr) \
+  if (!(cond)) SCM_WTA_DISPATCH_1((gf), (a1), (pos), (subr))
+
+extern SCM scm_call_generic_2 (SCM gf, SCM a1, SCM a2);
+
+#define SCM_WTA_DISPATCH_2(gf, a1, a2, pos, subr) \
+  return ((gf) \
+          ? scm_call_generic_2 ((gf), (a1), (a2)) \
+          : scm_wta ((pos) == SCM_ARG1 ? (a1) : (a2), (char *) (pos), (subr)))
+#define SCM_GASSERT2(cond, gf, a1, a2, pos, subr) \
+  if (!(cond)) SCM_WTA_DISPATCH_2((gf), (a1), (a2), (pos), (subr))
+
+extern SCM scm_apply_generic (SCM gf, SCM args);
+
+#define SCM_WTA_DISPATCH_n(gf, args, pos, subr) \
+  return ((gf) \
+          ? scm_apply_generic ((gf), (args)) \
+          : scm_wta (scm_list_ref ((args), SCM_MAKINUM ((pos) - 1)), \
+		     (char *) (pos), \
+		     (subr)))
+#define SCM_GASSERTn(cond, gf, args, pos, subr) \
+  if (!(cond)) SCM_WTA_DISPATCH_n((gf), (args), (pos), (subr))
+
 #define SCM_ARGn 		0
 #define SCM_ARG1 		1
 #define SCM_ARG2 		2
