@@ -16,7 +16,8 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -40,8 +41,7 @@
  *
  * If you write modifications of your own for GUILE, it is your choice
  * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  
- */
+ * If you do not wish that, delete this exception notice.  */
 
 
 #include "__scm.h"
@@ -76,7 +76,18 @@
  */
 #include <errno.h>
 
-/* SCM_SYSCALL retries system calls that have been interrupted (EINTR) */
+/* SCM_SYSCALL retries system calls that have been interrupted (EINTR).
+   However this can be avoided if the operating system can restart
+   system calls automatically.  We assume this is the case if
+   sigaction is available and SA_RESTART is defined; they will be used
+   when installing signal handlers.
+   */
+
+#ifdef HAVE_RESTARTABLE_SYSCALLS
+#define SCM_SYSCALL(line) line
+#endif
+
+#ifndef SCM_SYSCALL
 #ifdef vms
 # ifndef __GNUC__
 #  include <ssdef.h>
@@ -84,6 +95,7 @@
 	while(EVMSERR==errno && (vaxc$errno>>3)==(SS$_CONTROLC>>3))
 # endif /* ndef __GNUC__ */
 #endif /* def vms */
+#endif /* ndef SCM_SYSCALL  */
 
 #ifndef SCM_SYSCALL
 # ifdef EINTR
@@ -94,7 +106,7 @@
 #endif /* ndef SCM_SYSCALL */
 
 #ifndef SCM_SYSCALL
-# define SCM_SYSCALL(line) {line;}
+# define SCM_SYSCALL(line) line;
 #endif /* ndef SCM_SYSCALL */
 
 #ifndef MSDOS
