@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
+/*	Copyright (C) 1995, 1996, 1998 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -161,7 +161,7 @@ scm_assv(x, alist)
     SCM_ASRTGO(SCM_NIMP(tmp) && SCM_CONSP(tmp), badlst);
     if SCM_NFALSEP(scm_eqv_p(SCM_CAR(tmp), x)) return tmp;
   }
-# ifndef RECKLESS
+# ifndef SCM_RECKLESS
   if (!(SCM_NULLP(alist)))
     badlst: scm_wta(alist, (char *)SCM_ARG2, s_assv);
 # endif
