@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995,1996,1997 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996,1997,1998 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -104,10 +104,8 @@ static long scm_tc16_async;
 
 
 
-static int asyncs_pending SCM_P ((void));
-
-static int
-asyncs_pending ()
+int
+scm_asyncs_pending ()
 {
   SCM pos;
   pos = scm_asyncs;
@@ -238,7 +236,7 @@ scm_async_click ()
   scm_run_asyncs (scm_asyncs);
 
   SCM_DEFER_INTS;
-  if (asyncs_pending ())
+  if (scm_asyncs_pending ())
     {
       SCM_ALLOW_INTS_ONLY;
       goto tail;
@@ -265,21 +263,6 @@ scm_switch ()
 
 
 
-static int print_async SCM_P ((SCM exp, SCM port, scm_print_state *pstate));
-
-static int
-print_async (exp, port, pstate)
-     SCM exp;
-     SCM port;
-     scm_print_state *pstate;
-{
-  scm_gen_puts (scm_regular_string, "#<async ", port);
-  scm_intprint(exp, 16, port);
-  scm_gen_putc('>', port);
-  return 1;
-}
-
-
 static SCM mark_async SCM_P ((SCM obj));
 
 static SCM
@@ -287,36 +270,10 @@ mark_async (obj)
      SCM obj;
 {
   struct scm_async * it;
-  if (SCM_GC8MARKP (obj))
-    return SCM_BOOL_F;
-  SCM_SETGC8MARK (obj);
   it = SCM_ASYNC (obj);
   return it->thunk;
 }
 
-
-static scm_sizet free_async SCM_P ((SCM obj));
-
-static scm_sizet
-free_async (obj)
-     SCM obj;
-{
-  struct scm_async * it;
-  it = SCM_ASYNC (obj);
-  scm_must_free ((char *)it);
-  return (sizeof (*it));
-}
-
-
-static scm_smobfuns  async_smob =
-{
-  mark_async,
-  free_async,
-  print_async,
-  0
-};
-
-
 
 
 SCM_PROC(s_async, "async", 1, 0, 0, scm_async);
@@ -325,19 +282,11 @@ SCM
 scm_async (thunk)
      SCM thunk;
 {
-  SCM it;
-  struct scm_async * async;
-
-  SCM_NEWCELL (it);
-  SCM_DEFER_INTS;
-  SCM_SETCDR (it, SCM_EOL);
-  async = (struct scm_async *)scm_must_malloc (sizeof (*async), s_async);
+  struct scm_async * async
+    = (struct scm_async *) scm_must_malloc (sizeof (*async), s_async);
   async->got_it = 0;
   async->thunk = thunk;
-  SCM_SETCDR (it, (SCM)async);
-  SCM_SETCAR (it, (SCM)scm_tc16_async);
-  SCM_ALLOW_INTS;
-  return it;
+  SCM_RETURN_NEWSMOB (scm_tc16_async, async);
 }
 
 SCM_PROC(s_system_async, "system-async", 1, 0, 0, scm_system_async);
@@ -350,12 +299,8 @@ scm_system_async (thunk)
   SCM list;
 
   it = scm_async (thunk);
-  SCM_NEWCELL (list);
-  SCM_DEFER_INTS;
-  SCM_SETCAR (list, it);
-  SCM_SETCDR (list, scm_asyncs);
+  SCM_NEWSMOB (list, it, scm_asyncs);
   scm_asyncs = list;
-  SCM_ALLOW_INTS;
   return it;
 }
 
@@ -522,8 +467,8 @@ void
 scm_init_async ()
 {
   SCM a_thunk;
-  scm_tc16_async = scm_newsmob (&async_smob);
-
+  scm_tc16_async = scm_make_smob_type_mfpe ("async", sizeof (struct scm_async),
+                                           mark_async, NULL, NULL, NULL);
   scm_gc_vcell = scm_sysintern ("gc-thunk", SCM_BOOL_F);
   a_thunk = scm_make_gsubr ("%gc-thunk", 0, 0, 0, scm_sys_gc_async_thunk);
   scm_gc_async = scm_system_async (a_thunk);
