@@ -3,7 +3,7 @@
 #ifndef COOP_DEFSH
 #define COOP_DEFSH
 
-/*	Copyright (C) 1996 Free Software Foundation, Inc.
+/*	Copyright (C) 1996, 1997, 1998 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -58,6 +58,10 @@
 #  endif
 # endif
 
+#ifdef GUILE_ISELECT
+#include "iselect.h"
+#endif
+
 /* This file is included by threads.h, which, in turn, is included by
    libguile.h while coop-threads.h only is included by
    coop-threads.c. */
@@ -89,17 +93,92 @@ typedef struct coop_t {
   void *joining;         /* A queue of threads waiting to join this
 			    thread */
 
+#ifdef GUILE_ISELECT
+  int nfds;
+  SELECT_TYPE *readfds;
+  SELECT_TYPE *writefds;
+  SELECT_TYPE *exceptfds;
+  int timeoutp;
+  struct timeval wakeup_time;	/* Time to stop sleeping */
+  int _errno;
+  int retval;
+#else
   time_t wakeup_time;    /* Time to stop sleeping */
+#endif
 
 } coop_t;
 
+/* A queue is a circular list of threads.  The queue head is a
+   designated list element.  If this is a uniprocessor-only
+   implementation we can store the `main' thread in this, but in a
+   multiprocessor there are several `heavy' threads but only one run
+   queue.  A fancier implementation might have private run queues,
+   which would lead to a simpler (trivial) implementation */
+
+typedef struct coop_q_t {
+  coop_t t;
+  coop_t *tail;
+} coop_q_t;
+
+/* A Mutex variable is made up of a owner thread, and a queue of threads
+   waiting on the mutex */
+
+typedef struct coop_m {
+  coop_t *owner;          /* Mutex owner */
+  coop_q_t waiting;      /* Queue of waiting threads */
+} coop_m;
+
+typedef coop_m scm_mutex_t;
+
+extern int coop_mutex_init (coop_m*);
+extern int coop_mutex_lock (coop_m*);
+extern int coop_mutex_unlock (coop_m*);
+extern int coop_mutex_destroy (coop_m*);
+#define scm_mutex_init coop_mutex_init
+#define scm_mutex_lock coop_mutex_lock
+#define scm_mutex_unlock coop_mutex_unlock
+#define scm_mutex_destroy coop_mutex_destroy
+
+/* A Condition variable is made up of a list of threads waiting on the
+   condition. */
+
+typedef struct coop_c {
+  coop_q_t waiting;      /* Queue of waiting threads */
+} coop_c;
+
+typedef coop_c scm_cond_t;
+
+extern int coop_condition_variable_init (coop_c*);
+extern int coop_condition_variable_wait_mutex (coop_c*, coop_m*);
+extern int coop_condition_variable_signal (coop_c*);
+extern int coop_condition_variable_destroy (coop_c*);
+#define scm_cond_init(cond, attr) coop_condition_variable_init (cond)
+#define scm_cond_wait coop_condition_variable_wait_mutex
+#define scm_cond_signal coop_condition_variable_signal
+#define scm_cond_destroy coop_condition_variable_destroy
+
 extern coop_t *coop_global_curr;       	/* Currently-executing thread. */
 
+extern void coop_join (coop_t *t);
 extern void coop_yield (void);
 
 extern size_t scm_switch_counter;
 extern size_t scm_thread_count;
 
+
+/* Some iselect functions.  */ 
+
+/* I'm not sure whether these three declarations should be here.
+   They're really defined in iselect.c, so you'd think they'd go in
+   iselect.h, but they use coop_t, defined above, which uses things
+   defined in iselect.h.  Basically, we're making at best a flailing
+   (and failing) attempt at modularity here, and I don't have time to
+   rethink this at the moment.  This code awaits a Hero.  --JimB */
+extern coop_t *coop_next_runnable_thread (void);
+extern coop_t *coop_wait_for_runnable_thread_now (struct timeval *);
+extern coop_t *coop_wait_for_runnable_thread (void);
+
+
 
 
 /* Cooperative threads don't need to have these defined */
