@@ -2,7 +2,7 @@
 
 #ifndef DYNWINDH
 #define DYNWINDH
-/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
+/*	Copyright (C) 1995, 1996, 1998, 1999 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -47,8 +47,20 @@
 
 
 
+typedef void (*scm_guard_t) (void *);
+typedef SCM (*scm_inner_t) (void *);
+
 extern SCM scm_dynamic_wind SCM_P ((SCM thunk1, SCM thunk2, SCM thunk3));
+extern SCM scm_internal_dynamic_wind SCM_P ((scm_guard_t before,
+					     scm_inner_t inner,
+					     scm_guard_t after,
+					     void *inner_data,
+					     void *guard_data));
 extern void scm_dowinds SCM_P ((SCM to, long delta));
 extern void scm_init_dynwind SCM_P ((void));
 
+#ifdef GUILE_DEBUG
+extern SCM scm_wind_chain SCM_P ((void));
+#endif /*GUILE_DEBUG*/
+
 #endif  /* DYNWINDH */
