@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995,1996,1997 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996,1997,1998 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -70,13 +70,13 @@ scm_eqv_p (x, y)
      SCM y;
 {
   if (x==y) return SCM_BOOL_T;
-  if SCM_IMP(x) return SCM_BOOL_F;
-  if SCM_IMP(y) return SCM_BOOL_F;
+  if (SCM_IMP(x)) return SCM_BOOL_F;
+  if (SCM_IMP(y)) return SCM_BOOL_F;
   /* this ensures that types and scm_length are the same. */
   if (SCM_CAR(x) != SCM_CAR(y)) return SCM_BOOL_F;
-  if SCM_NUMP(x) {
+  if (SCM_NUMP(x)) {
 # ifdef SCM_BIGDIG
-    if SCM_BIGP(x) return (0==scm_bigcomp(x, y)) ? SCM_BOOL_T : SCM_BOOL_F;
+    if (SCM_BIGP(x)) return (0==scm_bigcomp(x, y)) ? SCM_BOOL_T : SCM_BOOL_F;
 # endif
 #ifdef SCM_FLOATS
     if (SCM_REALPART(x) != SCM_REALPART(y)) return SCM_BOOL_F;
@@ -96,7 +96,7 @@ scm_equal_p (x, y)
      SCM y;
 {
   SCM_CHECK_STACK;
- tailrecurse: SCM_ASYNC_TICK;
+ tailrecurse: SCM_TICK;
 	if (x==y) return SCM_BOOL_T;
 	if (SCM_IMP(x)) return SCM_BOOL_F;
 	if (SCM_IMP(y)) return SCM_BOOL_F;
@@ -106,8 +106,8 @@ scm_equal_p (x, y)
 		y = SCM_CDR(y);
 		goto tailrecurse;
 	}
-	if (SCM_TYP7SD (x) == scm_tc7_string
-	    && SCM_TYP7SD (y) == scm_tc7_string)
+	if (SCM_TYP7S (x) == scm_tc7_string
+	    && SCM_TYP7S (y) == scm_tc7_string)
 	  return scm_string_equal_p (x, y);
 	/* This ensures that types and scm_length are the same.  */
 	if (SCM_CAR(x) != SCM_CAR(y)) return SCM_BOOL_F;
