@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995,1996,1997 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996,1997,1998 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -60,21 +60,16 @@
 
 extern int errno;
 
-void (*scm_error_callback) () = 0;
-
 /* All errors should pass through here.  */
 void
 scm_error (key, subr, message, args, rest)
      SCM key;
-     char *subr;
-     char *message;
+     const char *subr;
+     const char *message;
      SCM args;
      SCM rest;
 {
   SCM arg_list;
-  if (scm_error_callback)
-    (*scm_error_callback) (key, subr, message, args, rest);
-
   arg_list = scm_listify (subr ? scm_makfrom0str (subr) : SCM_BOOL_F,
 			  message ? scm_makfrom0str (message) : SCM_BOOL_F,
 			  args,
@@ -116,10 +111,18 @@ scm_error_scm (key, subr, message, args, rest)
   /* not reached.  */
 }
 
+SCM_PROC (s_strerror, "strerror", 1, 0, 0, scm_strerror);
+SCM
+scm_strerror (SCM err)
+{
+  SCM_ASSERT (SCM_INUMP (err), err, SCM_ARG1, s_strerror);
+  return scm_makfrom0str (strerror (SCM_INUM (err)));
+}
+
 SCM_SYMBOL (scm_system_error_key, "system-error");
 void
 scm_syserror (subr)
-     char *subr;
+     const char *subr;
 {
   scm_error (scm_system_error_key,
 	     subr,
@@ -131,8 +134,8 @@ scm_syserror (subr)
 
 void
 scm_syserror_msg (subr, message, args, eno)
-     char *subr;
-     char *message;
+     const char *subr;
+     const char *message;
      SCM args;
      int eno;
 {
@@ -145,7 +148,7 @@ scm_syserror_msg (subr, message, args, eno)
 
 void
 scm_sysmissing (subr)
-     char *subr;
+     const char *subr;
 {
 #ifdef ENOSYS
   scm_error (scm_system_error_key,
@@ -165,7 +168,7 @@ scm_sysmissing (subr)
 SCM_SYMBOL (scm_num_overflow_key, "numerical-overflow");
 void
 scm_num_overflow (subr)
-  char *subr;
+  const char *subr;
 {
   scm_error (scm_num_overflow_key,
 	     subr,
@@ -177,7 +180,7 @@ scm_num_overflow (subr)
 SCM_SYMBOL (scm_out_of_range_key, "out-of-range");
 void
 scm_out_of_range (subr, bad_value)
-     char *subr;
+     const char *subr;
      SCM bad_value;
 {
   scm_error (scm_out_of_range_key,
@@ -202,7 +205,7 @@ scm_wrong_num_args (proc)
 SCM_SYMBOL (scm_arg_type_key, "wrong-type-arg");
 void
 scm_wrong_type_arg (subr, pos, bad_value)
-     char *subr;
+     const char *subr;
      int pos;
      SCM bad_value;
 {
@@ -218,7 +221,7 @@ scm_wrong_type_arg (subr, pos, bad_value)
 SCM_SYMBOL (scm_memory_alloc_key, "memory-allocation-error");
 void
 scm_memory_error (subr)
-     char *subr;
+     const char *subr;
 {
   scm_error (scm_memory_alloc_key,
 	     subr,
@@ -230,8 +233,8 @@ scm_memory_error (subr)
 SCM_SYMBOL (scm_misc_error_key, "misc-error");
 void
 scm_misc_error (subr, message, args)
-     char *subr;
-     char *message;
+     const char *subr;
+     const char *message;
      SCM args;
 {
   scm_error (scm_misc_error_key, subr, message, args, SCM_BOOL_F);
@@ -241,15 +244,15 @@ scm_misc_error (subr, message, args)
 SCM
 scm_wta (arg, pos, s_subr)
      SCM arg;
-     char *pos;
-     char *s_subr;
+     const char *pos;
+     const char *s_subr;
 {
   if (!s_subr || !*s_subr)
     s_subr = NULL;
   if ((~0x1fL) & (long) pos)
     {
       /* error string supplied.  */
-      scm_misc_error (s_subr, pos, SCM_BOOL_F);
+      scm_misc_error (s_subr, pos, SCM_LIST1 (arg));
     }
   else
     {
@@ -270,6 +273,10 @@ scm_wta (arg, pos, s_subr)
 	  scm_wrong_type_arg (s_subr, 4, arg);
 	case SCM_ARG5:
 	  scm_wrong_type_arg (s_subr, 5, arg);
+	case SCM_ARG6:
+	  scm_wrong_type_arg (s_subr, 6, arg);
+	case SCM_ARG7:
+	  scm_wrong_type_arg (s_subr, 7, arg);
 	case SCM_WNA:
 	  scm_wrong_num_args (arg);
 	case SCM_OUTOFRANGE:
@@ -278,7 +285,7 @@ scm_wta (arg, pos, s_subr)
 	  scm_memory_error (s_subr);
 	default:
 	  /* this shouldn't happen.  */
-	  scm_misc_error (s_subr, "Unknown error", SCM_BOOL_F);
+	  scm_misc_error (s_subr, "Unknown error", SCM_EOL);
 	}
     }
   return SCM_UNSPECIFIED;
