@@ -2,7 +2,7 @@
 
 #ifndef EVALH
 #define EVALH
-/*	Copyright (C) 1995,1996,1998 Free Software Foundation, Inc.
+/*	Copyright (C) 1995, 1996 ,1998, 1999 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -152,6 +152,8 @@ extern SCM scm_sym_apply;
 extern SCM scm_sym_set_x;
 extern SCM scm_sym_args;
 
+extern SCM scm_f_apply;
+
 extern long scm_tc16_macro;
 
 /* A resolved global variable reference in the CAR position
@@ -170,6 +172,7 @@ extern SCM * scm_lookupcar SCM_P ((SCM vloc, SCM genv, int check));
 extern SCM scm_unmemocar SCM_P ((SCM form, SCM env));
 extern SCM scm_unmemocopy SCM_P ((SCM form, SCM env));
 extern SCM scm_eval_car SCM_P ((SCM pair, SCM env));
+extern SCM scm_eval_body (SCM code, SCM env);
 extern SCM scm_eval_args SCM_P ((SCM i, SCM env, SCM proc));
 extern SCM scm_deval_args SCM_P ((SCM l, SCM env, SCM proc, SCM *lloc));
 extern SCM scm_m_quote SCM_P ((SCM xorig, SCM env));
