@@ -563,6 +563,11 @@ scm_igc (what)
   while (j--)
     scm_gc_mark (scm_sys_protects[j]);
 
+  /* FIXME: we should have a means to register C functions to be run
+   * in different phases of GC
+   */ 
+  scm_mark_subr_table ();
+  
 #ifndef USE_THREADS
   scm_gc_mark (scm_root->handle);
 #endif
@@ -660,10 +665,7 @@ gc_mark_nimp:
 	      
 	      if (vtable_data[scm_struct_i_flags] & SCM_STRUCTF_ENTITY)
 		{
-		  scm_gc_mark (mem[scm_struct_i_proc + 0]);
-		  scm_gc_mark (mem[scm_struct_i_proc + 1]);
-		  scm_gc_mark (mem[scm_struct_i_proc + 2]);
-		  scm_gc_mark (mem[scm_struct_i_proc + 3]);
+		  scm_gc_mark (mem[scm_struct_i_procedure]);
 		  scm_gc_mark (mem[scm_struct_i_setter]);
 		}
 	      if (len)
@@ -830,8 +832,7 @@ gc_mark_nimp:
       SCM_SETGC8MARK (ptr);
       break;
     case scm_tcs_subrs:
-      ptr = (SCM)(scm_heap_org + (((unsigned long)SCM_CAR (ptr)) >> 8));
-      goto gc_mark_loop;
+      break;
     case scm_tc7_port:
       i = SCM_PTOBNUM (ptr);
       if (!(i < scm_numptob))
@@ -1427,15 +1428,15 @@ scm_gc_sweep ()
  *
  * The limit scm_mtrigger may be raised by this allocation.
  */
-char *
+void *
 scm_must_malloc (scm_sizet size, const char *what)
 {
-  char *ptr;
+  void *ptr;
   unsigned long nm = scm_mallocated + size;
 
   if (nm <= scm_mtrigger)
     {
-      SCM_SYSCALL (ptr = (char *) malloc (size));
+      SCM_SYSCALL (ptr = malloc (size));
       if (NULL != ptr)
 	{
 	  scm_mallocated = nm;
@@ -1446,7 +1447,7 @@ scm_must_malloc (scm_sizet size, const char *what)
   scm_igc (what);
 
   nm = scm_mallocated + size;
-  SCM_SYSCALL (ptr = (char *) malloc (size));
+  SCM_SYSCALL (ptr = malloc (size));
   if (NULL != ptr)
     {
       scm_mallocated = nm;
@@ -1467,18 +1468,18 @@ scm_must_malloc (scm_sizet size, const char *what)
 /* scm_must_realloc
  * is similar to scm_must_malloc.
  */
-char *
-scm_must_realloc (char *where,
+void *
+scm_must_realloc (void *where,
 		  scm_sizet old_size,
 		  scm_sizet size,
 		  const char *what)
 {
-  char *ptr;
+  void *ptr;
   scm_sizet nm = scm_mallocated + size - old_size;
 
   if (nm <= scm_mtrigger)
     {
-      SCM_SYSCALL (ptr = (char *) realloc (where, size));
+      SCM_SYSCALL (ptr = realloc (where, size));
       if (NULL != ptr)
 	{
 	  scm_mallocated = nm;
@@ -1489,7 +1490,7 @@ scm_must_realloc (char *where,
   scm_igc (what);
 
   nm = scm_mallocated + size - old_size;
-  SCM_SYSCALL (ptr = (char *) realloc (where, size));
+  SCM_SYSCALL (ptr = realloc (where, size));
   if (NULL != ptr)
     {
       scm_mallocated = nm;
@@ -1507,8 +1508,7 @@ scm_must_realloc (char *where,
 }
 
 void 
-scm_must_free (obj)
-     char *obj;
+scm_must_free (void *obj)
 {
   if (obj)
     free (obj);
@@ -1858,7 +1858,19 @@ scm_unprotect_object (obj)
 int terminating;
 
 /* called on process termination.  */
-static void cleanup (void)
+#ifdef HAVE_ATEXIT
+static void
+cleanup (void)
+#else
+#ifdef HAVE_ON_EXIT
+extern int on_exit (void (*procp) (), int arg);
+
+static void
+cleanup (int status, void *arg)
+#else
+#error Dont know how to setup a cleanup handler on your system.
+#endif
+#endif
 {
   terminating = 1;
   scm_flush_all_ports ();
@@ -1903,7 +1915,13 @@ scm_init_storage (scm_sizet init_heap_size)
   if (!scm_port_table)
     return 1;
 
+#ifdef HAVE_ATEXIT
   atexit (cleanup);
+#else
+#ifdef HAVE_ON_EXIT
+  on_exit (cleanup, 0);
+#endif
+#endif
 
   scm_undefineds = scm_cons (SCM_UNDEFINED, SCM_EOL);
   SCM_SETCDR (scm_undefineds, scm_undefineds);
