@@ -2,7 +2,7 @@
 
 #ifndef GCH
 #define GCH
-/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
+/*	Copyright (C) 1995, 1996, 1998, 1999 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -50,6 +50,13 @@
 #define SCM_FREEP(x) (SCM_CAR(x)==scm_tc_free_cell)
 #define SCM_NFREEP(x) (!SCM_FREEP(x))
 
+/* 1. This shouldn't be used on immediates.
+   2. It thinks that subrs are always unmarked (harmless). */
+#define SCM_MARKEDP(x) ((SCM_CAR(x) & 5) == 5 \
+			? SCM_GC8MARKP(x) \
+			: SCM_GCMARKP(x))
+#define SCM_NMARKEDP(x) (!SCM_MARKEDP(x))
+
 extern struct scm_heap_seg_data *scm_heap_table;
 extern int scm_n_heap_segs;
 extern int scm_take_stdin;
@@ -58,44 +65,46 @@ extern int scm_gc_heap_lock;
 
 
 
-extern long scm_heap_size;
+extern unsigned long scm_heap_size;
 extern SCM_CELLPTR scm_heap_org;
 extern SCM scm_freelist;
 extern unsigned long scm_gc_cells_collected;
 extern unsigned long scm_gc_malloc_collected;
 extern unsigned long scm_gc_ports_collected;
 extern unsigned long scm_cells_allocated;
-extern unsigned long scm_mallocated;
-extern long scm_mtrigger;
+extern long scm_mallocated;
+extern unsigned long scm_mtrigger;
 
 #ifdef DEBUG_FREELIST
-extern void scm_debug_newcell SCM_P ((SCM *into));
+extern SCM scm_debug_newcell (void);
 #endif
 
 
 
-extern SCM scm_object_addr SCM_P ((SCM obj));
-extern SCM scm_unhash_name SCM_P ((SCM name));
-extern SCM scm_gc_stats SCM_P ((void));
-extern void scm_gc_start SCM_P ((char *what));
-extern void scm_gc_end SCM_P ((void));
-extern SCM scm_gc SCM_P ((void));
-extern void scm_gc_for_alloc SCM_P ((int ncells, SCM * freelistp));
-extern SCM scm_gc_for_newcell SCM_P ((void));
-extern void scm_igc SCM_P ((char *what));
-extern void scm_gc_mark SCM_P ((SCM p));
-extern void scm_mark_locations SCM_P ((SCM_STACKITEM x[], scm_sizet n));
-extern int scm_cellp SCM_P ((SCM value));
-extern void scm_gc_sweep SCM_P ((void));
-extern char * scm_must_malloc SCM_P ((long len, char *what));
-extern char * scm_must_realloc SCM_P ((char *where, long olen, long len,
-				       char *what));
-extern void scm_must_free SCM_P ((char *obj));
-extern void scm_remember SCM_P ((SCM * ptr));
-extern SCM scm_return_first SCM_P ((SCM elt, ...));
-extern SCM scm_permanent_object SCM_P ((SCM obj));
-extern SCM scm_protect_object SCM_P ((SCM obj));
-extern SCM scm_unprotect_object SCM_P ((SCM obj));
-extern int scm_init_storage SCM_P ((long init_heap_size));
-extern void scm_init_gc SCM_P ((void));
+extern SCM scm_object_address (SCM obj);
+extern SCM scm_unhash_name (SCM name);
+extern SCM scm_gc_stats (void);
+extern void scm_gc_start (const char *what);
+extern void scm_gc_end (void);
+extern SCM scm_gc (void);
+extern void scm_gc_for_alloc (int ncells, SCM * freelistp);
+extern SCM scm_gc_for_newcell (void);
+extern void scm_igc (const char *what);
+extern void scm_gc_mark (SCM p);
+extern void scm_mark_locations (SCM_STACKITEM x[], scm_sizet n);
+extern int scm_cellp (SCM value);
+extern void scm_gc_sweep (void);
+extern void * scm_must_malloc (scm_sizet len, const char *what);
+extern void * scm_must_realloc (void *where,
+				scm_sizet olen, scm_sizet len,
+				const char *what);
+extern void scm_done_malloc (long size);
+extern void scm_must_free (void *obj);
+extern void scm_remember (SCM * ptr);
+extern SCM scm_return_first (SCM elt, ...);
+extern SCM scm_permanent_object (SCM obj);
+extern SCM scm_protect_object (SCM obj);
+extern SCM scm_unprotect_object (SCM obj);
+extern int scm_init_storage (scm_sizet init_heap_size);
+extern void scm_init_gc (void);
 #endif  /* GCH */
