@@ -50,6 +50,12 @@ gh_display (SCM x)
   scm_display (x, scm_current_output_port ());
 }
 
+void 
+gh_write (SCM x)
+{
+  scm_write (x, scm_current_output_port ());
+}
+
 void 
 gh_newline ()
 {
