@@ -1,7 +1,7 @@
 #ifndef LIBGUILEH
 #define LIBGUILEH
 
-/*	Copyright (C) 1995, 1996 Free Software Foundation, Inc.
+/*	Copyright (C) 1995, 1996, 1997, 1998 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,7 +43,9 @@
  * If you do not wish that, delete this exception notice.  */
 
 
-
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 #include "libguile/__scm.h"
 
@@ -54,7 +56,6 @@
 #include "libguile/pairs.h"
 
 #include "libguile/alist.h"
-#include "libguile/append.h"
 #include "libguile/arbiters.h"
 #include "libguile/async.h"
 #include "libguile/boolean.h"
@@ -65,30 +66,35 @@
 #include "libguile/debug.h"
 #include "libguile/stacks.h"
 #endif
+#include "libguile/dynl.h"
 #include "libguile/dynwind.h"
 #include "libguile/eq.h"
 #include "libguile/error.h"
 #include "libguile/eval.h"
-#include "libguile/extchrs.h"
+#include "libguile/evalext.h"
 #include "libguile/feature.h"
 #include "libguile/filesys.h"
+#include "libguile/fluids.h"
 #include "libguile/fports.h"
 #include "libguile/gc.h"
 #include "libguile/gdbint.h"
 #include "libguile/genio.h"
 #include "libguile/gsubr.h"
+#include "libguile/guardians.h"
 #include "libguile/hash.h"
 #include "libguile/hashtab.h"
 #include "libguile/init.h"
 #include "libguile/ioext.h"
+#include "libguile/keywords.h"
 #include "libguile/kw.h"
 #include "libguile/list.h"
 #include "libguile/load.h"
+#include "libguile/macros.h"
 #include "libguile/mallocs.h"
-#include "libguile/markers.h"
-#include "libguile/mbstrings.h"
+#include "libguile/modules.h"
 #include "libguile/net_db.h"
 #include "libguile/numbers.h"
+#include "libguile/objects.h"
 #include "libguile/objprop.h"
 #include "libguile/options.h"
 #include "libguile/ports.h"
@@ -96,14 +102,15 @@
 #include "libguile/procprop.h"
 #include "libguile/procs.h"
 #include "libguile/ramap.h"
+#include "libguile/random.h"
 #include "libguile/read.h"
 #include "libguile/root.h"
 #include "libguile/scmsigs.h"
 #include "libguile/script.h"
-#include "libguile/sequences.h"
 #include "libguile/simpos.h"
 #include "libguile/snarf.h"
 #include "libguile/socket.h"
+#include "libguile/sort.h"
 #include "libguile/srcprop.h"
 #include "libguile/stackchk.h"
 #include "libguile/stime.h"
@@ -126,6 +133,10 @@
 #include "libguile/threads.h"
 #endif
 
+#ifdef __cplusplus
+}
+#endif
+
 
 
 #endif  /* LIBGUILEH */
