@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996,1997 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -93,6 +93,25 @@ scm_list(objs)
 }
 
 
+SCM_PROC (s_list_star, "list*", 1, 0, 1, scm_list_star);
+
+SCM
+scm_list_star (SCM arg, SCM rest)
+{
+  if (SCM_NIMP (rest))
+    {
+      SCM prev = arg = scm_cons (arg, rest);
+      while (SCM_NIMP (SCM_CDR (rest)))
+	{
+	  prev = rest;
+	  rest = SCM_CDR (rest);
+	}
+      SCM_SETCDR (prev, SCM_CAR (rest));
+    }
+  return arg;
+}
+
+
 
 
 /* general questions about lists --- null?, list?, length, etc.  */
@@ -130,12 +149,12 @@ scm_ilength(sx)
   register SCM hare = sx;
 
   do {
-    if SCM_IMP(hare) return SCM_NULLP(hare) ? i : -1;
-    if SCM_NCONSP(hare) return -1;
+    if (SCM_IMP(hare)) return SCM_NULLP(hare) ? i : -1;
+    if (SCM_NCONSP(hare)) return -1;
     hare = SCM_CDR(hare);
     i++;
-    if SCM_IMP(hare) return SCM_NULLP(hare) ? i : -1;
-    if SCM_NCONSP(hare) return -1;
+    if (SCM_IMP(hare)) return SCM_NULLP(hare) ? i : -1;
+    if (SCM_NCONSP(hare)) return -1;
     hare = SCM_CDR(hare);
     i++;
     /* For every two steps the hare takes, the tortoise takes one.  */
@@ -148,14 +167,14 @@ scm_ilength(sx)
   return -1;
 }
 
-SCM_PROC(s_list_length, "list-length", 1, 0, 0, scm_list_length);
+SCM_PROC(s_length, "length", 1, 0, 0, scm_length);
 SCM
-scm_list_length(x)
+scm_length(x)
      SCM x;
 {
   int i;
   i = scm_ilength(x);
-  SCM_ASSERT(i >= 0, x, SCM_ARG1, s_list_length);
+  SCM_ASSERT(i >= 0, x, SCM_ARG1, s_length);
   return SCM_MAKINUM (i);
 }
 
@@ -163,51 +182,51 @@ scm_list_length(x)
 
 /* appending lists */
 
-SCM_PROC (s_list_append, "list-append", 0, 0, 1, scm_list_append);
+SCM_PROC (s_append, "append", 0, 0, 1, scm_append);
 SCM
-scm_list_append(args)
+scm_append(args)
      SCM args;
 {
   SCM res = SCM_EOL;
   SCM *lloc = &res, arg;
-  if SCM_IMP(args) {
-    SCM_ASSERT(SCM_NULLP(args), args, SCM_ARGn, s_list_append);
+  if (SCM_IMP(args)) {
+    SCM_ASSERT(SCM_NULLP(args), args, SCM_ARGn, s_append);
     return res;
   }
-  SCM_ASSERT(SCM_CONSP(args), args, SCM_ARGn, s_list_append);
+  SCM_ASSERT(SCM_CONSP(args), args, SCM_ARGn, s_append);
   while (1) {
     arg = SCM_CAR(args);
     args = SCM_CDR(args);
-    if SCM_IMP(args) {
+    if (SCM_IMP(args)) {
       *lloc = arg;
-      SCM_ASSERT(SCM_NULLP(args), args, SCM_ARGn, s_list_append);
+      SCM_ASSERT(SCM_NULLP(args), args, SCM_ARGn, s_append);
       return res;
     }
-    SCM_ASSERT(SCM_CONSP(args), args, SCM_ARGn, s_list_append);
+    SCM_ASSERT(SCM_CONSP(args), args, SCM_ARGn, s_append);
     for(;SCM_NIMP(arg);arg = SCM_CDR(arg)) {
-      SCM_ASSERT(SCM_CONSP(arg), arg, SCM_ARGn, s_list_append);
+      SCM_ASSERT(SCM_CONSP(arg), arg, SCM_ARGn, s_append);
       *lloc = scm_cons(SCM_CAR(arg), SCM_EOL);
       lloc = SCM_CDRLOC(*lloc);
     }
-    SCM_ASSERT(SCM_NULLP(arg), arg, SCM_ARGn, s_list_append);
+    SCM_ASSERT(SCM_NULLP(arg), arg, SCM_ARGn, s_append);
   }
 }
 
 
-SCM_PROC (s_list_append_x, "list-append!", 0, 0, 1, scm_list_append_x);
+SCM_PROC (s_append_x, "append!", 0, 0, 1, scm_append_x);
 SCM
-scm_list_append_x(args)
+scm_append_x(args)
      SCM args;
 {
   SCM arg;
  tail:
-  if SCM_NULLP(args) return SCM_EOL;
+  if (SCM_NULLP(args)) return SCM_EOL;
   arg = SCM_CAR(args);
-  SCM_ASSERT(SCM_NULLP(arg) || (SCM_NIMP(arg) && SCM_CONSP(arg)), arg, SCM_ARG1, s_list_append_x);
   args = SCM_CDR(args);
-  if SCM_NULLP(args) return arg;
-  if SCM_NULLP(arg) goto tail;
-  SCM_SETCDR (scm_last_pair (arg), scm_list_append_x (args));
+  if (SCM_NULLP(args)) return arg;
+  if (SCM_NULLP(arg)) goto tail;
+  SCM_ASSERT(SCM_NIMP(arg) && SCM_CONSP(arg), arg, SCM_ARG1, s_append_x);
+  SCM_SETCDR (scm_last_pair (arg), scm_append_x (args));
   return arg;
 }
 
@@ -239,43 +258,52 @@ scm_last_pair(sx)
 
 /* reversing lists */
 
-SCM_PROC (s_list_reverse, "list-reverse", 1, 0, 0, scm_list_reverse);
+SCM_PROC (s_reverse, "reverse", 1, 0, 0, scm_reverse);
+
 SCM
-scm_list_reverse(lst)
-     SCM lst;
+scm_reverse (SCM ls)
 {
-	SCM res = SCM_EOL;
-	SCM p = lst;
-	for(;SCM_NIMP(p);p = SCM_CDR(p)) {
-		SCM_ASSERT(SCM_CONSP(p), lst, SCM_ARG1, s_list_reverse);
-		res = scm_cons(SCM_CAR(p), res);
-	}
-	SCM_ASSERT(SCM_NULLP(p), lst, SCM_ARG1, s_list_reverse);
-	return res;
+  SCM res = SCM_EOL;
+  SCM p = ls, t = ls;
+  while (SCM_NIMP (p))
+    {
+      SCM_ASSERT (SCM_CONSP (p), ls, SCM_ARG1, s_reverse);
+      res = scm_cons (SCM_CAR (p), res);
+      p = SCM_CDR (p);
+      if (SCM_IMP (p))
+	break;
+      SCM_ASSERT (SCM_CONSP (p), ls, SCM_ARG1, s_reverse);
+      res = scm_cons (SCM_CAR (p), res);
+      p = SCM_CDR (p);
+      t = SCM_CDR (t);
+      if (t == p)
+	scm_misc_error (s_reverse, "Circular structure: %S", SCM_LIST1 (ls));
+    }
+  SCM_ASSERT (SCM_NULLP (p), ls, SCM_ARG1, s_reverse);
+  return res;
 }
 
-SCM_PROC (s_list_reverse_x, "list-reverse!", 1, 1, 0, scm_list_reverse_x);
+SCM_PROC (s_reverse_x, "reverse!", 1, 1, 0, scm_reverse_x);
 SCM
-scm_list_reverse_x (lst, newtail)
-     SCM lst;
-     SCM newtail;
+scm_reverse_x (ls, new_tail)
+     SCM ls;
+     SCM new_tail;
 {
   SCM old_tail;
-  if (newtail == SCM_UNDEFINED)
-    newtail = SCM_EOL;
+  SCM_ASSERT (scm_ilength (ls) >= 0, ls, SCM_ARG1, s_reverse_x);
+  if (SCM_UNBNDP (new_tail))
+    new_tail = SCM_EOL;
+  else
+    SCM_ASSERT (scm_ilength (new_tail) >= 0, new_tail, SCM_ARG2, s_reverse_x);
 
- loop:
-  if (!(SCM_NIMP (lst) && SCM_CONSP (lst)))
-    return lst;
-
-  old_tail = SCM_CDR (lst);
-  SCM_SETCDR (lst, newtail);
-  if (SCM_NULLP (old_tail))
-    return lst;
-
-  newtail = lst;
-  lst = old_tail;
-  goto loop;
+  while (SCM_NIMP (ls))
+    {
+      old_tail = SCM_CDR (ls);
+      SCM_SETCDR (ls, new_tail);
+      new_tail = ls;
+      ls = old_tail;
+    }
+  return new_tail;
 }
 
 
@@ -418,18 +446,6 @@ scm_list_copy (lst)
 
 /* membership tests (memq, memv, etc.) */ 
 
-static void sloppy_mem_check SCM_P ((SCM obj, char * where, char * why));
-
-static void
-sloppy_mem_check (obj, where, why)
-     SCM obj;
-     char * where;
-     char * why;
-{
-  SCM_ASSERT ((scm_ilength (obj) >= 0), obj, where, why);
-}
-
-
 SCM_PROC (s_sloppy_memq, "sloppy-memq", 2, 0, 0, scm_sloppy_memq);
 SCM
 scm_sloppy_memq(x, lst)
@@ -483,8 +499,8 @@ scm_memq(x, lst)
      SCM lst;
 {
   SCM answer;
+  SCM_ASSERT (scm_ilength (lst) >= 0, lst, SCM_ARG2, s_memq);
   answer = scm_sloppy_memq (x, lst);
-  sloppy_mem_check (answer, (char *)SCM_ARG2, s_memq);
   return (answer == SCM_EOL) ? SCM_BOOL_F : answer;
 }
 
@@ -497,8 +513,8 @@ scm_memv(x, lst)
      SCM lst;
 {
   SCM answer;
+  SCM_ASSERT (scm_ilength (lst) >= 0, lst, SCM_ARG2, s_memv);
   answer = scm_sloppy_memv (x, lst);
-  sloppy_mem_check (answer, (char *)SCM_ARG2, s_memv);
   return (answer == SCM_EOL) ? SCM_BOOL_F : answer;
 }
 
@@ -510,8 +526,8 @@ scm_member(x, lst)
      SCM lst;
 {
   SCM answer;
+  SCM_ASSERT (scm_ilength (lst) >= 0, lst, SCM_ARG2, s_member);
   answer = scm_sloppy_member (x, lst);
-  sloppy_mem_check (answer, (char *)SCM_ARG2, s_member);
   return (answer == SCM_EOL) ? SCM_BOOL_F : answer;
 }
 
@@ -629,6 +645,84 @@ scm_delete (item, lst)
 }
 
 
+SCM_PROC(s_delq1_x, "delq1!", 2, 0, 0, scm_delq1_x);
+SCM
+scm_delq1_x (item, lst)
+     SCM item;
+     SCM lst;
+{
+  SCM walk;
+  SCM *prev;
+
+  for (prev = &lst, walk = lst;
+       SCM_NIMP (walk) && SCM_CONSP (walk);
+       walk = SCM_CDR (walk))
+    {
+      if (SCM_CAR (walk) == item)
+	{
+	  *prev = SCM_CDR (walk);
+	  break;
+	}
+      else
+	prev = SCM_CDRLOC (walk);
+    }
+    
+  return lst;
+}
+
+
+SCM_PROC(s_delv1_x, "delv1!", 2, 0, 0, scm_delv1_x);
+SCM
+scm_delv1_x (item, lst)
+     SCM item;
+     SCM lst;
+{
+  SCM walk;
+  SCM *prev;
+
+  for (prev = &lst, walk = lst;
+       SCM_NIMP (walk) && SCM_CONSP (walk);
+       walk = SCM_CDR (walk))
+    {
+      if (SCM_BOOL_F != scm_eqv_p (SCM_CAR (walk), item))
+	{
+	  *prev = SCM_CDR (walk);
+	  break;
+	}
+      else
+	prev = SCM_CDRLOC (walk);
+    }
+    
+  return lst;
+}
+
+
+SCM_PROC(s_delete1_x, "delete1!", 2, 0, 0, scm_delete1_x);
+SCM
+scm_delete1_x (item, lst)
+     SCM item;
+     SCM lst;
+{
+  SCM walk;
+  SCM *prev;
+
+  for (prev = &lst, walk = lst;
+       SCM_NIMP (walk) && SCM_CONSP (walk);
+       walk = SCM_CDR (walk))
+    {
+      if (SCM_BOOL_F != scm_equal_p (SCM_CAR (walk), item))
+	{
+	  *prev = SCM_CDR (walk);
+	  break;
+	}
+      else
+	prev = SCM_CDRLOC (walk);
+    }
+
+  return lst;
+}
+
+
 
 void
 scm_init_list ()
