@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995,1996,1998 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996,1998,1999 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -213,7 +213,9 @@ SCM scm_listofnullstr;
 /* Search PATH for a directory containing a file named FILENAME.
    The file must be readable, and not a directory.
    If we find one, return its full filename; otherwise, return #f.
-   If FILENAME is absolute, return it unchanged.  */
+   If FILENAME is absolute, return it unchanged.
+   If given, EXTENSIONS is a list of strings; for each directory 
+   in PATH, we search for FILENAME concatenated with each EXTENSION.  */
 SCM_PROC(s_search_path, "search-path", 2, 1, 0, scm_search_path);
 SCM 
 scm_search_path (path, filename, extensions)
@@ -221,24 +223,25 @@ scm_search_path (path, filename, extensions)
      SCM filename;
      SCM extensions;
 {
-  char *buf;
+  char *filename_chars;
   int filename_len;
-  size_t max_path_len;
-  size_t max_ext_len;
+  size_t max_path_len;		/* maximum length of any PATH element */
+  size_t max_ext_len;		/* maximum length of any EXTENSIONS element */
 
   SCM_ASSERT (scm_ilength (path) >= 0, path, SCM_ARG1, s_search_path);
   SCM_ASSERT (SCM_NIMP (filename) && SCM_ROSTRINGP (filename), filename,
 	      SCM_ARG2, s_search_path);
   if (SCM_UNBNDP (extensions))
-    extensions = scm_listofnullstr;
+    extensions = SCM_EOL;
   else
     SCM_ASSERT (scm_ilength (extensions) >= 0, extensions,
 		SCM_ARG3, s_search_path);
+
+  filename_chars = SCM_ROCHARS (filename);
   filename_len = SCM_ROLENGTH (filename);
 
   /* If FILENAME is absolute, return it unchanged.  */
-  if (filename_len >= 1
-      && SCM_ROCHARS (filename)[0] == '/')
+  if (filename_len >= 1 && filename_chars[0] == '/')
     return filename;
 
   /* Find the length of the longest element of path.  */
@@ -252,8 +255,30 @@ scm_search_path (path, filename, extensions)
 	SCM_ASSERT (SCM_NIMP (elt) && SCM_ROSTRINGP (elt), elt,
 		    "path is not a list of strings",
 		    s_search_path);
-	if (SCM_LENGTH (elt) > max_path_len)
-	  max_path_len = SCM_LENGTH (elt);
+	if (SCM_ROLENGTH (elt) > max_path_len)
+	  max_path_len = SCM_ROLENGTH (elt);
+      }
+  }
+
+  /* If FILENAME has an extension, don't try to add EXTENSIONS to it.  */
+  {
+    char *endp;
+
+    for (endp = filename_chars + filename_len - 1;
+	 endp >= filename_chars;
+	 endp--)
+      {
+	if (*endp == '.')
+	  {
+	    /* This filename already has an extension, so cancel the
+               list of extensions.  */
+	    extensions = SCM_EOL;
+	    break;
+	  }
+	else if (*endp == '/')
+	  /* This filename has no extension, so keep the current list
+             of extensions.  */
+	  break;
       }
   }
 
@@ -269,59 +294,67 @@ scm_search_path (path, filename, extensions)
 	SCM_ASSERT (SCM_NIMP (elt) && SCM_ROSTRINGP (elt), elt,
 		    "extension list is not a list of strings",
 		    s_search_path);
-	if (SCM_LENGTH (elt) > max_ext_len)
-	  max_ext_len = SCM_LENGTH (elt);
+	if (SCM_ROLENGTH (elt) > max_ext_len)
+	  max_ext_len = SCM_ROLENGTH (elt);
       }
   }
 
   SCM_DEFER_INTS;
 
-  buf = scm_must_malloc (max_path_len + 1 + filename_len + max_ext_len + 1,
-			 s_search_path);
+  {
+    SCM result = SCM_BOOL_F;
+    int buf_size = max_path_len + 1 + filename_len + max_ext_len + 1;
+    char *buf = scm_must_malloc (buf_size, s_search_path);
 
-  /* Try every path element.  At this point, we know it's a proper
-     list of strings.  */
-  for (; SCM_NIMP (path); path = SCM_CDR (path))
-    {
-      SCM path_elt = SCM_CAR (path), exts;
+    /* This simplifies the loop below a bit.  */
+    if (SCM_NULLP (extensions))
+      extensions = scm_listofnullstr;
 
-      /* Try every extension.  At this point, we know it's a proper
-         list of strings.  */
-      for (exts = extensions; SCM_NIMP (exts); exts = SCM_CDR (exts))
-	{
-	  SCM ext_elt = SCM_CAR (exts);
-	  int i;
+    /* Try every path element.  At this point, we know the path is a
+       proper list of strings.  */
+    for (; SCM_NIMP (path); path = SCM_CDR (path))
+      {
+	int len;
+	SCM dir = SCM_CAR (path);
+	SCM exts;
 
-	  /* Concatenate the path name, the filename, and the extension. */
-	  i = SCM_ROLENGTH (path_elt);
-	  memcpy (buf, SCM_ROCHARS (path_elt), i);
-	  if (i >= 1 && buf[i - 1] != '/')
-	    buf[i++] = '/';
-	  memcpy (buf + i, SCM_ROCHARS (filename), filename_len);
-	  i += filename_len;
-	  memcpy (buf + i, SCM_ROCHARS (ext_elt), SCM_LENGTH (ext_elt));
-	  i += SCM_LENGTH (ext_elt);
-	  buf[i] = '\0';
+	/* Concatenate the path name and the filename. */
+	len = SCM_ROLENGTH (dir);
+	memcpy (buf, SCM_ROCHARS (dir), len);
+	if (len >= 1 && buf[len - 1] != '/')
+	  buf[len++] = '/';
+	memcpy (buf + len, filename_chars, filename_len);
+	len += filename_len;
 
+	/* Try every extension.  At this point, we know the extension
+	   list is a proper, nonempty list of strings.  */
+	for (exts = extensions; SCM_NIMP (exts); exts = SCM_CDR (exts))
 	  {
+	    SCM ext = SCM_CAR (exts);
+	    int ext_len = SCM_ROLENGTH (ext);
 	    struct stat mode;
 
-	    if (stat (buf, &mode) >= 0
-		&& ! (mode.st_mode & S_IFDIR)
-		&& access (buf, R_OK) == 0)
+	    /* Concatenate the extension. */
+	    memcpy (buf + len, SCM_ROCHARS (ext), ext_len);
+	    buf[len + ext_len] = '\0';
+	    
+	    /* If the file exists at all, we should return it.  If the
+	       file is inaccessible, then that's an error.  */
+	    if (stat (buf, &mode) == 0
+		&& ! (mode.st_mode & S_IFDIR))
 	      {
-		SCM result = scm_makfromstr (buf, i, 0);
-		scm_must_free (buf);
-		SCM_ALLOW_INTS;
-		return result;
+		result = scm_makfromstr (buf, len + ext_len, 0);
+		goto end;
 	      }
 	  }
-	}
-    }
-  
-  scm_must_free (buf);
-  SCM_ALLOW_INTS;
-  return SCM_BOOL_F;
+      }
+
+  end:
+    scm_must_free (buf);
+    scm_done_malloc (- buf_size);
+    SCM_ALLOW_INTS;
+    return result;
+  }
 }
 
 
@@ -363,7 +396,7 @@ scm_primitive_load_path (filename)
 
   if (SCM_FALSEP (full_filename))
     {
-      int absolute = (SCM_LENGTH (filename) >= 1
+      int absolute = (SCM_ROLENGTH (filename) >= 1
 		      && SCM_ROCHARS (filename)[0] == '/');
       scm_misc_error (s_primitive_load_path,
 		      (absolute
@@ -421,8 +454,8 @@ scm_init_load ()
   scm_loc_load_path = SCM_CDRLOC (scm_sysintern ("%load-path", SCM_EOL));
   scm_loc_load_extensions
     = SCM_CDRLOC (scm_sysintern ("%load-extensions",
-				 scm_listify (scm_makfrom0str (""),
-					      scm_makfrom0str (".scm"),
+				 scm_listify (scm_makfrom0str (".scm"),
+					      scm_makfrom0str (""),
 					      SCM_UNDEFINED)));
   scm_loc_load_hook = SCM_CDRLOC (scm_sysintern ("%load-hook", SCM_BOOL_F));
 
