@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
+/*	Copyright (C) 1995, 1996, 1999 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,6 +52,8 @@
 #include "chars.h"
 #include "keywords.h"
 #include "smob.h"
+#include "eval.h"
+#include "alist.h"
 
 #include "objects.h"
 
@@ -62,7 +64,7 @@ SCM scm_metaclass_operator;
 /* These variables are filled in by the object system when loaded. */
 SCM scm_class_boolean, scm_class_char, scm_class_pair;
 SCM scm_class_procedure, scm_class_string, scm_class_symbol;
-SCM scm_class_procedure_with_setter;
+SCM scm_class_procedure_with_setter, scm_class_primitive_generic;
 SCM scm_class_vector, scm_class_null;
 SCM scm_class_integer, scm_class_real, scm_class_complex;
 SCM scm_class_unknown;
@@ -70,7 +72,7 @@ SCM scm_class_unknown;
 SCM *scm_port_class = 0;
 SCM *scm_smob_class = 0;
 
-SCM scm_apply_generic_env;
+SCM scm_no_applicable_method;
 
 SCM (*scm_make_extended_class) (char *type_name);
 void (*scm_make_port_classes) (int ptobnum, char *type_name);
@@ -137,17 +139,15 @@ scm_class_of (SCM x)
 	case scm_tc7_subr_2o:
 	case scm_tc7_lsubr_2:
 	case scm_tc7_lsubr:
+	  if (SCM_SUBR_GENERIC (x) && *SCM_SUBR_GENERIC (x))
+	    return scm_class_primitive_generic;
+	  else
+	    return scm_class_procedure;
 	case scm_tc7_cclo:
 	  return scm_class_procedure;
 	case scm_tc7_pws:
 	  return scm_class_procedure_with_setter;
 
-	case scm_tc7_port:
-	  return scm_port_class[(SCM_WRTNG & SCM_CAR (x)
-				 ? (SCM_RDNG & SCM_CAR (x)
-				    ? SCM_INOUT_PCLASS_INDEX | SCM_PTOBNUM (x)
-				    : SCM_OUT_PCLASS_INDEX | SCM_PTOBNUM (x))
-				 : SCM_IN_PCLASS_INDEX | SCM_PTOBNUM (x))];
 	case scm_tc7_smob:
 	  {
 	    SCM type = SCM_TYP16 (x);
@@ -158,9 +158,17 @@ scm_class_of (SCM x)
 		else
 		  return scm_class_real;
 	      }
-	    else
+	    else if (type != scm_tc16_port_with_ps)
 	      return scm_smob_class[SCM_TC2SMOBNUM (type)];
+	    x = SCM_PORT_WITH_PS_PORT (x);
+	    /* fall through to ports */
 	  }
+	case scm_tc7_port:
+	  return scm_port_class[(SCM_WRTNG & SCM_CAR (x)
+				 ? (SCM_RDNG & SCM_CAR (x)
+				    ? SCM_INOUT_PCLASS_INDEX | SCM_PTOBNUM (x)
+				    : SCM_OUT_PCLASS_INDEX | SCM_PTOBNUM (x))
+				 : SCM_IN_PCLASS_INDEX | SCM_PTOBNUM (x))];
 	case scm_tcs_cons_gloc:
 	  /* must be a struct */
 	  if (SCM_OBJ_CLASS_FLAGS (x) & SCM_CLASSF_GOOPS_VALID)
@@ -207,6 +215,143 @@ scm_class_of (SCM x)
   return scm_class_unknown;
 }
 
+/* (SCM_IM_DISPATCH ARGS N-SPECIALIZED
+ *   #((TYPE1 ... ENV FORMALS FORM ...) ...)
+ *   GF)
+ *
+ * (SCM_IM_HASH_DISPATCH ARGS N-SPECIALIZED HASHSET MASK
+ *   #((TYPE1 ... ENV FORMALS FORM ...) ...)
+ *   GF)
+ *
+ * ARGS is either a list of expressions, in which case they
+ * are interpreted as the arguments of an application, or
+ * a non-pair, which is interpreted as a single expression
+ * yielding all arguments.
+ *
+ * SCM_IM_DISPATCH expressions in generic functions always
+ * have ARGS = the symbol `args' or the iloc #@0-0.
+ *
+ * Need FORMALS in order to support varying arity.  This
+ * also avoids the need for renaming of bindings.
+ *
+ * We should probably not complicate this mechanism by
+ * introducing "optimizations" for getters and setters or
+ * primitive methods.  Getters and setter will normally be
+ * compiled into @slot-[ref|set!] or a procedure call.
+ * They rely on the dispatch performed before executing
+ * the code which contains them.
+ *
+ * We might want to use a more efficient representation of
+ * this form in the future, perhaps after we have introduced
+ * low-level support for syntax-case macros.
+ */
+
+SCM
+scm_mcache_lookup_cmethod (SCM cache, SCM args)
+{
+  int i, n, end, mask;
+  SCM ls, methods, z = SCM_CDDR (cache);
+  n = SCM_INUM (SCM_CAR (z)); /* maximum number of specializers */
+  methods = SCM_CADR (z);
+
+  if (SCM_NIMP (methods))
+    {
+      /* Prepare for linear search */
+      mask = -1;
+      i = 0;
+      end = SCM_LENGTH (methods);
+    }
+  else
+    {
+      /* Compute a hash value */
+      int hashset = SCM_INUM (methods);
+      int j = n;
+      mask = SCM_INUM (SCM_CAR (z = SCM_CDDR (z)));
+      methods = SCM_CADR (z);
+      i = 0;
+      ls = args;
+      if (SCM_NIMP (ls))
+	do
+	  {
+	    i += (SCM_STRUCT_DATA (scm_class_of (SCM_CAR (ls)))
+		  [scm_si_hashsets + hashset]);
+	    ls = SCM_CDR (ls);
+	  }
+	while (--j && SCM_NIMP (ls));
+      i &= mask;
+      end = i;
+    }
+
+  /* Search for match  */
+  do
+    {
+      int j = n;
+      z = SCM_VELTS (methods)[i];
+      ls = args; /* list of arguments */
+      if (SCM_NIMP (ls))
+	do
+	  {
+	    /* More arguments than specifiers => CLASS != ENV */
+	    if (scm_class_of (SCM_CAR (ls)) != SCM_CAR (z))
+	      goto next_method;
+	    ls = SCM_CDR (ls);
+	    z = SCM_CDR (z);
+	  }
+	while (--j && SCM_NIMP (ls));
+      /* Fewer arguments than specifiers => CAR != ENV */
+      if (!(SCM_IMP (SCM_CAR (z)) || SCM_CONSP (SCM_CAR (z))))
+	goto next_method;
+      return z;
+    next_method:
+      i = (i + 1) & mask;
+    } while (i != end);
+  return SCM_BOOL_F;
+}
+
+SCM
+scm_mcache_compute_cmethod (SCM cache, SCM args)
+{
+  SCM cmethod = scm_mcache_lookup_cmethod (cache, args);
+  if (SCM_IMP (cmethod))
+    /* No match - memoize */
+    return scm_memoize_method (cache, args);
+  return cmethod;
+}
+
+SCM
+scm_apply_generic (SCM gf, SCM args)
+{
+  SCM cmethod = scm_mcache_compute_cmethod (SCM_ENTITY_PROCEDURE (gf), args);
+  return scm_eval_body (SCM_CDR (SCM_CMETHOD_CODE (cmethod)),
+			SCM_EXTEND_ENV (SCM_CAR (SCM_CMETHOD_CODE (cmethod)),
+					args,
+					SCM_CMETHOD_ENV (cmethod)));
+}
+
+SCM
+scm_call_generic_0 (SCM gf)
+{
+  return scm_apply_generic (gf, SCM_EOL);
+}
+
+SCM
+scm_call_generic_1 (SCM gf, SCM a1)
+{
+  return scm_apply_generic (gf, SCM_LIST1 (a1));
+}
+
+SCM
+scm_call_generic_2 (SCM gf, SCM a1, SCM a2)
+{
+  return scm_apply_generic (gf, SCM_LIST2 (a1, a2));
+}
+
+SCM
+scm_call_generic_3 (SCM gf, SCM a1, SCM a2, SCM a3)
+{
+  return scm_apply_generic (gf, SCM_LIST3 (a1, a2, a3));
+}
+
 SCM_PROC (s_entity_p, "entity?", 1, 0, 0, scm_entity_p);
 
 SCM
@@ -230,15 +375,11 @@ scm_operator_p (SCM obj)
 	  : SCM_BOOL_F);
 }
 
-SCM_PROC (s_set_object_procedure_x, "set-object-procedure!", 1, 0, 1, scm_set_object_procedure_x);
-
-SCM_SYMBOL (scm_sym_atdispatch, "@dispatch");
+SCM_PROC (s_set_object_procedure_x, "set-object-procedure!", 2, 0, 0, scm_set_object_procedure_x);
 
 SCM
-scm_set_object_procedure_x (SCM obj, SCM procs)
+scm_set_object_procedure_x (SCM obj, SCM proc)
 {
-  SCM proc[4], *pp, p, setp, arity;
-  int i, a, r;
   SCM_ASSERT (SCM_NIMP (obj) && SCM_STRUCTP (obj)
 	      && ((SCM_CLASS_FLAGS (obj) & SCM_CLASSF_OPERATOR)
 		  || (SCM_I_ENTITYP (obj)
@@ -247,104 +388,28 @@ scm_set_object_procedure_x (SCM obj, SCM procs)
 	      obj,
 	      SCM_ARG1,
 	      s_set_object_procedure_x);
-  for (i = 0; i < 4; ++i)
-    proc[i] = SCM_BOOL_F;
-  i = 0;
-  while (SCM_NIMP (procs))
-    {
-      if (i == 4)
-	scm_wrong_num_args (scm_makfrom0str (s_set_object_procedure_x));
-      p = SCM_CAR (procs);
-      setp = 0;
-      SCM_ASSERT (SCM_NIMP (p), p, SCM_ARG2 + i, s_set_object_procedure_x);
-      if (SCM_CLOSUREP (p))
-	{
-	  arity = scm_procedure_property (p, scm_sym_arity);
-	  a = SCM_INUM (SCM_CAR (arity));
-	  /* Closures have zero optional args */
-	  r = SCM_NFALSEP (SCM_CADDR (arity));
-	  if (a == 1 || (a <= 1 && r))
-	    {
-	      if (SCM_NFALSEP (proc[0]))
-		goto ambiguous;
-	      proc[0] = setp = p;
-	    }
-	  if (a == 2 || (a <= 2 && r))
-	    {
-	      if (SCM_NFALSEP (proc[1]))
-		goto ambiguous;
-	      proc[1] = setp = p;
-	    }
-	  if (a == 3 || (a <= 3 && r))
-	    {
-	      if (SCM_NFALSEP (proc[2]))
-		goto ambiguous;
-	      proc[2] = setp = p;
-	    }
-	  if (a <= 4 && r)
-	    {
-	      if (SCM_NFALSEP (proc[3]))
-		goto ambiguous;
-	      proc[3] = setp = p;
-	    }
-	}
-      else if (SCM_TYP7 (p) == scm_tc7_subr_1)
-	{
-	  if (SCM_NFALSEP (proc[0]))
-	    goto ambiguous;
-	  proc[0] = setp = p;
-	}
-      else if (SCM_TYP7 (p) == scm_tc7_subr_2)
-	{
-	  if (SCM_NFALSEP (proc[1]))
-	    goto ambiguous;
-	  proc[1] = setp = p;
-	}
-      else if (SCM_TYP7 (p) == scm_tc7_subr_3)
-	{
-	  if (SCM_NFALSEP (proc[2]))
-	    goto ambiguous;
-	  proc[2] = setp = p;
-	}
-      else if (SCM_TYP7 (p) == scm_tc7_lsubr_2)
-	{
-	  if (SCM_NFALSEP (proc[3]))
-	    {
-	    ambiguous:
-	      SCM_ASSERT (0, p, "Ambiguous procedure arities",
-			  s_set_object_procedure_x);
-	    }
-	  proc[3] = setp = p;
-	}
-      SCM_ASSERT (setp, p, SCM_ARG2 + i, s_set_object_procedure_x);
-      ++i;
-      procs = SCM_CDR (procs);
-    }
-  pp = (SCM_I_ENTITYP (obj)
-	? &SCM_ENTITY_PROC_0 (obj)
-	: &SCM_OPERATOR_CLASS (obj)->proc0);
-  for (i = 0; i < 4; ++i)
-    *pp++ = proc[i];
+  SCM_ASSERT (SCM_NFALSEP (scm_procedure_p (proc)),
+	      proc, SCM_ARG2, s_set_object_procedure_x);
+  if (SCM_I_ENTITYP (obj))
+    SCM_ENTITY_PROCEDURE (obj) = proc;
+  else
+    SCM_OPERATOR_CLASS (obj)->procedure = proc;
   return SCM_UNSPECIFIED;
 }
 
 #ifdef GUILE_DEBUG
-SCM_PROC (s_object_procedures, "object-procedures", 1, 0, 0, scm_object_procedures);
+SCM_PROC (s_object_procedure, "object-procedure", 1, 0, 0, scm_object_procedure);
 
 SCM
-scm_object_procedures (SCM obj)
+scm_object_procedure (SCM obj)
 {
-  SCM_ASSERT (SCM_NIMP (obj) && SCM_STRUCTP (obj) && SCM_I_OPERATORP (obj),
-	      obj, SCM_ARG1, s_object_procedures);
+  SCM_ASSERT (SCM_NIMP (obj) && SCM_STRUCTP (obj)
+	      && ((SCM_CLASS_FLAGS (obj) & SCM_CLASSF_OPERATOR)
+		  || SCM_I_ENTITYP (obj)),
+	      obj, SCM_ARG1, s_object_procedure);
   return (SCM_I_ENTITYP (obj)
-	  ? SCM_LIST4 (SCM_ENTITY_PROC_0 (obj),
-		       SCM_ENTITY_PROC_1 (obj),
-		       SCM_ENTITY_PROC_2 (obj),
-		       SCM_ENTITY_PROC_3 (obj))
-	  : SCM_LIST4 (SCM_OPERATOR_PROC_0 (obj),
-		       SCM_OPERATOR_PROC_1 (obj),
-		       SCM_OPERATOR_PROC_2 (obj),
-		       SCM_OPERATOR_PROC_3 (obj)));
+	  ? SCM_ENTITY_PROCEDURE (obj)
+	  : SCM_OPERATOR_CLASS (obj)->procedure);
 }
 #endif /* GUILE_DEBUG */
 
