@@ -12,7 +12,8 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -36,8 +37,7 @@
  *
  * If you write modifications of your own for GUILE, it is your choice
  * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  
- */
+ * If you do not wish that, delete this exception notice.  */
 
 #include <stdio.h>
 #include "_scm.h"
@@ -80,44 +80,46 @@ scm_cons2 (w, x, y)
 }
 
 
-SCM_PROC(s_pair_p, "pair?", 1, 0, 0, scm_pair_p);
+SCM_PROC (s_pair_p, "pair?", 1, 0, 0, scm_pair_p);
 
 SCM
-scm_pair_p(x)
+scm_pair_p (x)
      SCM x;
 {
-	if SCM_IMP(x) return SCM_BOOL_F;
-	return SCM_CONSP(x) ? SCM_BOOL_T : SCM_BOOL_F;
+  if (SCM_IMP (x))
+    return SCM_BOOL_F;
+  return SCM_CONSP (x) ? SCM_BOOL_T : SCM_BOOL_F;
 }
 
-SCM_PROC(s_set_car_x, "set-car!", 2, 0, 0, scm_set_car_x);
+SCM_PROC (s_set_car_x, "set-car!", 2, 0, 0, scm_set_car_x);
 
 SCM
-scm_set_car_x(pair, value)
+scm_set_car_x (pair, value)
      SCM pair;
      SCM value;
 {
-	SCM_ASSERT(SCM_NIMP(pair) && SCM_CONSP(pair), pair, SCM_ARG1, s_set_car_x);
-	SCM_SETCAR (pair, value);
-	return value;
+  SCM_ASSERT (SCM_NIMP (pair) && SCM_CONSP (pair),
+	      pair, SCM_ARG1, s_set_car_x);
+  SCM_SETCAR (pair, value);
+  return SCM_UNSPECIFIED;
 }
 
-SCM_PROC(s_set_cdr_x, "set-cdr!", 2, 0, 0, scm_set_cdr_x);
+SCM_PROC (s_set_cdr_x, "set-cdr!", 2, 0, 0, scm_set_cdr_x);
 
 SCM
-scm_set_cdr_x(pair, value)
+scm_set_cdr_x (pair, value)
      SCM pair;
      SCM value;
 {
-	SCM_ASSERT(SCM_NIMP(pair) && SCM_CONSP(pair), pair, SCM_ARG1, s_set_cdr_x);
-	SCM_SETCDR (pair, value);
-	return value;
+  SCM_ASSERT (SCM_NIMP(pair) && SCM_CONSP (pair), pair, SCM_ARG1, s_set_cdr_x);
+  SCM_SETCDR (pair, value);
+  return SCM_UNSPECIFIED;
 }
 
 
 
 
-static scm_iproc cxrs[] = 
+static const scm_iproc cxrs[] = 
 {
   {"car", 0},
   {"cdr", 0},
@@ -157,7 +159,7 @@ static scm_iproc cxrs[] =
 void
 scm_init_pairs ()
 {
-  scm_init_iprocs(cxrs, scm_tc7_cxr);
+  scm_init_iprocs (cxrs, scm_tc7_cxr);
 #include "pairs.x"
 }
 
