@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995, 1996, 1997 Free Software Foundation, Inc.
+/*	Copyright (C) 1995, 1996, 1997, 1998 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -158,7 +158,8 @@ extern char ** environ;
    encourage header files to do strange things.  */
 
 
-
+SCM_SYMBOL (sym_read_pipe, "read pipe");
+SCM_SYMBOL (sym_write_pipe, "write pipe");
 
 SCM_PROC (s_pipe, "pipe", 0, 0, 0, scm_pipe);
 
@@ -166,48 +167,19 @@ SCM
 scm_pipe ()
 {
   int fd[2], rv;
-  FILE *f_rd, *f_wt;
   SCM p_rd, p_wt;
-  struct scm_port_table * ptr;
-  struct scm_port_table * ptw;
 
-  SCM_NEWCELL (p_rd);
-  SCM_NEWCELL (p_wt);
   rv = pipe (fd);
   if (rv)
     scm_syserror (s_pipe);
-  f_rd = fdopen (fd[0], "r");
-  if (!f_rd)
-    {
-      SCM_SYSCALL (close (fd[0]));
-      SCM_SYSCALL (close (fd[1]));
-      scm_syserror (s_pipe);
-    }
-  f_wt = fdopen (fd[1], "w");
-  if (!f_wt)
-    {
-      int en;
-      en = errno;
-      fclose (f_rd);
-      SCM_SYSCALL (close (fd[1]));
-      errno = en;
-      scm_syserror (s_pipe);
-    }
-  ptr = scm_add_to_port_table (p_rd);
-  ptw = scm_add_to_port_table (p_wt);
-  SCM_SETPTAB_ENTRY (p_rd, ptr);
-  SCM_SETPTAB_ENTRY (p_wt, ptw);
-  SCM_SETCAR (p_rd, scm_tc16_fport | scm_mode_bits ("r"));
-  SCM_SETCAR (p_wt, scm_tc16_fport | scm_mode_bits ("w"));
-  SCM_SETSTREAM (p_rd, (SCM)f_rd);
-  SCM_SETSTREAM (p_wt, (SCM)f_wt);
-
-  SCM_ALLOW_INTS;
+  
+  p_rd = scm_fdes_to_port (fd[0], "r", sym_read_pipe);
+  p_wt = scm_fdes_to_port (fd[1], "w", sym_write_pipe);
   return scm_cons (p_rd, p_wt);
 }
 
 
-
+#ifdef HAVE_GETGROUPS
 SCM_PROC (s_getgroups, "getgroups", 0, 0, 0, scm_getgroups);
 
 SCM
@@ -228,19 +200,21 @@ scm_getgroups()
     val = getgroups(ngroups, groups);
     if (val < 0)
       {
+	int en = errno;
 	scm_must_free((char *)groups);
+	errno = en;
 	scm_syserror (s_getgroups);
       }
     SCM_SETCHARS(grps, groups);	/* set up grps as a GC protect */
     SCM_SETLENGTH(grps, 0L + ngroups * sizeof(GETGROUPS_T), scm_tc7_string);
-    SCM_ALLOW_INTS;
-    ans = scm_make_vector(SCM_MAKINUM(ngroups), SCM_UNDEFINED, SCM_BOOL_F);
+    ans = scm_make_vector (SCM_MAKINUM(ngroups), SCM_UNDEFINED);
     while (--ngroups >= 0) SCM_VELTS(ans)[ngroups] = SCM_MAKINUM(groups[ngroups]);
     SCM_SETCHARS(grps, groups);	/* to make sure grps stays around. */
+    SCM_ALLOW_INTS;
     return ans;
   }
 }  
-
+#endif
 
 
 SCM_PROC (s_getpwuid, "getpw", 0, 1, 0, scm_getpwuid);
@@ -253,21 +227,18 @@ scm_getpwuid (user)
   struct passwd *entry;
   SCM *ve;
 
-  result = scm_make_vector (SCM_MAKINUM (7), SCM_UNSPECIFIED, SCM_BOOL_F);
+  result = scm_make_vector (SCM_MAKINUM (7), SCM_UNSPECIFIED);
   ve = SCM_VELTS (result);
   if (SCM_UNBNDP (user) || SCM_FALSEP (user))
     {
-      SCM_DEFER_INTS;
       SCM_SYSCALL (entry = getpwent ());
       if (! entry)
 	{
-	  SCM_ALLOW_INTS;
 	  return SCM_BOOL_F;
 	}
     }
   else if (SCM_INUMP (user))
     {
-      SCM_DEFER_INTS;
       entry = getpwuid (SCM_INUM (user));
     }
   else
@@ -275,11 +246,10 @@ scm_getpwuid (user)
       SCM_ASSERT (SCM_NIMP (user) && SCM_ROSTRINGP (user), user, SCM_ARG1, s_getpwuid);
       if (SCM_SUBSTRP (user))
 	user = scm_makfromstr (SCM_ROCHARS (user), SCM_ROLENGTH (user), 0);
-      SCM_DEFER_INTS;
       entry = getpwnam (SCM_ROCHARS (user));
     }
   if (!entry)
-    scm_syserror (s_getpwuid);
+    scm_misc_error (s_getpwuid, "entry not found", SCM_EOL);
 
   ve[0] = scm_makfrom0str (entry->pw_name);
   ve[1] = scm_makfrom0str (entry->pw_passwd);
@@ -294,12 +264,11 @@ scm_getpwuid (user)
     ve[6] = scm_makfrom0str ("");
   else
     ve[6] = scm_makfrom0str (entry->pw_shell);
-  SCM_ALLOW_INTS;
   return result;
 }
 
 
-
+#ifdef HAVE_SETPWENT
 SCM_PROC (s_setpwent, "setpw", 0, 1, 0, scm_setpwent);
 
 SCM 
@@ -312,6 +281,7 @@ scm_setpwent (arg)
     setpwent ();
   return SCM_UNSPECIFIED;
 }
+#endif
 
 
 
@@ -325,15 +295,13 @@ scm_getgrgid (name)
   SCM result;
   struct group *entry;
   SCM *ve;
-  result = scm_make_vector (SCM_MAKINUM (4), SCM_UNSPECIFIED, SCM_BOOL_F);
+  result = scm_make_vector (SCM_MAKINUM (4), SCM_UNSPECIFIED);
   ve = SCM_VELTS (result);
-  SCM_DEFER_INTS;
   if (SCM_UNBNDP (name) || (name == SCM_BOOL_F))
     {
       SCM_SYSCALL (entry = getgrent ());
       if (! entry)
 	{
-	  SCM_ALLOW_INTS;
 	  return SCM_BOOL_F;
 	}
     }
@@ -353,7 +321,6 @@ scm_getgrgid (name)
   ve[1] = scm_makfrom0str (entry->gr_passwd);
   ve[2] = scm_ulong2num ((unsigned long) entry->gr_gid);
   ve[3] = scm_makfromstrs (-1, entry->gr_mem);
-  SCM_ALLOW_INTS;
   return result;
 }
 
@@ -572,6 +539,7 @@ scm_seteuid (id)
   return SCM_UNSPECIFIED;
 }
 
+#ifdef HAVE_SETEGID
 SCM_PROC (s_setegid, "setegid", 1, 0, 0, scm_setegid);
 
 SCM 
@@ -591,6 +559,7 @@ scm_setegid (id)
   return SCM_UNSPECIFIED;
     
 }
+#endif
 
 SCM_PROC (s_getpgrp, "getpgrp", 0, 0, 0, scm_getpgrp);
 SCM 
@@ -644,12 +613,12 @@ scm_ttyname (port)
 {
   char *ans;
   int fd;
+
+  port = SCM_COERCE_OUTPORT (port);
   SCM_ASSERT (SCM_NIMP (port) && SCM_OPPORTP (port), port, SCM_ARG1, s_ttyname);
   if (scm_tc16_fport != SCM_TYP16 (port))
     return SCM_BOOL_F;
-  fd = fileno ((FILE *)SCM_STREAM (port));
-  if (fd == -1)
-    scm_syserror (s_ttyname);
+  fd = SCM_FPORT_FDES (port);
   SCM_SYSCALL (ans = ttyname (fd));
   if (!ans)
     scm_syserror (s_ttyname);
@@ -682,9 +651,12 @@ scm_tcgetpgrp (port)
 #ifdef HAVE_TCGETPGRP
   int fd;
   pid_t pgid;
+
+  port = SCM_COERCE_OUTPORT (port);
+
   SCM_ASSERT (SCM_NIMP (port) && SCM_OPFPORTP (port), port, SCM_ARG1, s_tcgetpgrp);
-  fd = fileno ((FILE *)SCM_STREAM (port));
-  if (fd == -1 || (pgid = tcgetpgrp (fd)) == -1)
+  fd = SCM_FPORT_FDES (port);
+  if ((pgid = tcgetpgrp (fd)) == -1)
     scm_syserror (s_tcgetpgrp);
   return SCM_MAKINUM (pgid);
 #else
@@ -701,10 +673,13 @@ scm_tcsetpgrp (port, pgid)
 {
 #ifdef HAVE_TCSETPGRP
   int fd;
+
+  port = SCM_COERCE_OUTPORT (port);
+
   SCM_ASSERT (SCM_NIMP (port) && SCM_OPFPORTP (port), port, SCM_ARG1, s_tcsetpgrp);
   SCM_ASSERT (SCM_INUMP (pgid), pgid, SCM_ARG2, s_tcsetpgrp);
-  fd = fileno ((FILE *)SCM_STREAM (port));
-  if (fd == -1 || tcsetpgrp (fd, SCM_INUM (pgid)) == -1)
+  fd = SCM_FPORT_FDES (port);
+  if (tcsetpgrp (fd, SCM_INUM (pgid)) == -1)
     scm_syserror (s_tcsetpgrp);
   return SCM_UNSPECIFIED;
 #else
@@ -716,78 +691,128 @@ scm_tcsetpgrp (port, pgid)
 
 /* Copy exec args from an SCM vector into a new C array.  */
 
-static char ** scm_convert_exec_args SCM_P ((SCM args));
-
 static char **
-scm_convert_exec_args (args)
-     SCM args;
+scm_convert_exec_args (SCM args, int pos, const char *subr)
 {
   char **execargv;
   int num_args;
   int i;
-  SCM_DEFER_INTS;
+
+  SCM_ASSERT (SCM_NULLP (args)
+	      || (SCM_NIMP (args) && SCM_CONSP (args)),
+	      args, pos, subr);
   num_args = scm_ilength (args);
   execargv = (char **) 
-    scm_must_malloc ((num_args + 1) * sizeof (char *), s_ttyname);
+    scm_must_malloc ((num_args + 1) * sizeof (char *), subr);
   for (i = 0; SCM_NNULLP (args); args = SCM_CDR (args), ++i)
     {
       scm_sizet len;
       char *dst;
       char *src;
       SCM_ASSERT (SCM_NIMP (SCM_CAR (args)) && SCM_ROSTRINGP (SCM_CAR (args)),
-		  SCM_CAR (args), "wrong type in SCM_ARG", "exec arg");
+		  SCM_CAR (args), SCM_ARGn, subr);
       len = 1 + SCM_ROLENGTH (SCM_CAR (args));
-      dst = (char *) scm_must_malloc ((long) len, s_ttyname);
+      dst = (char *) scm_must_malloc ((long) len, subr);
       src = SCM_ROCHARS (SCM_CAR (args));
       while (len--) 
 	dst[len] = src[len];
       execargv[i] = dst;
     }
   execargv[i] = 0;
-  SCM_ALLOW_INTS;
   return execargv;
 }
 
-SCM_PROC (s_execl, "execl", 0, 0, 1, scm_execl);
+SCM_PROC (s_execl, "execl", 1, 0, 1, scm_execl);
 
 SCM
-scm_execl (args)
-     SCM args;
+scm_execl (filename, args)
+     SCM filename, args;
 {
   char **execargv;
-  SCM filename = SCM_CAR (args);
-  SCM_ASSERT (SCM_NIMP (filename) && SCM_ROSTRINGP (filename), filename, SCM_ARG1, s_execl);
-  if (SCM_SUBSTRP (filename))
-    filename = scm_makfromstr (SCM_ROCHARS (filename), SCM_ROLENGTH (filename), 0);
-  args = SCM_CDR (args);
-  execargv = scm_convert_exec_args (args);
+  SCM_ASSERT (SCM_NIMP (filename) && SCM_ROSTRINGP (filename), filename,
+	      SCM_ARG1, s_execl);
+  SCM_COERCE_SUBSTR (filename);
+  execargv = scm_convert_exec_args (args, SCM_ARG2, s_execl);
   execv (SCM_ROCHARS (filename), execargv);
   scm_syserror (s_execl);
   /* not reached.  */
   return SCM_BOOL_F;
 }
 
-SCM_PROC (s_execlp, "execlp", 0, 0, 1, scm_execlp);
+SCM_PROC (s_execlp, "execlp", 1, 0, 1, scm_execlp);
 
 SCM
-scm_execlp (args)
-     SCM args;
+scm_execlp (filename, args)
+     SCM filename, args;
 {
   char **execargv;
-  SCM filename = SCM_CAR (args);
   SCM_ASSERT (SCM_NIMP (filename) && SCM_ROSTRINGP (filename), filename,
 	      SCM_ARG1, s_execlp);
-  if (SCM_SUBSTRP (filename))
-    filename = scm_makfromstr (SCM_ROCHARS (filename),
-			       SCM_ROLENGTH (filename), 0);
-  args = SCM_CDR (args);
-  execargv = scm_convert_exec_args (args);
+  SCM_COERCE_SUBSTR (filename);
+  execargv = scm_convert_exec_args (args, SCM_ARG2, s_execlp);
   execvp (SCM_ROCHARS (filename), execargv);
   scm_syserror (s_execlp);
   /* not reached.  */
   return SCM_BOOL_F;
 }
 
+static char **
+environ_list_to_c (SCM envlist, int arg, const char *proc)
+{
+  int num_strings;
+  char **result;
+  int i = 0;
+
+  SCM_ASSERT (SCM_NULLP (envlist)
+	      || (SCM_NIMP (envlist) && SCM_CONSP (envlist)),
+	      envlist, arg, proc);
+  num_strings = scm_ilength (envlist);
+  result = (char **) malloc ((num_strings + 1) * sizeof (char *));
+  if (result == NULL)
+    scm_memory_error (proc);
+  while (SCM_NNULLP (envlist))
+    {
+      int len;
+      char *src;
+
+      SCM_ASSERT (SCM_NIMP (SCM_CAR (envlist))
+		  && SCM_ROSTRINGP (SCM_CAR (envlist)),
+		  envlist, arg, proc);
+      len = 1 + SCM_ROLENGTH (SCM_CAR (envlist));
+      result[i] = malloc ((long) len);
+      if (result[i] == NULL)
+	scm_memory_error (proc);
+      src = SCM_ROCHARS (SCM_CAR (envlist));
+      while (len--) 
+	result[i][len] = src[len];
+      envlist = SCM_CDR (envlist);
+      i++;
+    }
+  result[i] = 0;
+  return result;
+}
+
+SCM_PROC (s_execle, "execle", 2, 0, 1, scm_execle);
+
+SCM
+scm_execle (filename, env, args)
+     SCM filename, env, args;
+{
+  char **execargv;
+  char **exec_env;
+
+  SCM_ASSERT (SCM_NIMP (filename) && SCM_ROSTRINGP (filename), filename,
+	      SCM_ARG1, s_execle);
+  SCM_COERCE_SUBSTR (filename);
+  
+  execargv = scm_convert_exec_args (args, SCM_ARG1, s_execle);
+  exec_env = environ_list_to_c (env, SCM_ARG2, s_execle);
+  execve (SCM_ROCHARS (filename), execargv, exec_env);
+  scm_syserror (s_execle);
+  /* not reached.  */
+  return SCM_BOOL_F;
+}
+
 SCM_PROC (s_fork, "primitive-fork", 0, 0, 0, scm_fork);
 
 SCM
@@ -808,9 +833,8 @@ scm_uname ()
 {
 #ifdef HAVE_UNAME
   struct utsname buf;
-  SCM ans = scm_make_vector(SCM_MAKINUM(5), SCM_UNSPECIFIED, SCM_BOOL_F);
+  SCM ans = scm_make_vector (SCM_MAKINUM(5), SCM_UNSPECIFIED);
   SCM *ve = SCM_VELTS (ans);
-  SCM_DEFER_INTS;
   if (uname (&buf) < 0)
     scm_syserror (s_uname);
   ve[0] = scm_makfrom0str (buf.sysname);
@@ -822,7 +846,6 @@ scm_uname ()
    a linux special?
   ve[5] = scm_makfrom0str (buf.domainname);
 */
-  SCM_ALLOW_INTS;
   return ans;
 #else
   scm_sysmissing (s_uname);
@@ -841,33 +864,9 @@ scm_environ (env)
     return scm_makfromstrs (-1, environ);
   else
     {
-      int num_strings;
       char **new_environ;
-      int i = 0;
-      SCM_ASSERT (SCM_NULLP (env) || (SCM_NIMP (env) && SCM_CONSP (env)),
-	      env, SCM_ARG1, s_environ);
-      num_strings = scm_ilength (env);
-      new_environ = (char **) malloc ((num_strings + 1) * sizeof (char *));
-      if (new_environ == NULL)
-	scm_memory_error (s_environ);
-      while (SCM_NNULLP (env))
-	{
-	  int len;
-	  char *src;
-	  SCM_ASSERT (SCM_NIMP (SCM_CAR (env))
-		      && SCM_ROSTRINGP (SCM_CAR (env)),
-		      env, SCM_ARG1, s_environ);
-	  len = 1 + SCM_ROLENGTH (SCM_CAR (env));
-	  new_environ[i] = malloc ((long) len);
-	  if (new_environ[i] == NULL)
-	    scm_memory_error (s_environ);
-	  src = SCM_ROCHARS (SCM_CAR (env));
-	  while (len--) 
-	    new_environ[i][len] = src[len];
-	  env = SCM_CDR (env);
-	  i++;
-	}
-      new_environ[i] = 0;
+
+      new_environ = environ_list_to_c (env, SCM_ARG1, s_environ);
       /* Free the old environment, except when called for the first
        * time.
        */
@@ -899,58 +898,6 @@ SCM scm_tmpnam()
 }
 #endif
 
-SCM_PROC (s_open_pipe, "open-pipe", 2, 0, 0, scm_open_pipe);
-
-SCM 
-scm_open_pipe (pipestr, modes)
-     SCM pipestr;
-     SCM modes;
-{
-  FILE *f;
-  register SCM z;
-  struct scm_port_table * pt;
-
-  SCM_ASSERT (SCM_NIMP (pipestr) && SCM_ROSTRINGP (pipestr), pipestr,
-	      SCM_ARG1, s_open_pipe);
-  if (SCM_SUBSTRP (pipestr))
-    pipestr = scm_makfromstr (SCM_ROCHARS (pipestr),
-			      SCM_ROLENGTH (pipestr), 0);
-  SCM_ASSERT (SCM_NIMP (modes) && SCM_ROSTRINGP (modes), modes, SCM_ARG2,
-	      s_open_pipe);
-  if (SCM_SUBSTRP (modes))
-    modes = scm_makfromstr (SCM_ROCHARS (modes), SCM_ROLENGTH (modes), 0);
-  SCM_NEWCELL (z);
-  SCM_DEFER_INTS;
-  SCM_SYSCALL (f = popen (SCM_ROCHARS (pipestr), SCM_ROCHARS (modes)));
-  if (!f)
-    scm_syserror (s_open_pipe);
-  pt = scm_add_to_port_table (z);
-  SCM_SETPTAB_ENTRY (z, pt);
-  SCM_SETCAR (z, scm_tc16_pipe | SCM_OPN 
-    | (strchr (SCM_ROCHARS (modes), 'r') ? SCM_RDNG : SCM_WRTNG));
-  SCM_SETSTREAM (z, (SCM)f);
-  SCM_ALLOW_INTS;
-  return z;
-}
-
-SCM_PROC (s_close_pipe, "close-pipe", 1, 0, 0, scm_close_pipe);
-
-SCM 
-scm_close_pipe (port)
-     SCM port;
-{
-  int rv;
-
-  SCM_ASSERT (SCM_NIMP (port) && SCM_TYP16(port) == scm_tc16_pipe 
-	      && SCM_OPENP (port), port, SCM_ARG1, s_close_pipe);
-  SCM_DEFER_INTS;
-  rv = pclose ((FILE *) SCM_STREAM (port));
-  if (rv == -1)
-    scm_syserror (s_close_pipe);
-  SCM_ALLOW_INTS;
-  return SCM_MAKINUM (rv);
-}
-
 SCM_PROC (s_utime, "utime", 1, 2, 0, scm_utime);
 
 SCM 
@@ -1077,7 +1024,7 @@ scm_mknod(path, type, perms, dev)
 #ifdef HAVE_MKNOD
   int val;
   char *p;
-  int ctype;
+  int ctype = 0;
 
   SCM_ASSERT (SCM_NIMP(path) && SCM_ROSTRINGP(path), path, SCM_ARG1, s_mknod);
   SCM_ASSERT (SCM_NIMP(type) && SCM_SYMBOLP (type), type, SCM_ARG2, s_mknod);
@@ -1103,12 +1050,10 @@ scm_mknod(path, type, perms, dev)
   else
     scm_out_of_range (s_mknod, type);
 
-  SCM_DEFER_INTS;
   SCM_SYSCALL (val = mknod(SCM_ROCHARS(path), ctype | SCM_INUM (perms),
 			   SCM_INUM (dev)));
   if (val != 0)
     scm_syserror (s_mknod);
-  SCM_ALLOW_INTS;
   return SCM_UNSPECIFIED;
 #else
   scm_sysmissing (s_mknod);
@@ -1148,12 +1093,9 @@ scm_sync()
   scm_sysmissing (s_sync);
   /* not reached.  */
 #endif
-  return SCM_BOOL_F;
+  return SCM_UNSPECIFIED;
 }
 
-
-
-
 void 
 scm_init_posix ()
 {
