@@ -2,7 +2,7 @@
 
 #ifndef POSIXH
 #define POSIXH
-/*	Copyright (C) 1995, 1996, 1997 Free Software Foundation, Inc.
+/*	Copyright (C) 1995, 1996, 1997, 1998 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -78,11 +78,13 @@ extern SCM scm_setgid SCM_P ((SCM id));
 extern SCM scm_seteuid SCM_P ((SCM id));
 extern SCM scm_setegid SCM_P ((SCM id));
 extern SCM scm_ttyname SCM_P ((SCM port));
-extern SCM scm_execl SCM_P ((SCM args));
-extern SCM scm_execlp SCM_P ((SCM args));
+extern SCM scm_execl SCM_P ((SCM filename, SCM args));
+extern SCM scm_execlp SCM_P ((SCM filename, SCM args));
+extern SCM scm_execle SCM_P ((SCM filename, SCM env, SCM args));
 extern SCM scm_fork SCM_P ((void));
 extern SCM scm_uname SCM_P ((void));
 extern SCM scm_environ SCM_P ((SCM env));
+extern SCM scm_tmpnam SCM_P ((void));
 extern SCM scm_open_pipe SCM_P ((SCM pipestr, SCM modes));
 extern SCM scm_close_pipe SCM_P ((SCM port));
 extern SCM scm_utime SCM_P ((SCM pathname, SCM actime, SCM modtime));
