@@ -1,4 +1,4 @@
-/*	Copyright (C) 1996 Free Software Foundation, Inc.
+/*	Copyright (C) 1996, 1998 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -152,18 +152,22 @@ scm_ra_matchp (ra0, ras)
   scm_sizet bas0 = 0;
   int i, ndim = 1;
   int exact = 2			/* 4 */ ;	/* Don't care about values >2 (yet?) */
-  if SCM_IMP
-    (ra0) return 0;
+  if (SCM_IMP (ra0)) return 0;
   switch (SCM_TYP7 (ra0))
       {
       default:
 	return 0;
       case scm_tc7_vector:
+      case scm_tc7_wvect:
       case scm_tc7_string:
       case scm_tc7_byvect:
       case scm_tc7_bvect:
       case scm_tc7_uvect:
       case scm_tc7_ivect:
+      case scm_tc7_svect:
+#ifdef LONGLONGS
+      case scm_tc7_llvect:
+#endif
       case scm_tc7_fvect:
       case scm_tc7_dvect:
       case scm_tc7_cvect:
@@ -183,19 +187,24 @@ scm_ra_matchp (ra0, ras)
     (ras)
       {
 	ra1 = SCM_CAR (ras);
-	if SCM_IMP
-	  (ra1) return 0;
+	if (SCM_IMP (ra1))
+          return 0;
 	switch SCM_TYP7
 	  (ra1)
 	    {
 	    default:
 	      return 0;
 	    case scm_tc7_vector:
+	    case scm_tc7_wvect:
 	    case scm_tc7_string:
 	    case scm_tc7_byvect:
 	    case scm_tc7_bvect:
 	    case scm_tc7_uvect:
 	    case scm_tc7_ivect:
+	    case scm_tc7_svect:
+#ifdef LONGLONGS
+	    case scm_tc7_llvect:
+#endif
 	    case scm_tc7_fvect:
 	    case scm_tc7_dvect:
 	    case scm_tc7_cvect:
@@ -254,7 +263,7 @@ scm_ramapc (cproc, data, ra0, lra, what)
      SCM data;
      SCM ra0;
      SCM lra;
-     char *what;
+     const char *what;
 {
   SCM inds, z;
   SCM vra0, ra1, vra1;
@@ -273,8 +282,7 @@ scm_ramapc (cproc, data, ra0, lra, what)
       if (kmax < 0)
 	goto gencase;
       vra0 = scm_array_contents (ra0, SCM_UNDEFINED);
-      if SCM_IMP
-	(vra0) goto gencase;
+      if (SCM_IMP (vra0)) goto gencase;
       if (!SCM_ARRAYP (vra0))
 	{
 	  vra1 = scm_make_ra (1);
@@ -314,8 +322,7 @@ scm_ramapc (cproc, data, ra0, lra, what)
     case 1:
     gencase:			/* Have to loop over all dimensions. */
       vra0 = scm_make_ra (1);
-      if SCM_ARRAYP
-	(ra0)
+      if (SCM_ARRAYP (ra0))
 	  {
 	    kmax = SCM_ARRAY_NDIM (ra0) - 1;
 	    if (kmax < 0)
@@ -351,8 +358,7 @@ scm_ramapc (cproc, data, ra0, lra, what)
 	  vra1 = scm_make_ra (1);
 	  SCM_ARRAY_DIMS (vra1)->lbnd = SCM_ARRAY_DIMS (vra0)->lbnd;
 	  SCM_ARRAY_DIMS (vra1)->ubnd = SCM_ARRAY_DIMS (vra0)->ubnd;
-	  if SCM_ARRAYP
-	    (ra1)
+	  if (SCM_ARRAYP (ra1))
 	      {
 		if (kmax >= 0)
 		  SCM_ARRAY_DIMS (vra1)->inc = SCM_ARRAY_DIMS (ra1)[kmax].inc;
@@ -429,6 +435,7 @@ scm_array_fill_int (ra, fill, ignore)
 	  scm_array_set_x (ra, fill, SCM_MAKINUM (i));
 	break;
       case scm_tc7_vector:
+      case scm_tc7_wvect:
 	for (i = base; n--; i += inc)
 	  SCM_VELTS (ra)[i] = fill;
 	break;
@@ -496,6 +503,24 @@ scm_array_fill_int (ra, fill, ignore)
 	    ve[i] = f;
 	  break;
 	}
+      case scm_tc7_svect:
+	SCM_ASRTGO (SCM_INUMP (fill), badarg2);
+	{
+	  short f = SCM_INUM (fill), *ve = (short *) SCM_VELTS (ra);
+	  for (i = base; n--; i += inc)
+	    ve[i] = f;
+	  break;
+	}
+#ifdef LONGLONGS
+      case scm_tc7_llvect:
+	SCM_ASRTGO (SCM_INUMP (fill), badarg2);
+	{
+	  long long f = SCM_INUM (fill), *ve = (long long *) SCM_VELTS (ra);
+	  for (i = base; n--; i += inc)
+	    ve[i] = f;
+	  break;
+	}
+#endif
 #ifdef SCM_FLOATS
 #ifdef SCM_SINGLES
       case scm_tc7_fvect:
@@ -558,7 +583,10 @@ racp (src, dst)
     (dst)
       {
       default:
-      gencase: case scm_tc7_vector:
+      gencase:
+      case scm_tc7_vector:
+      case scm_tc7_wvect:
+
 	for (; n-- > 0; i_s += inc_s, i_d += inc_d)
 	  scm_array_set_x (dst, scm_cvref (src, i_s, SCM_UNDEFINED), SCM_MAKINUM (i_d));
 	break;
@@ -735,7 +763,9 @@ racp (src, dst)
 }
 
 
+/* This name is obsolete.  Will go away in release 1.5.  */
 SCM_PROC(s_serial_array_copy_x, "serial-array-copy!", 2, 0, 0, scm_array_copy_x);
+SCM_PROC(s_array_copy_in_order_x, "array-copy-in-order!", 2, 0, 0, scm_array_copy_x);
 SCM_PROC(s_array_copy_x, "array-copy!", 2, 0, 0, scm_array_copy_x);
 
 SCM
@@ -770,18 +800,15 @@ scm_ra_eqp (ra0, ras)
       {
 	SCM e1 = SCM_UNDEFINED, e2 = SCM_UNDEFINED;
 	for (; n-- > 0; i0 += inc0, i1 += inc1, i2 += inc2)
-	  if BVE_REF
-	    (ra0, i0)
-	      if SCM_FALSEP
-		(scm_eq_p (RVREF (ra1, i1, e1), RVREF (ra2, i2, e2)))
+            if (BVE_REF (ra0, i0))
+	      if (SCM_FALSEP(scm_eq_p (RVREF (ra1, i1, e1), RVREF (ra2, i2, e2))))
 		  BVE_CLR (ra0, i0);
 	break;
       }
     case scm_tc7_uvect:
     case scm_tc7_ivect:
       for (; n-- > 0; i0 += inc0, i1 += inc1, i2 += inc2)
-	if BVE_REF
-	  (ra0, i0)
+          if (BVE_REF (ra0, i0))
 	    if (SCM_VELTS (ra1)[i1] != SCM_VELTS (ra2)[i2])
 	      BVE_CLR (ra0, i0);
       break;
@@ -789,23 +816,20 @@ scm_ra_eqp (ra0, ras)
 #ifdef SCM_SINGLES
     case scm_tc7_fvect:
       for (; n-- > 0; i0 += inc0, i1 += inc1, i2 += inc2)
-	if BVE_REF
-	  (ra0, i0)
+          if (BVE_REF (ra0, i0))
 	    if (((float *) SCM_VELTS (ra1))[i1] != ((float *) SCM_VELTS (ra2))[i2])
 	      BVE_CLR (ra0, i0);
       break;
 #endif /*SCM_SINGLES*/
     case scm_tc7_dvect:
       for (; n-- > 0; i0 += inc0, i1 += inc1, i2 += inc2)
-	if BVE_REF
-	  (ra0, i0)
+          if (BVE_REF (ra0, i0))
 	    if (((double *) SCM_VELTS (ra1))[i1] != ((double *) SCM_VELTS (ra2))[i2])
 	      BVE_CLR (ra0, i0);
       break;
     case scm_tc7_cvect:
       for (; n-- > 0; i0 += inc0, i1 += inc1, i2 += inc2)
-	if BVE_REF
-	  (ra0, i0)
+          if (BVE_REF (ra0, i0))
 	    if (((double *) SCM_VELTS (ra1))[2 * i1] != ((double *) SCM_VELTS (ra2))[2 * i2] ||
 		((double *) SCM_VELTS (ra1))[2 * i1 + 1] != ((double *) SCM_VELTS (ra2))[2 * i2 + 1])
 	      BVE_CLR (ra0, i0);
@@ -840,8 +864,7 @@ ra_compare (ra0, ra1, ra2, opt)
       {
 	SCM e1 = SCM_UNDEFINED, e2 = SCM_UNDEFINED;
 	for (; n-- > 0; i0 += inc0, i1 += inc1, i2 += inc2)
-	  if BVE_REF
-	    (ra0, i0)
+            if (BVE_REF (ra0, i0))
 	      if (opt ?
 		  SCM_NFALSEP (scm_less_p (RVREF (ra1, i1, e1), RVREF (ra2, i2, e2))) :
 		  SCM_FALSEP (scm_less_p (RVREF (ra1, i1, e1), RVREF (ra2, i2, e2))))
@@ -852,8 +875,7 @@ ra_compare (ra0, ra1, ra2, opt)
     case scm_tc7_ivect:
       for (; n-- > 0; i0 += inc0, i1 += inc1, i2 += inc2)
 	{
-	  if BVE_REF
-	    (ra0, i0)
+            if (BVE_REF (ra0, i0))
 	      if (opt ?
 		  SCM_VELTS (ra1)[i1] < SCM_VELTS (ra2)[i2] :
 		  SCM_VELTS (ra1)[i1] >= SCM_VELTS (ra2)[i2])
@@ -864,8 +886,7 @@ ra_compare (ra0, ra1, ra2, opt)
 #ifdef SCM_SINGLES
     case scm_tc7_fvect:
       for (; n-- > 0; i0 += inc0, i1 += inc1, i2 += inc2)
-	if BVE_REF
-	  (ra0, i0)
+          if (BVE_REF(ra0, i0))
 	    if (opt ?
 		((float *) SCM_VELTS (ra1))[i1] < ((float *) SCM_VELTS (ra2))[i2] :
 		((float *) SCM_VELTS (ra1))[i1] >= ((float *) SCM_VELTS (ra2))[i2])
@@ -874,8 +895,7 @@ ra_compare (ra0, ra1, ra2, opt)
 #endif /*SCM_SINGLES*/
     case scm_tc7_dvect:
       for (; n-- > 0; i0 += inc0, i1 += inc1, i2 += inc2)
-	if BVE_REF
-	  (ra0, i0)
+          if (BVE_REF (ra0, i0))
 	    if (opt ?
 		((double *) SCM_VELTS (ra1))[i1] < ((double *) SCM_VELTS (ra2))[i2] :
 		((double *) SCM_VELTS (ra1))[i1] >= ((double *) SCM_VELTS (ra2))[i2])
@@ -934,8 +954,7 @@ scm_ra_sum (ra0, ras)
   scm_sizet i0 = SCM_ARRAY_BASE (ra0);
   long inc0 = SCM_ARRAY_DIMS (ra0)->inc;
   ra0 = SCM_ARRAY_V (ra0);
-  if SCM_NNULLP
-    (ras)
+  if (SCM_NNULLP(ras))
       {
 	SCM ra1 = SCM_CAR (ras);
 	scm_sizet i1 = SCM_ARRAY_BASE (ra1);
@@ -1012,11 +1031,9 @@ scm_ra_difference (ra0, ras)
   scm_sizet i0 = SCM_ARRAY_BASE (ra0);
   long inc0 = SCM_ARRAY_DIMS (ra0)->inc;
   ra0 = SCM_ARRAY_V (ra0);
-  if SCM_NULLP
-    (ras)
+  if (SCM_NULLP (ras))
       {
-	switch SCM_TYP7
-	  (ra0)
+	switch (SCM_TYP7 (ra0))
 	    {
 	    default:
 	      {
@@ -1121,8 +1138,7 @@ scm_ra_product (ra0, ras)
   scm_sizet i0 = SCM_ARRAY_BASE (ra0);
   long inc0 = SCM_ARRAY_DIMS (ra0)->inc;
   ra0 = SCM_ARRAY_V (ra0);
-  if SCM_NNULLP
-    (ras)
+  if (SCM_NNULLP (ras))
       {
 	SCM ra1 = SCM_CAR (ras);
 	scm_sizet i1 = SCM_ARRAY_BASE (ra1);
@@ -1200,11 +1216,9 @@ scm_ra_divide (ra0, ras)
   scm_sizet i0 = SCM_ARRAY_BASE (ra0);
   long inc0 = SCM_ARRAY_DIMS (ra0)->inc;
   ra0 = SCM_ARRAY_V (ra0);
-  if SCM_NULLP
-    (ras)
+  if (SCM_NULLP (ras))
       {
-	switch SCM_TYP7
-	  (ra0)
+	switch (SCM_TYP7 (ra0))
 	    {
 	    default:
 	      {
@@ -1327,8 +1341,7 @@ ramap (ra0, proc, ras)
   long n = SCM_ARRAY_DIMS (ra0)->ubnd;
   long base = SCM_ARRAY_BASE (ra0) - i * inc;
   ra0 = SCM_ARRAY_V (ra0);
-  if SCM_NULLP
-    (ras)
+  if (SCM_NULLP (ras))
       for (; i <= n; i++)
 	scm_array_set_x (ra0, scm_apply (proc, SCM_EOL, SCM_EOL), SCM_MAKINUM (i * inc + base));
   else
@@ -1339,8 +1352,7 @@ ramap (ra0, proc, ras)
       long inc1 = SCM_ARRAY_DIMS (ra1)->inc;
       ra1 = SCM_ARRAY_V (ra1);
       ras = SCM_CDR (ras);
-      if SCM_NULLP
-	(ras)
+      if (SCM_NULLP(ras))
 	  ras = scm_nullvect;
       else
 	{
@@ -1375,8 +1387,7 @@ ramap_cxr (ra0, proc, ras)
   long n = SCM_ARRAY_DIMS (ra0)->ubnd - SCM_ARRAY_DIMS (ra1)->lbnd + 1;
   ra0 = SCM_ARRAY_V (ra0);
   ra1 = SCM_ARRAY_V (ra1);
-  switch SCM_TYP7
-    (ra0)
+  switch (SCM_TYP7 (ra0))
       {
       default:
       gencase:
@@ -1388,8 +1399,7 @@ ramap_cxr (ra0, proc, ras)
       case scm_tc7_fvect:
 	{
 	  float *dst = (float *) SCM_VELTS (ra0);
-	  switch SCM_TYP7
-	    (ra1)
+	  switch (SCM_TYP7 (ra1))
 	      {
 	      default:
 		goto gencase;
@@ -1409,8 +1419,7 @@ ramap_cxr (ra0, proc, ras)
       case scm_tc7_dvect:
 	{
 	  double *dst = (double *) SCM_VELTS (ra0);
-	  switch SCM_TYP7
-	    (ra1)
+	  switch (SCM_TYP7 (ra1))
 	      {
 	      default:
 		goto gencase;
@@ -1455,21 +1464,17 @@ ramap_rp (ra0, proc, ras)
     {
     default:
       for (; n-- > 0; i0 += inc0, i1 += inc1, i2 += inc2)
-	if BVE_REF
-	  (ra0, i0)
-	    if SCM_FALSEP
-	      (SCM_SUBRF (proc) (RVREF (ra1, i1, e1), RVREF (ra2, i2, e2)))
+          if (BVE_REF  (ra0, i0))
+            if (SCM_FALSEP (SCM_SUBRF (proc) (RVREF (ra1, i1, e1), RVREF (ra2, i2, e2))))
 		BVE_CLR (ra0, i0);
       break;
     case scm_tc7_uvect:
     case scm_tc7_ivect:
       for (; n-- > 0; i0 += inc0, i1 += inc1, i2 += inc2)
-	if BVE_REF
-	  (ra0, i0)
+          if (BVE_REF (ra0, i0))
 	    {
-	      if SCM_FALSEP
-		(SCM_SUBRF (proc) (SCM_MAKINUM (SCM_VELTS (ra1)[i1]),
-			       SCM_MAKINUM (SCM_VELTS (ra2)[i2])))
+	      if (SCM_FALSEP (SCM_SUBRF (proc) (SCM_MAKINUM (SCM_VELTS (ra1)[i1]),
+                                                SCM_MAKINUM (SCM_VELTS (ra2)[i2]))))
 		  BVE_CLR (ra0, i0);
 	    }
       break;
@@ -1479,13 +1484,11 @@ ramap_rp (ra0, proc, ras)
       {
 	SCM a1 = scm_makflo (1.0), a2 = scm_makflo (1.0);
 	for (; n-- > 0; i0 += inc0, i1 += inc1, i2 += inc2)
-	  if BVE_REF
-	    (ra0, i0)
+            if (BVE_REF (ra0, i0))
 	      {
 		SCM_FLO (a1) = ((float *) SCM_VELTS (ra1))[i1];
 		SCM_FLO (a2) = ((float *) SCM_VELTS (ra2))[i2];
-		if SCM_FALSEP
-		  (SCM_SUBRF (proc) (a1, a2))
+		if (SCM_FALSEP (SCM_SUBRF (proc) (a1, a2)))
 		    BVE_CLR (ra0, i0);
 	      }
 	break;
@@ -1495,13 +1498,11 @@ ramap_rp (ra0, proc, ras)
       {
 	SCM a1 = scm_makdbl (1.0 / 3.0, 0.0), a2 = scm_makdbl (1.0 / 3.0, 0.0);
 	for (; n-- > 0; i0 += inc0, i1 += inc1, i2 += inc2)
-	  if BVE_REF
-	    (ra0, i0)
+            if (BVE_REF (ra0, i0))
 	      {
 		SCM_REAL (a1) = ((double *) SCM_VELTS (ra1))[i1];
 		SCM_REAL (a2) = ((double *) SCM_VELTS (ra2))[i2];
-		if SCM_FALSEP
-		  (SCM_SUBRF (proc) (a1, a2))
+		if (SCM_FALSEP (SCM_SUBRF (proc) (a1, a2)))
 		    BVE_CLR (ra0, i0);
 	      }
 	break;
@@ -1510,15 +1511,13 @@ ramap_rp (ra0, proc, ras)
       {
 	SCM a1 = scm_makdbl (1.0, 1.0), a2 = scm_makdbl (1.0, 1.0);
 	for (; n-- > 0; i0 += inc0, i1 += inc1, i2 += inc2)
-	  if BVE_REF
-	    (ra0, i0)
+            if (BVE_REF (ra0, i0))
 	      {
 		SCM_REAL (a1) = ((double *) SCM_VELTS (ra1))[2 * i1];
 		SCM_IMAG (a1) = ((double *) SCM_VELTS (ra1))[2 * i1 + 1];
 		SCM_REAL (a2) = ((double *) SCM_VELTS (ra2))[2 * i2];
 		SCM_IMAG (a2) = ((double *) SCM_VELTS (ra2))[2 * i2 + 1];
-		if SCM_FALSEP
-		  (SCM_SUBRF (proc) (a1, a2))
+		if (SCM_FALSEP (SCM_SUBRF (proc) (a1, a2)))
 		    BVE_CLR (ra0, i0);
 	      }
 	break;
@@ -1545,7 +1544,7 @@ ramap_1 (ra0, proc, ras)
   long inc0 = SCM_ARRAY_DIMS (ra0)->inc, inc1 = SCM_ARRAY_DIMS (ra1)->inc;
   ra0 = SCM_ARRAY_V (ra0);
   ra1 = SCM_ARRAY_V (ra1);
-  if (scm_tc7_vector == SCM_TYP7 (ra0))
+  if (scm_tc7_vector == SCM_TYP7 (ra0) || scm_tc7_wvect == SCM_TYP7 (ra0))
     for (; n-- > 0; i0 += inc0, i1 += inc1)
       scm_array_set_x (ra0, SCM_SUBRF (proc) (scm_cvref (ra1, i1, SCM_UNDEFINED)), SCM_MAKINUM (i0));
   else
@@ -1572,10 +1571,11 @@ ramap_2o (ra0, proc, ras)
   ra0 = SCM_ARRAY_V (ra0);
   ra1 = SCM_ARRAY_V (ra1);
   ras = SCM_CDR (ras);
-  if SCM_NULLP
-    (ras)
+  if (SCM_NULLP (ras))
       {
-	if (scm_tc7_vector == SCM_TYP7 (ra0))
+	if (scm_tc7_vector == SCM_TYP7 (ra0)
+	    || scm_tc7_wvect == SCM_TYP7 (ra0))
+
 	  for (; n-- > 0; i0 += inc0, i1 += inc1)
 	    scm_array_set_x (ra0, SCM_SUBRF (proc) (scm_cvref (ra1, i1, SCM_UNDEFINED), SCM_UNDEFINED),
 		  SCM_MAKINUM (i0));
@@ -1591,7 +1591,7 @@ ramap_2o (ra0, proc, ras)
       scm_sizet i2 = SCM_ARRAY_BASE (ra2);
       long inc2 = SCM_ARRAY_DIMS (ra2)->inc;
       ra2 = SCM_ARRAY_V (ra2);
-      if (scm_tc7_vector == SCM_TYP7 (ra0))
+      if (scm_tc7_vector == SCM_TYP7 (ra0) || scm_tc7_wvect == SCM_TYP7 (ra0))
 	for (; n-- > 0; i0 += inc0, i1 += inc1, i2 += inc2)
 	  scm_array_set_x (ra0,
 		SCM_SUBRF (proc) (scm_cvref (ra1, i1, SCM_UNDEFINED), scm_cvref (ra2, i2, SCM_UNDEFINED)),
@@ -1620,8 +1620,7 @@ ramap_a (ra0, proc, ras)
   scm_sizet i0 = SCM_ARRAY_BASE (ra0);
   long inc0 = SCM_ARRAY_DIMS (ra0)->inc;
   ra0 = SCM_ARRAY_V (ra0);
-  if SCM_NULLP
-    (ras)
+  if (SCM_NULLP (ras))
       for (; n-- > 0; i0 += inc0)
 	scm_array_set_x (ra0, SCM_SUBRF (proc) (RVREF (ra0, i0, e0), SCM_UNDEFINED), SCM_MAKINUM (i0));
   else
@@ -1637,33 +1636,35 @@ ramap_a (ra0, proc, ras)
   return 1;
 }
 
-SCM_PROC(s_serial_array_map, "serial-array-map", 2, 0, 1, scm_array_map);
-SCM_PROC(s_array_map, "array-map", 2, 0, 1, scm_array_map);
+/* This name is obsolete.  Will go away in release 1.5.  */
+SCM_PROC(s_serial_array_map_x, "serial-array-map!", 2, 0, 1, scm_array_map_x);
+SCM_PROC(s_array_map_in_order_x, "array-map-in-order!", 2, 0, 1, scm_array_map_x);
+SCM_PROC(s_array_map_x, "array-map!", 2, 0, 1, scm_array_map_x);
 
 SCM
-scm_array_map (ra0, proc, lra)
+scm_array_map_x (ra0, proc, lra)
      SCM ra0;
      SCM proc;
      SCM lra;
 {
-  SCM_ASSERT (SCM_BOOL_T == scm_procedure_p (proc), proc, SCM_ARG2, s_array_map);
+  SCM_ASSERT (SCM_BOOL_T == scm_procedure_p (proc), proc, SCM_ARG2, s_array_map_x);
   switch (SCM_TYP7 (proc))
       {
       default:
       gencase:
-	scm_ramapc (ramap, proc, ra0, lra, s_array_map);
+	scm_ramapc (ramap, proc, ra0, lra, s_array_map_x);
 	return SCM_UNSPECIFIED;
       case scm_tc7_subr_1:
-	scm_ramapc (ramap_1, proc, ra0, lra, s_array_map);
+	scm_ramapc (ramap_1, proc, ra0, lra, s_array_map_x);
 	return SCM_UNSPECIFIED;
       case scm_tc7_subr_2:
       case scm_tc7_subr_2o:
-	scm_ramapc (ramap_2o, proc, ra0, lra, s_array_map);
+	scm_ramapc (ramap_2o, proc, ra0, lra, s_array_map_x);
 	return SCM_UNSPECIFIED;
       case scm_tc7_cxr:
 	if (!SCM_SUBRF (proc))
 	  goto gencase;
-	scm_ramapc (ramap_cxr, proc, ra0, lra, s_array_map);
+	scm_ramapc (ramap_cxr, proc, ra0, lra, s_array_map_x);
 	return SCM_UNSPECIFIED;
       case scm_tc7_rpsubr:
 	{
@@ -1676,25 +1677,23 @@ scm_array_map (ra0, proc, lra)
 	      {
 		while (SCM_NNULLP (lra) && SCM_NNULLP (SCM_CDR (lra)))
 		  {
-		    scm_ramapc (p->vproc, SCM_UNDEFINED, ra0, lra, s_array_map);
+		    scm_ramapc (p->vproc, SCM_UNDEFINED, ra0, lra, s_array_map_x);
 		    lra = SCM_CDR (lra);
 		  }
 		return SCM_UNSPECIFIED;
 	      }
 	  while (SCM_NNULLP (lra) && SCM_NNULLP (SCM_CDR (lra)))
 	    {
-	      scm_ramapc (ramap_rp, proc, ra0, lra, s_array_map);
+	      scm_ramapc (ramap_rp, proc, ra0, lra, s_array_map_x);
 	      lra = SCM_CDR (lra);
 	    }
 	  return SCM_UNSPECIFIED;
 	}
       case scm_tc7_asubr:
-	if SCM_NULLP
-	  (lra)
+          if (SCM_NULLP (lra))
 	    {
 	      SCM prot, fill = SCM_SUBRF (proc) (SCM_UNDEFINED, SCM_UNDEFINED);
-	      if SCM_INUMP
-		(fill)
+	      if (SCM_INUMP(fill))
 		  {
 		    prot = scm_array_prototype (ra0);
 		    if (SCM_NIMP (prot) && SCM_INEXP (prot))
@@ -1724,22 +1723,21 @@ scm_array_map (ra0, proc, lra)
 	      if (proc == p->sproc)
 		{
 		  if (ra0 != SCM_CAR (lra))
-		    scm_ramapc (scm_array_identity, SCM_UNDEFINED, ra0, scm_cons (SCM_CAR (lra), SCM_EOL), s_array_map);
+		    scm_ramapc (scm_array_identity, SCM_UNDEFINED, ra0, scm_cons (SCM_CAR (lra), SCM_EOL), s_array_map_x);
 		  lra = SCM_CDR (lra);
 		  while (1)
 		    {
-		      scm_ramapc (p->vproc, SCM_UNDEFINED, ra0, lra, s_array_map);
+		      scm_ramapc (p->vproc, SCM_UNDEFINED, ra0, lra, s_array_map_x);
 		      if (SCM_IMP (lra) || SCM_IMP (SCM_CDR (lra)))
 			return SCM_UNSPECIFIED;
 		      lra = SCM_CDR (lra);
 		    }
 		}
-	    scm_ramapc (ramap_2o, proc, ra0, lra, s_array_map);
+	    scm_ramapc (ramap_2o, proc, ra0, lra, s_array_map_x);
 	    lra = SCM_CDR (lra);
-	    if SCM_NIMP
-	      (lra)
+              if (SCM_NIMP (lra))
 		for (lra = SCM_CDR (lra); SCM_NIMP (lra); lra = SCM_CDR (lra))
-		  scm_ramapc (ramap_a, proc, ra0, lra, s_array_map);
+		  scm_ramapc (ramap_a, proc, ra0, lra, s_array_map_x);
 	  }
 	return SCM_UNSPECIFIED;
       }
@@ -1759,8 +1757,7 @@ rafe (ra0, proc, ras)
   long inc0 = SCM_ARRAY_DIMS (ra0)->inc;
   long n = SCM_ARRAY_DIMS (ra0)->ubnd;
   ra0 = SCM_ARRAY_V (ra0);
-  if SCM_NULLP
-    (ras)
+  if (SCM_NULLP (ras))
       for (; i <= n; i++, i0 += inc0)
 	scm_apply (proc, scm_cvref (ra0, i0, SCM_UNDEFINED), scm_listofnull);
   else
@@ -1771,8 +1768,7 @@ rafe (ra0, proc, ras)
       long inc1 = SCM_ARRAY_DIMS (ra1)->inc;
       ra1 = SCM_ARRAY_V (ra1);
       ras = SCM_CDR (ras);
-      if SCM_NULLP
-	(ras)
+      if (SCM_NULLP(ras))
 	  ras = scm_nullvect;
       else
 	{
@@ -1814,69 +1810,80 @@ scm_array_index_map_x (ra, proc)
 {
   scm_sizet i;
   SCM_ASSERT (SCM_NIMP (ra), ra, SCM_ARG1, s_array_index_map_x);
-  SCM_ASSERT (SCM_BOOL_T == scm_procedure_p (proc), proc, SCM_ARG2, s_array_index_map_x);
-  switch SCM_TYP7
-    (ra)
+  SCM_ASSERT (SCM_BOOL_T == scm_procedure_p (proc), proc, SCM_ARG2,
+	      s_array_index_map_x);
+  switch (SCM_TYP7(ra))
+    {
+    default:
+    badarg:scm_wta (ra, (char *) SCM_ARG1, s_array_index_map_x);
+    case scm_tc7_vector:
+    case scm_tc7_wvect:
       {
-      default:
-      badarg:scm_wta (ra, (char *) SCM_ARG1, s_array_index_map_x);
-      case scm_tc7_vector:
-	{
-	  SCM *ve = SCM_VELTS (ra);
-	  for (i = 0; i < SCM_LENGTH (ra); i++)
-	    ve[i] = scm_apply (proc, SCM_MAKINUM (i), scm_listofnull);
-	  return SCM_UNSPECIFIED;
-	}
-      case scm_tc7_string:
-      case scm_tc7_byvect:
-      case scm_tc7_bvect:
-      case scm_tc7_uvect:
-      case scm_tc7_ivect:
-      case scm_tc7_fvect:
-      case scm_tc7_dvect:
-      case scm_tc7_cvect:
+	SCM *ve = SCM_VELTS (ra);
 	for (i = 0; i < SCM_LENGTH (ra); i++)
-	  scm_array_set_x (ra, scm_apply (proc, SCM_MAKINUM (i), scm_listofnull), SCM_MAKINUM (i));
+	  ve[i] = scm_apply (proc, SCM_MAKINUM (i), scm_listofnull);
 	return SCM_UNSPECIFIED;
-      case scm_tc7_smob:
-	SCM_ASRTGO (SCM_ARRAYP (ra), badarg);
-	{
-	  SCM args = SCM_EOL;
-	  SCM inds = scm_make_uve (SCM_ARRAY_NDIM (ra), SCM_MAKINUM (-1L));
-	  long *vinds = SCM_VELTS (inds);
-	  int j, k, kmax = SCM_ARRAY_NDIM (ra) - 1;
-	  for (k = 0; k <= kmax; k++)
-	    vinds[k] = SCM_ARRAY_DIMS (ra)[k].lbnd;
-	  k = kmax;
-	  do
-	    {
-	      if (k == kmax)
-		{
-		  vinds[k] = SCM_ARRAY_DIMS (ra)[k].lbnd;
-		  i = cind (ra, inds);
-		  for (; vinds[k] <= SCM_ARRAY_DIMS (ra)[k].ubnd; vinds[k]++)
-		    {
-		      for (j = kmax + 1, args = SCM_EOL; j--;)
-			args = scm_cons (SCM_MAKINUM (vinds[j]), args);
-		      scm_array_set_x (SCM_ARRAY_V (ra), scm_apply (proc, args, SCM_EOL), SCM_MAKINUM (i));
-		      i += SCM_ARRAY_DIMS (ra)[k].inc;
-		    }
-		  k--;
-		  continue;
-		}
-	      if (vinds[k] < SCM_ARRAY_DIMS (ra)[k].ubnd)
-		{
-		  vinds[k]++;
-		  k++;
-		  continue;
-		}
-	      vinds[k] = SCM_ARRAY_DIMS (ra)[k].lbnd - 1;
-	      k--;
-	    }
-	  while (k >= 0);
-	  return SCM_UNSPECIFIED;
-	}
       }
+    case scm_tc7_string:
+    case scm_tc7_byvect:
+    case scm_tc7_bvect:
+    case scm_tc7_uvect:
+    case scm_tc7_ivect:
+    case scm_tc7_svect:
+#ifdef LONGLONGS
+    case scm_tc7_llvect:
+#endif
+    case scm_tc7_fvect:
+    case scm_tc7_dvect:
+    case scm_tc7_cvect:
+      for (i = 0; i < SCM_LENGTH (ra); i++)
+	scm_array_set_x (ra, scm_apply (proc, SCM_MAKINUM (i), scm_listofnull),
+			 SCM_MAKINUM (i));
+      return SCM_UNSPECIFIED;
+    case scm_tc7_smob:
+      SCM_ASRTGO (SCM_ARRAYP (ra), badarg);
+      {
+	SCM args = SCM_EOL;
+	SCM inds = scm_make_uve (SCM_ARRAY_NDIM (ra), SCM_MAKINUM (-1L));
+	long *vinds = SCM_VELTS (inds);
+	int j, k, kmax = SCM_ARRAY_NDIM (ra) - 1;
+	if (kmax < 0)
+	  return scm_array_set_x (ra, scm_apply(proc, SCM_EOL, SCM_EOL),
+				  SCM_EOL);
+	for (k = 0; k <= kmax; k++)
+	  vinds[k] = SCM_ARRAY_DIMS (ra)[k].lbnd;
+	k = kmax;
+	do
+	  {
+	    if (k == kmax)
+	      {
+		vinds[k] = SCM_ARRAY_DIMS (ra)[k].lbnd;
+		i = cind (ra, inds);
+		for (; vinds[k] <= SCM_ARRAY_DIMS (ra)[k].ubnd; vinds[k]++)
+		  {
+		    for (j = kmax + 1, args = SCM_EOL; j--;)
+		      args = scm_cons (SCM_MAKINUM (vinds[j]), args);
+		    scm_array_set_x (SCM_ARRAY_V (ra),
+				     scm_apply (proc, args, SCM_EOL),
+				     SCM_MAKINUM (i));
+		    i += SCM_ARRAY_DIMS (ra)[k].inc;
+		  }
+		k--;
+		continue;
+	      }
+	    if (vinds[k] < SCM_ARRAY_DIMS (ra)[k].ubnd)
+	      {
+		vinds[k]++;
+		k++;
+		continue;
+	      }
+	    vinds[k] = SCM_ARRAY_DIMS (ra)[k].lbnd - 1;
+	    k--;
+	  }
+	while (k >= 0);
+	return SCM_UNSPECIFIED;
+      }
+    }
 }
 
 
@@ -1893,37 +1900,32 @@ raeql_1 (ra0, as_equal, ra1)
   long inc0 = 1, inc1 = 1;
   scm_sizet n = SCM_LENGTH (ra0);
   ra1 = SCM_CAR (ra1);
-  if SCM_ARRAYP
-    (ra0)
+  if (SCM_ARRAYP(ra0))
       {
 	n = SCM_ARRAY_DIMS (ra0)->ubnd - SCM_ARRAY_DIMS (ra0)->lbnd + 1;
 	i0 = SCM_ARRAY_BASE (ra0);
 	inc0 = SCM_ARRAY_DIMS (ra0)->inc;
 	ra0 = SCM_ARRAY_V (ra0);
       }
-  if SCM_ARRAYP
-    (ra1)
+  if (SCM_ARRAYP (ra1))
       {
 	i1 = SCM_ARRAY_BASE (ra1);
 	inc1 = SCM_ARRAY_DIMS (ra1)->inc;
 	ra1 = SCM_ARRAY_V (ra1);
       }
-  switch SCM_TYP7
-    (ra0)
+  switch (SCM_TYP7 (ra0))
       {
       case scm_tc7_vector:
+      case scm_tc7_wvect:
       default:
 	for (; n--; i0 += inc0, i1 += inc1)
 	  {
-	    if SCM_FALSEP
-	      (as_equal)
+	    if (SCM_FALSEP (as_equal))
 		{
-		  if SCM_FALSEP
-		    (scm_array_equal_p (RVREF (ra0, i0, e0), RVREF (ra1, i1, e1)))
+                if (SCM_FALSEP (scm_array_equal_p (RVREF (ra0, i0, e0), RVREF (ra1, i1, e1))))
 		      return 0;
 		}
-	    else if SCM_FALSEP
-	      (scm_equal_p (RVREF (ra0, i0, e0), RVREF (ra1, i1, e1)))
+	    else if (SCM_FALSEP (scm_equal_p (RVREF (ra0, i0, e0), RVREF (ra1, i1, e1))))
 		return 0;
 	  }
 	return 1;
@@ -1952,6 +1954,26 @@ raeql_1 (ra0, as_equal, ra1)
 	      return 0;
 	  return 1;
 	}
+      case scm_tc7_svect:
+	{
+	  short *v0 = (short *) SCM_VELTS (ra0) + i0;
+	  short *v1 = (short *) SCM_VELTS (ra1) + i1;
+	  for (; n--; v0 += inc0, v1 += inc1)
+	    if (*v0 != *v1)
+	      return 0;
+	  return 1;
+	}
+#ifdef LONGLONGS
+      case scm_tc7_llvect:
+	{
+	  long long *v0 = (long long *) SCM_VELTS (ra0) + i0;
+	  long long *v1 = (long long *) SCM_VELTS (ra1) + i1;
+	  for (; n--; v0 += inc0, v1 += inc1)
+	    if (*v0 != *v1)
+	      return 0;
+	  return 1;
+	}
+#endif
 #ifdef SCM_FLOATS
 #ifdef SCM_SINGLES
       case scm_tc7_fvect:
@@ -2005,8 +2027,7 @@ raeql (ra0, as_equal, ra1)
   scm_array_dim *s0 = &dim0, *s1 = &dim1;
   scm_sizet bas0 = 0, bas1 = 0;
   int k, unroll = 1, vlen = 1, ndim = 1;
-  if SCM_ARRAYP
-    (ra0)
+  if (SCM_ARRAYP (ra0))
       {
 	ndim = SCM_ARRAY_NDIM (ra0);
 	s0 = SCM_ARRAY_DIMS (ra0);
@@ -2020,8 +2041,7 @@ raeql (ra0, as_equal, ra1)
       s0->ubnd = SCM_LENGTH (v0) - 1;
       unroll = 0;
     }
-  if SCM_ARRAYP
-    (ra1)
+  if (SCM_ARRAYP (ra1))
       {
 	if (ndim != SCM_ARRAY_NDIM (ra1))
 	  return 0;
@@ -2074,8 +2094,7 @@ scm_array_equal_p (ra0, ra1)
 {
   if (SCM_IMP (ra0) || SCM_IMP (ra1))
   callequal:return scm_equal_p (ra0, ra1);
-  switch SCM_TYP7
-    (ra0)
+  switch (SCM_TYP7(ra0))
       {
       default:
 	goto callequal;
@@ -2088,13 +2107,13 @@ scm_array_equal_p (ra0, ra1)
       case scm_tc7_dvect:
       case scm_tc7_cvect:
       case scm_tc7_vector:
+      case scm_tc7_wvect:
 	break;
       case scm_tc7_smob:
 	if (!SCM_ARRAYP (ra0))
 	  goto callequal;
       }
-  switch SCM_TYP7
-    (ra1)
+  switch (SCM_TYP7 (ra1))
       {
       default:
 	goto callequal;
@@ -2107,6 +2126,7 @@ scm_array_equal_p (ra0, ra1)
       case scm_tc7_dvect:
       case scm_tc7_cvect:
       case scm_tc7_vector:
+      case scm_tc7_wvect:
 	break;
       case scm_tc7_smob:
 	if (!SCM_ARRAYP (ra1))
