@@ -1,4 +1,4 @@
-/*	Copyright (C) 1997 Free Software Foundation, Inc.
+/*	Copyright (C) 1997, 1998, 1999 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,7 +12,8 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -58,8 +59,18 @@
    the file is CPP'able (for dependency scanning) even on systems that
    don't have a <regex.h> header.  */
 #ifdef HAVE_REGCOMP
+#ifdef HAVE_REGEX_H
 #include <regex.h>
-#endif 
+#else
+#ifdef HAVE_RXPOSIX_H
+#include <rxposix.h>		/* GNU Rx library */
+#else
+#ifdef HAVE_RX_RXPOSIX_H
+#include <rx/rxposix.h>		/* GNU Rx library on Linux */
+#endif
+#endif
+#endif
+#endif
 
 #include "smob.h"
 #include "symbols.h"
@@ -70,42 +81,28 @@
 
 #include "regex-posix.h"
 
-long scm_tc16_regex_t;
+/* This is defined by some regex libraries and omitted by others. */
+#ifndef REG_BASIC
+#define REG_BASIC 0
+#endif
 
-static size_t
-scm_free_regex_t (obj)
+long scm_tc16_regex;
+
+static scm_sizet
+free_regex (obj)
      SCM obj;
 {
   regfree (SCM_RGX (obj));
   free (SCM_RGX (obj));
-  return 0;
+  return sizeof(regex_t);
 }
 
-static int
-scm_print_regex_t (obj, port, pstate)
-     SCM obj;
-     SCM port;
-     scm_print_state *pstate;
-{
-  regex_t *r;
-  r = SCM_RGX (obj);
-  scm_gen_puts (scm_regular_string, "#<rgx ", port);
-  scm_intprint (obj, 16, port);
-  scm_gen_puts (scm_regular_string, ">", port);
-  return 1;
-}
-
-
-static scm_smobfuns regex_t_smob =
-{ scm_mark0, scm_free_regex_t, scm_print_regex_t, 0 };
 
 
 SCM_SYMBOL (scm_regexp_error_key, "regular-expression-syntax");
 
-char *
-scm_regexp_error_msg (regerrno, rx)
-     int regerrno;
-     SCM rx;
+static char *
+scm_regexp_error_msg (int regerrno, regex_t *rx)
 {
   SCM errmsg;
   int l;
@@ -122,11 +119,11 @@ scm_regexp_error_msg (regerrno, rx)
 
   errmsg = scm_make_string (SCM_MAKINUM (80), SCM_UNDEFINED);
   SCM_DEFER_INTS;
-  l = regerror (regerrno, SCM_RGX (rx), SCM_CHARS (errmsg), 80);
+  l = regerror (regerrno, rx, SCM_CHARS (errmsg), 80);
   if (l > 80)
     {
       errmsg = scm_make_string (SCM_MAKINUM (l), SCM_UNDEFINED);
-      regerror (regerrno, SCM_RGX (rx), SCM_CHARS (errmsg), l);
+      regerror (regerrno, rx, SCM_CHARS (errmsg), l);
     }
   SCM_ALLOW_INTS;
   return SCM_CHARS (errmsg);
@@ -141,31 +138,39 @@ scm_regexp_p (x)
   return (SCM_NIMP (x) && SCM_RGXP (x) ? SCM_BOOL_T : SCM_BOOL_F);
 }
 
-SCM_PROC (s_make_regexp, "make-regexp", 1, 1, 0, scm_make_regexp);
+SCM_PROC (s_make_regexp, "make-regexp", 1, 0, 1, scm_make_regexp);
 
 SCM
 scm_make_regexp (SCM pat, SCM flags)
 {
-  SCM result;
+  SCM flag;
   regex_t *rx;
-  int status;
+  int status, cflags;
 
   SCM_ASSERT (SCM_NIMP(pat) && SCM_ROSTRINGP(pat), pat, SCM_ARG1, 
 	      s_make_regexp);
   SCM_COERCE_SUBSTR (pat);
-  if (SCM_UNBNDP (flags))
-    flags = SCM_MAKINUM (REG_EXTENDED);
-  SCM_ASSERT (SCM_INUMP (flags), flags, SCM_ARG2, s_make_regexp);
 
-  SCM_DEFER_INTS;
+  /* Examine list of regexp flags.  If REG_BASIC is supplied, then
+     turn off REG_EXTENDED flag (on by default). */
+  cflags = REG_EXTENDED;
+  flag = flags;
+  while (SCM_NNULLP (flag))
+    {
+      if (SCM_INUM (SCM_CAR (flag)) == REG_BASIC)
+	cflags &= ~REG_EXTENDED;
+      else
+	cflags |= SCM_INUM (SCM_CAR (flag));
+      flag = SCM_CDR (flag);
+    }
+	  
   rx = (regex_t *) scm_must_malloc (sizeof (regex_t), s_make_regexp);
   status = regcomp (rx, SCM_ROCHARS (pat),
 		    /* Make sure they're not passing REG_NOSUB;
                        regexp-exec assumes we're getting match data.  */
-		    (SCM_INUM (flags) & ~REG_NOSUB));
+		    cflags & ~REG_NOSUB);
   if (status != 0)
     {
-      SCM_ALLOW_INTS;
       scm_error (scm_regexp_error_key,
 		 s_make_regexp,
 		 scm_regexp_error_msg (status, rx),
@@ -173,11 +178,7 @@ scm_make_regexp (SCM pat, SCM flags)
 		 SCM_BOOL_F);
       /* never returns */
     }
-  SCM_NEWCELL (result);
-  SCM_SETCAR (result, scm_tc16_regex_t);
-  SCM_SETCDR (result, rx);
-  SCM_ALLOW_INTS;
-  return result;
+  SCM_RETURN_NEWSMOB (scm_tc16_regex, rx);
 }
 
 SCM_PROC (s_regexp_exec, "regexp-exec", 2, 2, 0, scm_regexp_exec);
@@ -199,7 +200,7 @@ scm_regexp_exec (SCM rx, SCM str, SCM start, SCM flags)
     {
       SCM_ASSERT (SCM_INUMP (start), start, SCM_ARG3, s_regexp_exec);
       offset = SCM_INUM (start);
-      SCM_ASSERT (offset >= 0 && offset <= SCM_LENGTH (str), start,
+      SCM_ASSERT (offset >= 0 && (unsigned) offset <= SCM_LENGTH (str), start,
 		  SCM_OUTOFRANGE, s_regexp_exec);
     }
 
@@ -224,19 +225,23 @@ scm_regexp_exec (SCM rx, SCM str, SCM start, SCM flags)
       int i;
       /* The match vector must include a cell for the string that was matched,
 	 so add 1. */
-      mvec = scm_make_vector (SCM_MAKINUM (nmatches + 1), SCM_UNSPECIFIED,
-			      SCM_UNDEFINED);
+      mvec = scm_make_vector (SCM_MAKINUM (nmatches + 1), SCM_UNSPECIFIED);
       SCM_VELTS(mvec)[0] = str;
       for (i = 0; i < nmatches; ++i)
-	SCM_VELTS(mvec)[i+1] = scm_cons(SCM_MAKINUM(matches[i].rm_so + offset),
-					SCM_MAKINUM(matches[i].rm_eo + offset));
+	if (matches[i].rm_so == -1)
+	  SCM_VELTS(mvec)[i+1] = scm_cons (SCM_MAKINUM (-1), SCM_MAKINUM (-1));
+	else
+	  SCM_VELTS(mvec)[i+1]
+	    = scm_cons(SCM_MAKINUM(matches[i].rm_so + offset),
+		       SCM_MAKINUM(matches[i].rm_eo + offset));
     }
+  scm_must_free ((char *) matches);
   SCM_ALLOW_INTS;
 
   if (status != 0 && status != REG_NOMATCH)
     scm_error (scm_regexp_error_key,
 	       s_regexp_exec,
-	       scm_regexp_error_msg (status),
+	       scm_regexp_error_msg (status, SCM_RGX (rx)),
 	       SCM_BOOL_F,
 	       SCM_BOOL_F);
   return mvec;
@@ -245,9 +250,11 @@ scm_regexp_exec (SCM rx, SCM str, SCM start, SCM flags)
 void
 scm_init_regex_posix ()
 {
-  scm_tc16_regex_t = scm_newsmob (&regex_t_smob);
+  scm_tc16_regex = scm_make_smob_type_mfpe ("regexp", sizeof (regex_t),
+                                            NULL, free_regex, NULL, NULL);
 
   /* Compilation flags.  */
+  scm_sysintern ("regexp/basic", scm_long2num (REG_BASIC));
   scm_sysintern ("regexp/extended", scm_long2num (REG_EXTENDED));
   scm_sysintern ("regexp/icase", scm_long2num (REG_ICASE));
   scm_sysintern ("regexp/newline", scm_long2num (REG_NEWLINE));
