@@ -1,4 +1,4 @@
-/* Copyright (C) 1994, 1995, 1996, 1997 Free Software Foundation, Inc.
+/* Copyright (C) 1994, 1995, 1996, 1997, 1998 Free Software Foundation, Inc.
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -131,7 +131,6 @@ scm_sep_init_try (path, sep, initname)
 #define X_OK 1
 #endif /* ndef X_OK */
 
-#ifdef unix
 char *
 scm_find_executable (const char *name)
 {
@@ -163,23 +162,6 @@ scm_find_executable (const char *name)
   fclose (f);
   return scm_cat_path (0L, name, 0L);
 }
-#endif /* unix */
-
-#ifdef MSDOS
-
-#define DEFAULT_PATH "C:\\DOS"
-#define PATH_DELIMITER ';'
-#define ABSOLUTE_FILENAME_P(fname) ((fname[0] == '\\') \
-				     || (fname[0] && (fname[1] == ':')))
-
-char *
-dld_find_executable (file)
-     const char *file;
-{
-  /* fprintf(stderr, "dld_find_executable %s -> %s\n", file, scm_cat_path(0L, file, 0L)); fflush(stderr); */
-  return scm_cat_path (0L, file, 0L);
-}
-#endif /* def MSDOS */
 
 
 /* Read a \nnn-style escape.  We've just read the backslash.  */
@@ -403,6 +385,7 @@ scm_shell_usage (int fatal, char *message)
            "  -e FUNCTION    after reading script, apply FUNCTION to\n"
            "                 command line arguments\n"
            "  -ds            do -s script at this point\n"
+	   "  -q             inhibit loading of user init file\n"
            "  --emacs        enable Emacs protocol (experimental)\n"
            "  -h, --help     display this help and exit\n"
            "  -v, --version  display version information and exit\n"
@@ -424,12 +407,6 @@ SCM_SYMBOL (sym_top_repl, "top-repl");
 SCM_SYMBOL (sym_quit, "quit");
 
 
-/* The boot code "ice-9/boot-9" is only loaded by
-   SCM_COMPILE_SHELL_SWITCHES when this is false.  The unexec code
-   uses this, to keep ice_9 from being loaded into dumped guile
-   executables.  */
-int scm_ice_9_already_loaded = 0;
-
 /* Given an array of command-line switches, return a Scheme expression
    to carry out the actions specified by the switches.
 
@@ -448,6 +425,8 @@ int scm_ice_9_already_loaded = 0;
    the usual way, and the auto-generated inner_main will do the right
    thing. */
 
+static char guile[] = "guile";
+
 SCM
 scm_compile_shell_switches (int argc, char **argv)
 {
@@ -459,12 +438,14 @@ scm_compile_shell_switches (int argc, char **argv)
 				   the "-ds" switch.  */
   SCM entry_point = SCM_EOL;	/* for -e switch */
   int interactive = 1;		/* Should we go interactive when done? */
+  int inhibit_user_init = 0;	/* Don't load user init file */
   int use_emacs_interface = 0;
   int i;
-  char *argv0;
+  char *argv0 = guile;
 
   if (argc > 0)
     {
+      argv0 = argv[0];
       scm_usage_name = strrchr (argv[0], '/');
       if (! scm_usage_name)
 	scm_usage_name = argv[0];
@@ -472,8 +453,7 @@ scm_compile_shell_switches (int argc, char **argv)
 	scm_usage_name++;
     }
   if (! scm_usage_name)
-    scm_usage_name = "guile";
-  argv0 = scm_usage_name;
+    scm_usage_name = guile;
   
   for (i = 1; i < argc; i++)
     {
@@ -554,6 +534,9 @@ scm_compile_shell_switches (int argc, char **argv)
       else if (! strcmp (argv[i], "--emacs")) /* use emacs protocol */ 
 	use_emacs_interface = 1;
 
+      else if (! strcmp (argv[i], "-q")) /* don't load user init */ 
+	inhibit_user_init = 1;
+
       else if (! strcmp (argv[i], "-h")
 	       || ! strcmp (argv[i], "--help"))
 	{
@@ -589,11 +572,13 @@ scm_compile_shell_switches (int argc, char **argv)
 
   /* Make any remaining arguments available to the
      script/command/whatever.  */
-  scm_set_program_arguments (argc - i, argv + i, argv0);
+  scm_set_program_arguments (argc ? argc - i : 0, argv + i, argv0);
   
   /* If the --emacs switch was set, now is when we process it.  */
-  scm_sysintern ("use-emacs-interface",
-		 (use_emacs_interface) ? SCM_BOOL_T : SCM_BOOL_F);
+  {
+    SCM vcell = scm_sysintern0_no_module_lookup ("use-emacs-interface");
+    SCM_SETCDR (vcell, use_emacs_interface ? SCM_BOOL_T : SCM_BOOL_F);
+  }
 
   /* Handle the `-e' switch, if it was specified.  */
   if (entry_point != SCM_EOL)
@@ -602,11 +587,9 @@ scm_compile_shell_switches (int argc, char **argv)
 				SCM_EOL),
 		       tail);
 
-  /* If we didn't end with a -c or a -s, load the user's customization
-     file, and start the repl.  */
+  /* If we didn't end with a -c or a -s, start the repl.  */
   if (interactive)
     {
-      tail = scm_cons (scm_cons (sym_load_user_init, SCM_EOL), tail);
       tail = scm_cons (scm_cons (sym_top_repl, SCM_EOL), tail);
     }
   else
@@ -614,27 +597,23 @@ scm_compile_shell_switches (int argc, char **argv)
       /* After doing all the other actions prescribed by the command line,
 	 quit.  */
       tail = scm_cons (scm_cons (sym_quit, SCM_EOL),
-		   tail);
+		       tail);
       /* Allow asyncs (signal handlers etc.) to be run.  */
       scm_mask_ints = 0;
     }
-  {
-    /* We want a path only containing directories from SCHEME_LOAD_PATH,
-       SCM_SITE_DIR and SCM_LIBRARY_DIR when searching for the site init
-       file, so we do this before loading Ice-9.  */
-    SCM init_path = scm_sys_search_load_path (scm_makfrom0str ("init.scm"));
 
-    /* Load Ice-9.  */
-    if (!scm_ice_9_already_loaded)
-      scm_primitive_load_path (scm_makfrom0str ("ice-9/boot-9.scm"));
-
-    /* Load the init.scm file.  */
-    if (SCM_NFALSEP (init_path))
-      scm_primitive_load (init_path);
-  }
+  /* After the following line, actions will be added to the front. */
+  tail = scm_reverse_x (tail, SCM_UNDEFINED);
+  
+  /* If we didn't end with a -c or a -s and didn't supply a -q, load
+     the user's customization file.  */
+  if (interactive && !inhibit_user_init)
+    {
+      tail = scm_cons (scm_cons (sym_load_user_init, SCM_EOL), tail);
+    }
 
   {
-    SCM val = scm_cons (sym_begin, scm_list_reverse_x (tail, SCM_UNDEFINED));
+    SCM val = scm_cons (sym_begin, tail);
 
 #if 0
     scm_write (val, SCM_UNDEFINED);
