@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996,1997,1998 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -57,24 +57,38 @@
 extern int system();
 
 
-#ifndef _Windows
-SCM_PROC(s_system, "system", 1, 0, 0, scm_system);
+SCM_PROC(s_system, "system", 0, 1, 0, scm_system);
 
 SCM
 scm_system(cmd)
      SCM cmd;
 {
+  int rv;
+
+  if (SCM_UNBNDP (cmd))
+    {
+#ifdef HAVE_SYSTEM
+      rv = system (NULL);
+#else
+      rv = 0;
+#endif
+      return rv ? SCM_BOOL_T : SCM_BOOL_F;
+    }
   SCM_ASSERT(SCM_NIMP(cmd) && SCM_ROSTRINGP(cmd), cmd, SCM_ARG1, s_system);
+#ifdef HAVE_SYSTEM
+  SCM_DEFER_INTS;
+  errno = 0;
   if (SCM_ROSTRINGP (cmd))
     cmd = scm_makfromstr (SCM_ROCHARS (cmd), SCM_ROLENGTH (cmd), 0);
-# ifdef AZTEC_C
-  cmd = SCM_MAKINUM(Execute(SCM_ROCHARS(cmd), 0, 0));
-# else
-  cmd = SCM_MAKINUM(0L+system(SCM_ROCHARS(cmd)));
-# endif
-  return cmd;
-}
+  rv = system(SCM_ROCHARS(cmd));
+  if (rv == -1 || (rv == 127 && errno != 0))
+    scm_syserror (s_system);
+  SCM_ALLOW_INTS;
+  return SCM_MAKINUM (rv);
+#else
+  scm_sysmissing (s_system);
 #endif
+}
 
 extern char *getenv();
 SCM_PROC (s_getenv, "getenv", 1, 0, 0, scm_getenv);
@@ -91,64 +105,18 @@ scm_getenv(nam)
   return (val) ? scm_makfromstr(val, (scm_sizet)strlen(val), 0) : SCM_BOOL_F;
 }
 
-/* I have a feeling this whole arrangement below is a bad idea.  One
-   should always test for the presence or absence of a particular
-   feature, instead of checking the system name.  Older versions of a
-   system may lack features posessed by new ones, and a feature
-   appearing on one system will soon appear on others.  And autoconf
-   provides the mechanisms for detecting features.  -JimB  */
-
-#ifdef _AIX
-# define SYSTNAME "AIX"
-#endif
-#ifdef vms
-# define SYSTNAME "VMS"
-#endif
-#if defined (unix) || defined (__unix)
-# define SYSTNAME "UNIX"
-#endif
-#ifdef MWC
-# define SYSTNAME "COHERENT"
-#endif
-#ifdef _Windows
-# define SYSTNAME "WINDOWS"
-#else
-# ifdef MSDOS
-#  define SYSTNAME "MS-DOS"
-# endif
-#endif
-#ifdef __EMX__
-# define SYSTNAME "OS/2"
-#endif
-#ifdef __IBMC__
-# define SYSTNAME "OS/2"
-#endif
-#ifdef THINK_C
-# define SYSTNAME "THINKC"
-#endif
-#ifdef AMIGA
-# define SYSTNAME "AMIGA"
-#endif
-#ifdef atarist
-# define SYSTNAME "ATARIST"
-#endif
-#ifdef mach
-# define SYSTNAME "MACH"
-#endif
-#ifdef ARM_ULIB
-# define SYSTNAME "ACORN"
-#endif
-
-SCM_PROC(s_software_type, "software-type", 0, 0, 0, scm_software_type);
-
+/* simple exit, without unwinding the scheme stack or flushing ports.  */
+SCM_PROC (s_primitive_exit, "primitive-exit", 0, 1, 0, scm_primitive_exit);
 SCM
-scm_software_type()
+scm_primitive_exit (SCM status)
 {
-#ifdef nosve
-  return SCM_CAR(scm_intern("nosve", 5));
-#else
-  return SCM_CAR(scm_intern(SYSTNAME, sizeof SYSTNAME/sizeof(char) -1));
-#endif
+  int cstatus = 0;
+  if (!SCM_UNBNDP (status))
+    {
+      SCM_ASSERT (SCM_INUMP (status), status, SCM_ARG1, s_primitive_exit);
+      cstatus = SCM_INUM (status);
+    }
+  exit (cstatus);
 }
 
 
