@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
+/*	Copyright (C) 1995, 1996, 1999 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -110,8 +110,8 @@ scm_string_less_p (s1, s2)
   SCM_ASSERT (SCM_NIMP (s1) && SCM_ROSTRINGP (s1), s1, SCM_ARG1, s_string_less_p);
   SCM_ASSERT (SCM_NIMP (s2) && SCM_ROSTRINGP (s2), s2, SCM_ARG2, s_string_less_p);
   len = SCM_ROLENGTH (s1);
-  s2len = i = SCM_ROLENGTH (s2);
-  if (len>i) i = len;
+  s2len = SCM_ROLENGTH (s2);
+  if (len>s2len) len = s2len;
   c1 = SCM_ROUCHARS (s1);
   c2 = SCM_ROUCHARS (s2);
 
@@ -172,8 +172,8 @@ scm_string_ci_less_p (s1, s2)
   SCM_ASSERT (SCM_NIMP (s1) && SCM_ROSTRINGP (s1), s1, SCM_ARG1, s_string_ci_less_p);
   SCM_ASSERT (SCM_NIMP (s2) && SCM_ROSTRINGP (s2), s2, SCM_ARG2, s_string_ci_less_p);
   len = SCM_ROLENGTH (s1);
-  s2len = i = SCM_ROLENGTH (s2);
-  if (len>i) i=len;
+  s2len = SCM_ROLENGTH (s2);
+  if (len>s2len) len = s2len;
   c1 = SCM_ROUCHARS (s1);
   c2 = SCM_ROUCHARS (s2);
   for (i = 0;i<len;i++) {
