@@ -1,4 +1,4 @@
-/*	Copyright (C) 1996, 1997, 1998 Free Software Foundation, Inc.
+/*	Copyright (C) 1996, 1997, 1998, 1999 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -331,20 +331,20 @@ scm_alloc_struct (int n_words, int n_extra, char *who)
   return p;
 }
 
-size_t
+scm_sizet
 scm_struct_free_0 (SCM *vtable, SCM *data)
 {
   return 0;
 }
 
-size_t
+scm_sizet
 scm_struct_free_light (SCM *vtable, SCM *data)
 {
   free (data);
   return vtable[scm_struct_i_size] & ~SCM_STRUCTF_MASK;
 }
 
-size_t
+scm_sizet
 scm_struct_free_standard (SCM *vtable, SCM *data)
 {
   size_t n = ((data[scm_struct_i_n_words] + scm_struct_n_extra_words)
@@ -353,7 +353,7 @@ scm_struct_free_standard (SCM *vtable, SCM *data)
   return n;
 }
 
-size_t
+scm_sizet
 scm_struct_free_entity (SCM *vtable, SCM *data)
 {
   size_t n = ((data[scm_struct_i_n_words] + scm_struct_entity_n_extra_words)
@@ -391,10 +391,7 @@ scm_make_struct (vtable, tail_array_size, init)
       data = scm_alloc_struct (basic_size + tail_elts,
 			       scm_struct_entity_n_extra_words,
 			       "make-struct");
-      data[scm_struct_i_proc + 0] = SCM_BOOL_F;
-      data[scm_struct_i_proc + 1] = SCM_BOOL_F;
-      data[scm_struct_i_proc + 2] = SCM_BOOL_F;
-      data[scm_struct_i_proc + 3] = SCM_BOOL_F;
+      data[scm_struct_i_procedure] = SCM_BOOL_F;
       data[scm_struct_i_setter] = SCM_BOOL_F;
     }
   else
