@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995,1996,1997 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996,1997,1998 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,7 +46,6 @@
 #include "eval.h"
 #include "variable.h"
 #include "alist.h"
-#include "mbstrings.h"
 #include "weaks.h"
 
 #include "symbols.h"
@@ -231,7 +230,7 @@ scm_sym2ovcell (sym, obarray)
 
 SCM 
 scm_intern_obarray_soft (name, len, obarray, softness)
-     char *name;
+     const char *name;
      scm_sizet len;
      SCM obarray;
      int softness;
@@ -298,7 +297,6 @@ scm_intern_obarray_soft (name, len, obarray, softness)
   lsym = scm_makfromstr (name, len, SCM_SYMBOL_SLOTS);
 
   SCM_SETLENGTH (lsym, (long) len, scm_tc7_msymbol);
-  SCM_SYMBOL_MULTI_BYTE_STRINGP (lsym) = SCM_BOOL_F;
   SCM_SYMBOL_HASH (lsym) = scm_hash;
   SCM_SYMBOL_PROPS (lsym) = SCM_EOL;
   if (obarray == SCM_BOOL_F)
@@ -332,7 +330,7 @@ scm_intern_obarray_soft (name, len, obarray, softness)
 
 SCM
 scm_intern_obarray (name, len, obarray)
-     char *name;
+     const char *name;
      scm_sizet len;
      SCM obarray;
 {
@@ -342,7 +340,7 @@ scm_intern_obarray (name, len, obarray)
 
 SCM 
 scm_intern (name, len)
-     char *name;
+     const char *name;
      scm_sizet len;
 {
   return scm_intern_obarray (name, len, scm_symhash);
@@ -351,7 +349,7 @@ scm_intern (name, len)
 
 SCM
 scm_intern0 (name)
-     char * name;
+     const char * name;
 {
   return scm_intern (name, strlen (name));
 }
@@ -360,7 +358,7 @@ scm_intern0 (name)
 /* Intern the symbol named NAME in scm_symhash, NAME is null-terminated.  */
 SCM 
 scm_sysintern0_no_module_lookup (name)
-     char *name;
+     const char *name;
 {
   SCM easy_answer;
   SCM_DEFER_INTS;
@@ -397,7 +395,7 @@ int scm_can_use_top_level_lookup_closure_var;
  */
 SCM
 scm_sysintern (name, val)
-     char *name;
+     const char *name;
      SCM val;
 {
   SCM vcell = scm_sysintern0 (name);
@@ -407,7 +405,7 @@ scm_sysintern (name, val)
 
 SCM
 scm_sysintern0 (name)
-     char *name;
+     const char *name;
 {
   SCM lookup_proc;
   if (scm_can_use_top_level_lookup_closure_var && 
@@ -427,7 +425,7 @@ scm_sysintern0 (name)
    NAME in the current module.  */
 SCM
 scm_symbol_value0 (name)
-     char *name;
+     const char *name;
 {
   /* This looks silly - we look up the symbol twice.  But it is in
      fact necessary given the current module system because the module
@@ -474,13 +472,6 @@ scm_string_to_symbol(s)
   SCM_ASSERT(SCM_NIMP(s) && SCM_ROSTRINGP(s), s, SCM_ARG1, s_string_to_symbol);
   vcell = scm_intern(SCM_ROCHARS(s), (scm_sizet)SCM_LENGTH(s));
   answer = SCM_CAR (vcell);
-  if (SCM_TYP7 (answer) == scm_tc7_msymbol)
-    {
-      if (SCM_REGULAR_STRINGP (s))
-	SCM_SYMBOL_MULTI_BYTE_STRINGP (answer) = SCM_BOOL_F;
-      else
-	SCM_SYMBOL_MULTI_BYTE_STRINGP (answer) = SCM_BOOL_T;
-    }
   return answer;
 }
 
@@ -520,13 +511,6 @@ scm_string_to_obarray_symbol(o, s, softp)
   if (vcell == SCM_BOOL_F)
     return vcell;
   answer = SCM_CAR (vcell);
-  if (SCM_TYP7 (s) == scm_tc7_msymbol)
-    {
-      if (SCM_REGULAR_STRINGP (s))
-	SCM_SYMBOL_MULTI_BYTE_STRINGP (answer) = SCM_BOOL_F;
-      else
-	SCM_SYMBOL_MULTI_BYTE_STRINGP (answer) = SCM_BOOL_T;
-    }
   return answer;
 }
 
@@ -689,7 +673,6 @@ msymbolize (s)
   string = scm_makfromstr (SCM_CHARS (s), SCM_LENGTH (s), SCM_SYMBOL_SLOTS);
   SCM_SETCHARS (s, SCM_CHARS (string));
   SCM_SETLENGTH (s, SCM_LENGTH (s), scm_tc7_msymbol);
-  SCM_SYMBOL_MULTI_BYTE_STRINGP (s) = SCM_BOOL_F;
   SCM_SETCDR (string, SCM_EOL);
   SCM_SETCAR (string, SCM_EOL);
   SCM_SYMBOL_PROPS (s) = SCM_EOL;
@@ -812,7 +795,7 @@ SCM
 scm_builtin_bindings ()
 {
   int length = SCM_LENGTH (scm_symhash);
-  SCM obarray = scm_make_vector (SCM_MAKINUM (length), SCM_EOL, SCM_UNDEFINED);
+  SCM obarray = scm_make_vector (SCM_MAKINUM (length), SCM_EOL);
   copy_and_prune_obarray (scm_symhash, obarray);
   return obarray;
 }
@@ -829,11 +812,50 @@ scm_builtin_weak_bindings ()
   return obarray;
 }
 
+static int gensym_counter;
+static SCM gensym_prefix;
 
+/*fixme* Optimize */
+SCM_PROC (s_gensym, "gensym", 0, 2, 0, scm_gensym);
+
+SCM
+scm_gensym (name, obarray)
+     SCM name;
+     SCM obarray;
+{
+  SCM new;
+  if (SCM_UNBNDP (name))
+    name = gensym_prefix;
+  else
+    SCM_ASSERT (SCM_NIMP (name) && SCM_ROSTRINGP (name),
+		name, SCM_ARG1, s_gensym);
+  new = name;
+  if (SCM_UNBNDP (obarray))
+    {
+      obarray = SCM_BOOL_F;
+      goto skip_test;
+    }
+  else
+    SCM_ASSERT (SCM_NIMP (obarray)
+		&& (SCM_VECTORP (obarray) || SCM_WVECTP (obarray)),
+		obarray,
+		SCM_ARG2,
+		s_gensym);
+  while (scm_string_to_obarray_symbol (obarray, new, SCM_BOOL_T)
+	 != SCM_BOOL_F)
+    skip_test:
+    new = scm_string_append
+      (scm_cons2 (name,
+		  scm_number_to_string (SCM_MAKINUM (gensym_counter++),
+					SCM_UNDEFINED),
+		  SCM_EOL));
+  return scm_string_to_obarray_symbol (obarray, new, SCM_BOOL_F);
+}
 
 void
 scm_init_symbols ()
 {
+  gensym_counter = 0;
+  gensym_prefix = scm_permanent_object (scm_makfrom0str ("%%gensym"));
 #include "symbols.x"
 }
-
