@@ -2,7 +2,7 @@
 
 #ifndef SYMBOLSH
 #define SYMBOLSH
-/*	Copyright (C) 1995,1996,1997 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996,1997,1998 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -59,9 +59,8 @@ extern int scm_symhash_dim;
    and that's it.  They use the scm_tc7_ssymbol tag (S bit clear).
 
    Msymbols are symbols with extra slots.  These slots hold a property
-   list and a function value (for Emacs Lisp compatibility), a hash
-   code, and a flag to indicate whether their name contains multibyte
-   characters.  They use the scm_tc7_msymbol tag.
+   list and a function value (for Emacs Lisp compatibility), and a hash
+   code.  They use the scm_tc7_msymbol tag.
 
    We'd like SCM_CHARS to work on msymbols just as it does on
    ssymbols, so we'll have it point to the symbol's name as usual, and
@@ -82,21 +81,21 @@ extern int scm_symhash_dim;
 #define SCM_CHARS(x) ((char *)(SCM_CDR(x)))
 #define SCM_UCHARS(x) ((unsigned char *)(SCM_CDR(x)))
 #define SCM_SLOTS(x) ((SCM *) (* ((SCM *)SCM_CHARS(x) - 1)))
-#define SCM_SYMBOL_SLOTS 5
+#define SCM_SYMBOL_SLOTS 4
 #define SCM_SYMBOL_FUNC(X) (SCM_SLOTS(X)[0])
 #define SCM_SYMBOL_PROPS(X) (SCM_SLOTS(X)[1])
 #define SCM_SYMBOL_HASH(X) (*(unsigned long*)(&SCM_SLOTS(X)[2]))
-#define SCM_SYMBOL_MULTI_BYTE_STRINGP(X) (*(unsigned long*)(&SCM_SLOTS(X)[3]))
 
-#define SCM_ROSTRINGP(x) ((SCM_TYP7SD(x)==scm_tc7_string) || (SCM_TYP7S(x) == scm_tc7_ssymbol))
+#define SCM_ROSTRINGP(x) ((SCM_TYP7S(x)==scm_tc7_string) \
+			  || (SCM_TYP7S(x) == scm_tc7_ssymbol))
 #define SCM_ROCHARS(x) ((SCM_TYP7(x) == scm_tc7_substring) \
 			? SCM_INUM (SCM_CADR (x)) + SCM_CHARS (SCM_CDDR (x))  \
 			: SCM_CHARS (x))
 #define SCM_ROUCHARS(x) ((SCM_TYP7(x) == scm_tc7_substring) \
-			 ? SCM_INUM (SCM_CADR (x)) + SCM_UCHARS (SCM_CDDR (x))  \
+			 ? SCM_INUM (SCM_CADR (x)) + SCM_UCHARS (SCM_CDDR (x))\
 			 : SCM_UCHARS (x))
 #define SCM_ROLENGTH(x) SCM_LENGTH (x)
-#define SCM_SUBSTRP(x) ((SCM_TYP7S(x) == scm_tc7_substring))
+#define SCM_SUBSTRP(x) ((SCM_TYP7(x) == scm_tc7_substring))
 #define SCM_SUBSTR_STR(x) (SCM_CDDR (x))
 #define SCM_SUBSTR_OFFSET(x) (SCM_CADR (x))
 
@@ -110,13 +109,14 @@ extern unsigned long scm_strhash SCM_P ((unsigned char *str, scm_sizet len, unsi
 extern SCM scm_sym2vcell SCM_P ((SCM sym, SCM thunk, SCM definep));
 extern SCM scm_sym2ovcell_soft SCM_P ((SCM sym, SCM obarray));
 extern SCM scm_sym2ovcell SCM_P ((SCM sym, SCM obarray));
-extern SCM scm_intern_obarray_soft SCM_P ((char *name, scm_sizet len, SCM obarray, int softness));
-extern SCM scm_intern_obarray SCM_P ((char *name, scm_sizet len, SCM obarray));
-extern SCM scm_intern SCM_P ((char *name, scm_sizet len));
-extern SCM scm_intern0 SCM_P ((char * name));
-extern SCM scm_sysintern SCM_P ((char *name, SCM val));
-extern SCM scm_sysintern0 SCM_P ((char *name));
-extern SCM scm_symbol_value0 SCM_P ((char *name));
+extern SCM scm_intern_obarray_soft SCM_P ((const char *name, scm_sizet len, SCM obarray, int softness));
+extern SCM scm_intern_obarray SCM_P ((const char *name, scm_sizet len, SCM obarray));
+extern SCM scm_intern SCM_P ((const char *name, scm_sizet len));
+extern SCM scm_intern0 SCM_P ((const char * name));
+extern SCM scm_sysintern SCM_P ((const char *name, SCM val));
+extern SCM scm_sysintern0 SCM_P ((const char *name));
+extern SCM scm_sysintern0_no_module_lookup SCM_P ((const char *name));
+extern SCM scm_symbol_value0 SCM_P ((const char *name));
 extern SCM scm_symbol_p SCM_P ((SCM x));
 extern SCM scm_symbol_to_string SCM_P ((SCM s));
 extern SCM scm_string_to_symbol SCM_P ((SCM s));
@@ -132,6 +132,9 @@ extern SCM scm_symbol_pref SCM_P ((SCM s));
 extern SCM scm_symbol_fset_x SCM_P ((SCM s, SCM val));
 extern SCM scm_symbol_pset_x SCM_P ((SCM s, SCM val));
 extern SCM scm_symbol_hash SCM_P ((SCM s));
+extern SCM scm_builtin_bindings SCM_P ((void));
+extern SCM scm_builtin_weak_bindings SCM_P ((void));
+extern SCM scm_gensym SCM_P ((SCM name, SCM obarray));
 extern void scm_init_symbols SCM_P ((void));
 
 extern int scm_can_use_top_level_lookup_closure_var;
