@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995,1996,1997 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996,1997,1998 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,9 +44,8 @@
 #include "_scm.h"
 #include "chars.h"
 #include "eval.h"
-#include "genio.h"
+#include "fports.h"
 #include "smob.h"
-#include "sequences.h"
 #include "strop.h"
 #include "feature.h"
 
@@ -63,8 +62,8 @@
  * unsigned char	string
  * char			byvect
  * boolean		bvect
- * signed int		ivect
- * unsigned int		uvect
+ * signed long		ivect
+ * unsigned long	uvect
  * float		fvect
  * double		dvect
  * complex double	cvect
@@ -100,12 +99,12 @@ scm_vector_set_length_x (vect, len)
     default:
     badarg1: scm_wta (vect, (char *) SCM_ARG1, s_vector_set_length_x);
     case scm_tc7_string:
-    case scm_tc7_mb_string:
       SCM_ASRTGO (vect != scm_nullstr, badarg1);
       sz = sizeof (char);
       l++;
       break;
     case scm_tc7_vector:
+    case scm_tc7_wvect:
       SCM_ASRTGO (vect != scm_nullvect, badarg1);
       sz = sizeof (SCM);
       break;
@@ -243,7 +242,7 @@ scm_make_uve (k, prot)
 #endif
       else
 	{
-	  return scm_make_vector (SCM_MAKINUM (k), SCM_UNDEFINED, SCM_UNDEFINED);
+	  return scm_make_vector (SCM_MAKINUM (k), SCM_UNDEFINED);
 	}
     }
   else
@@ -251,7 +250,7 @@ scm_make_uve (k, prot)
   if (SCM_IMP (prot) || !SCM_INEXP (prot))
 #endif
     /* Huge non-unif vectors are NOT supported. */
-    return scm_make_vector (SCM_MAKINUM (k), SCM_UNDEFINED, SCM_UNDEFINED);	/* no special scm_vector */
+    return scm_make_vector (SCM_MAKINUM (k), SCM_UNDEFINED);	/* no special scm_vector */
 #ifdef SCM_FLOATS
 #ifdef SCM_SINGLES
   else if (SCM_SINGP (prot))
@@ -306,6 +305,7 @@ scm_uniform_vector_length (v)
     case scm_tc7_dvect:
     case scm_tc7_cvect:
     case scm_tc7_vector:
+    case scm_tc7_wvect:
     case scm_tc7_svect:
 #ifdef LONGLONGS
     case scm_tc7_llvect:
@@ -374,6 +374,7 @@ loop:
       return nprot || (SCM_NIMP(prot) && SCM_CPLXP(prot)) ? SCM_BOOL_T : SCM_BOOL_F;
 # endif
     case scm_tc7_vector:
+    case scm_tc7_wvect:
       return nprot || SCM_NULLP(prot) ? SCM_BOOL_T : SCM_BOOL_F;
     default:;
     }
@@ -395,6 +396,7 @@ scm_array_rank (ra)
       return SCM_INUM0;
     case scm_tc7_string:
     case scm_tc7_vector:
+    case scm_tc7_wvect:
     case scm_tc7_byvect:
     case scm_tc7_uvect:
     case scm_tc7_ivect:
@@ -431,6 +433,7 @@ scm_array_dimensions (ra)
       return SCM_BOOL_F;
     case scm_tc7_string:
     case scm_tc7_vector:
+    case scm_tc7_wvect:
     case scm_tc7_bvect:
     case scm_tc7_byvect:
     case scm_tc7_uvect:
@@ -464,7 +467,7 @@ long
 scm_aind (ra, args, what)
      SCM ra;
      SCM args;
-     char *what;
+     const char *what;
 {
   SCM ind;
   register long j;
@@ -501,9 +504,9 @@ scm_make_ra (ndim)
   SCM ra;
   SCM_NEWCELL (ra);
   SCM_DEFER_INTS;
-  SCM_SETCDR (ra, scm_must_malloc ((long) (sizeof (scm_array) + ndim * sizeof (scm_array_dim)),
+  SCM_NEWSMOB(ra, ((long) ndim << 17) + scm_tc16_array,
+              scm_must_malloc ((long) (sizeof (scm_array) + ndim * sizeof (scm_array_dim)),
 			       "array"));
-  SCM_SETCAR (ra, ((long) ndim << 17) + scm_tc16_array);
   SCM_ARRAY_V (ra) = scm_nullvect;
   SCM_ALLOW_INTS;
   return ra;
@@ -516,7 +519,7 @@ static char s_bad_spec[] = "Bad scm_array dimension";
 SCM 
 scm_shap2ra (args, what)
      SCM args;
-     char *what;
+     const char *what;
 {
   scm_array_dim *s;
   SCM ra, spec, sp;
@@ -531,18 +534,21 @@ scm_shap2ra (args, what)
       if (SCM_IMP (spec))
 
 	{
-	  SCM_ASSERT (SCM_INUMP (spec) && SCM_INUM (spec) >= 0, spec, s_bad_spec, what);
+	  SCM_ASSERT (SCM_INUMP (spec) && SCM_INUM (spec) >= 0, spec,
+		      s_bad_spec, what);
 	  s->lbnd = 0;
 	  s->ubnd = SCM_INUM (spec) - 1;
 	  s->inc = 1;
 	}
       else
 	{
-	  SCM_ASSERT (SCM_CONSP (spec) && SCM_INUMP (SCM_CAR (spec)), spec, s_bad_spec, what);
+	  SCM_ASSERT (SCM_CONSP (spec) && SCM_INUMP (SCM_CAR (spec)), spec,
+		      s_bad_spec, what);
 	  s->lbnd = SCM_INUM (SCM_CAR (spec));
 	  sp = SCM_CDR (spec);
-	  SCM_ASSERT (SCM_INUMP (SCM_CAR (sp)) && SCM_NULLP (SCM_CDR (sp)),
-		  spec, s_bad_spec, what);
+	  SCM_ASSERT (SCM_NIMP (sp) && SCM_CONSP (sp)
+		      && SCM_INUMP (SCM_CAR (sp)) && SCM_NULLP (SCM_CDR (sp)),
+		      spec, s_bad_spec, what);
 	  s->ubnd = SCM_INUM (SCM_CAR (sp));
 	  s->inc = 1;
 	}
@@ -563,6 +569,7 @@ scm_dimensions_to_uniform_array (dims, prot, fill)
   scm_array_dim *s;
   SCM ra;
   if (SCM_INUMP (dims))
+    {
       if (SCM_INUM (dims) < SCM_LENGTH_MAX)
 	{
 	  SCM answer;
@@ -582,6 +589,7 @@ scm_dimensions_to_uniform_array (dims, prot, fill)
 	}
     else
       dims = scm_cons (dims, SCM_EOL);
+    }
   SCM_ASSERT (SCM_NULLP (dims) || (SCM_NIMP (dims) && SCM_CONSP (dims)),
 	  dims, SCM_ARG1, s_dimensions_to_uniform_array);
   ra = scm_shap2ra (dims, s_dimensions_to_uniform_array);
@@ -717,7 +725,7 @@ scm_make_shared_array (oldra, mapfunc, dims)
 	  return ra;
 	}
     }
-  imap = scm_apply (mapfunc, scm_list_reverse (inds), SCM_EOL);
+  imap = scm_apply (mapfunc, scm_reverse (inds), SCM_EOL);
   if (SCM_ARRAYP (oldra))
       i = (scm_sizet) scm_aind (oldra, imap, s_make_shared_array);
   else
@@ -797,7 +805,7 @@ scm_transpose_array (args)
   switch (SCM_TYP7 (ra))
     {
     default:
-    badarg:scm_wta (ra, (char *) SCM_ARGn, s_transpose_array);
+    badarg:scm_wta (ra, (char *) SCM_ARG1, s_transpose_array);
     case scm_tc7_bvect:
     case scm_tc7_string:
     case scm_tc7_byvect:
@@ -826,9 +834,11 @@ scm_transpose_array (args)
       ndim = 0;
       for (k = 0; k < SCM_ARRAY_NDIM (ra); k++)
 	{
+	  SCM_ASSERT (SCM_INUMP (ve[k]), ve[k], (SCM_ARG2 + k),
+		      s_transpose_array);
 	  i = SCM_INUM (ve[k]);
-	  SCM_ASSERT (SCM_INUMP (ve[k]) && i >= 0 && i < SCM_ARRAY_NDIM (ra),
-		  ve[k], SCM_ARG2, s_transpose_array);
+	  SCM_ASSERT (i >= 0 && i < SCM_ARRAY_NDIM (ra), ve[k],
+		      SCM_OUTOFRANGE, s_transpose_array);
 	  if (ndim < i)
 	    ndim = i;
 	}
@@ -865,7 +875,7 @@ scm_transpose_array (args)
 	      r->inc += s->inc;
 	    }
 	}
-      SCM_ASSERT (ndim <= 0, args, "bad argument scm_list", s_transpose_array);
+      SCM_ASSERT (ndim <= 0, args, "bad argument list", s_transpose_array);
       scm_ra_set_contp (res);
       return res;
     }
@@ -905,6 +915,7 @@ scm_enclose_array (axes)
     case scm_tc7_dvect:
     case scm_tc7_cvect:
     case scm_tc7_vector:
+    case scm_tc7_wvect:
     case scm_tc7_svect:
 #ifdef LONGLONGS
     case scm_tc7_llvect:
@@ -1028,6 +1039,7 @@ tail:
     case scm_tc7_llvect:
 #endif
     case scm_tc7_vector:
+    case scm_tc7_wvect:
       SCM_ASRTGO (SCM_NULLP (args) && SCM_INUMP (ind), wna);
       return pos >= 0 && pos < SCM_LENGTH (v) ? SCM_BOOL_T : SCM_BOOL_F;
     }
@@ -1101,7 +1113,7 @@ scm_uniform_vector_ref (v, args)
       else
 	return SCM_BOOL_F;
     case scm_tc7_string:
-      return SCM_MAKICHR (SCM_CHARS (v)[pos]);
+      return SCM_MAKICHR (SCM_UCHARS (v)[pos]);
     case scm_tc7_byvect:
       return SCM_MAKINUM (((char *)SCM_CHARS (v))[pos]);
 # ifdef SCM_INUMS_ONLY
@@ -1134,6 +1146,7 @@ scm_uniform_vector_ref (v, args)
 			 ((double *) SCM_CDR (v))[2 * pos + 1]);
 #endif
     case scm_tc7_vector:
+    case scm_tc7_wvect:
       return SCM_VELTS (v)[pos];
     }
 }
@@ -1158,7 +1171,7 @@ scm_cvref (v, pos, last)
       else
 	return SCM_BOOL_F;
     case scm_tc7_string:
-      return SCM_MAKICHR (SCM_CHARS (v)[pos]);
+      return SCM_MAKICHR (SCM_UCHARS (v)[pos]);
     case scm_tc7_byvect:
       return SCM_MAKINUM (((char *)SCM_CHARS (v))[pos]);
 # ifdef SCM_INUMS_ONLY
@@ -1209,6 +1222,7 @@ scm_cvref (v, pos, last)
 			 ((double *) SCM_CDR (v))[2 * pos + 1]);
 #endif
     case scm_tc7_vector:
+    case scm_tc7_wvect:
       return SCM_VELTS (v)[pos];
     case scm_tc7_smob:
       {				/* enclosed scm_array */
@@ -1238,7 +1252,7 @@ scm_array_set_x (v, obj, args)
      SCM obj;
      SCM args;
 {
-  long pos;
+  long pos = 0;
   SCM_ASRTGO (SCM_NIMP (v), badarg1);
   if (SCM_ARRAYP (v))
     {
@@ -1280,7 +1294,7 @@ scm_array_set_x (v, obj, args)
       break;
     case scm_tc7_string:
       SCM_ASRTGO (SCM_ICHRP (obj), badobj);
-      SCM_CHARS (v)[pos] = SCM_ICHR (obj);
+      SCM_UCHARS (v)[pos] = SCM_ICHR (obj);
       break;
     case scm_tc7_byvect:
       if (SCM_ICHRP (obj))
@@ -1297,7 +1311,7 @@ scm_array_set_x (v, obj, args)
   case scm_tc7_uvect:
     SCM_VELTS(v)[pos] = scm_num2ulong(obj, (char *)SCM_ARG2, s_array_set_x); break;
   case scm_tc7_ivect:
-    SCM_VELTS(v)[pos] = num2long(obj, (char *)SCM_ARG2, s_array_set_x); break;
+    SCM_VELTS(v)[pos] = scm_num2long(obj, (char *)SCM_ARG2, s_array_set_x); break;
 # endif
       break;
 
@@ -1315,13 +1329,11 @@ scm_array_set_x (v, obj, args)
 #ifdef SCM_FLOATS
 #ifdef SCM_SINGLES
     case scm_tc7_fvect:
-      SCM_ASRTGO (SCM_NIMP (obj) && SCM_REALP (obj), badobj);
-      ((float *) SCM_CDR (v))[pos] = SCM_REALPART (obj);
+      ((float *) SCM_CDR (v))[pos] = (float)scm_num2dbl(obj, s_array_set_x); break;
       break;
 #endif
     case scm_tc7_dvect:
-      SCM_ASRTGO (SCM_NIMP (obj) && SCM_REALP (obj), badobj);
-      ((double *) SCM_CDR (v))[pos] = SCM_REALPART (obj);
+      ((double *) SCM_CDR (v))[pos] = scm_num2dbl(obj, s_array_set_x); break;
       break;
     case scm_tc7_cvect:
       SCM_ASRTGO (SCM_NIMP (obj) && SCM_INEXP (obj), badobj);
@@ -1330,6 +1342,7 @@ scm_array_set_x (v, obj, args)
       break;
 #endif
     case scm_tc7_vector:
+    case scm_tc7_wvect:
       SCM_VELTS (v)[pos] = obj;
       break;
     }
@@ -1345,13 +1358,14 @@ scm_array_contents (ra, strict)
 {
   SCM sra;
   if (SCM_IMP (ra))
- return SCM_BOOL_F;
+    return SCM_BOOL_F;
   switch SCM_TYP7
     (ra)
     {
     default:
       return SCM_BOOL_F;
     case scm_tc7_vector:
+    case scm_tc7_wvect:
     case scm_tc7_string:
     case scm_tc7_bvect:
     case scm_tc7_byvect:
@@ -1374,8 +1388,6 @@ scm_array_contents (ra, strict)
 	  len *= SCM_ARRAY_DIMS (ra)[k].ubnd - SCM_ARRAY_DIMS (ra)[k].lbnd + 1;
 	if (!SCM_UNBNDP (strict))
 	  {
-	    if SCM_ARRAY_BASE
-	      (ra) return SCM_BOOL_F;
 	    if (ndim && (1 != SCM_ARRAY_DIMS (ra)[ndim - 1].inc))
 	      return SCM_BOOL_F;
 	    if (scm_tc7_bvect == SCM_TYP7 (SCM_ARRAY_V (ra)))
@@ -1457,7 +1469,7 @@ scm_uniform_array_read_x (ra, port_or_fd, start, end)
     port_or_fd = scm_cur_inp;
   else
     SCM_ASSERT (SCM_INUMP (port_or_fd)
-		|| (SCM_NIMP (port_or_fd) && SCM_OPINFPORTP (port_or_fd)),
+		|| (SCM_NIMP (port_or_fd) && SCM_OPINPORTP (port_or_fd)),
 		port_or_fd, SCM_ARG2, s_uniform_array_read_x);
   vlen = SCM_LENGTH (v);
 
@@ -1530,18 +1542,44 @@ loop:
 
   if (SCM_NIMP (port_or_fd))
     {
-      /* if we have stored a character from the port in our own buffer,
-	 push it back onto the stream.  */
-      /* An ungetc before an fread will not work on some systems if
-	 setbuf(0).  do #define NOSETBUF in scmfig.h to fix this. */
-      if (SCM_CRDYP (port_or_fd))
+      scm_port *pt = SCM_PTAB_ENTRY (port_or_fd);
+      int remaining = (cend - offset) * sz;
+      char *dest = SCM_CHARS (v) + (cstart + offset) * sz;
+
+      if (pt->rw_active == SCM_PORT_WRITE)
+	scm_flush (port_or_fd);
+
+      ans = cend - offset;
+      while (remaining > 0)
 	{
-	  ungetc (SCM_CGETUN (port_or_fd), (FILE *)SCM_STREAM (port_or_fd));
-	  SCM_CLRDY (port_or_fd); /* Clear ungetted char */
+	  if (pt->read_pos < pt->read_end)
+	    {
+	      int to_copy = min (pt->read_end - pt->read_pos,
+				 remaining);
+
+	      memcpy (dest, pt->read_pos, to_copy);
+	      pt->read_pos += to_copy;
+	      remaining -= to_copy;
+	      dest += to_copy;
+	    }
+	  else
+	    {
+	      if (scm_fill_input (port_or_fd) == EOF)
+		{
+		  if (remaining % sz != 0)
+		    {
+		      scm_misc_error (s_uniform_array_read_x,
+				      "unexpected EOF",
+				      SCM_EOL);
+		    }
+		  ans -= remaining / sz;
+		  break;
+		}
+	    }
 	}
-      SCM_SYSCALL (ans = fread (SCM_CHARS (v) + (cstart + offset) * sz,
-			       (scm_sizet) sz, (scm_sizet) (cend - offset),
-				(FILE *)SCM_STREAM (port_or_fd)));
+      
+      if (pt->rw_random)
+	pt->rw_active = SCM_PORT_READ;
     }
   else /* file descriptor.  */
     {
@@ -1574,12 +1612,14 @@ scm_uniform_array_write (v, port_or_fd, start, end)
   long cstart = 0;
   long cend;
 
+  port_or_fd = SCM_COERCE_OUTPORT (port_or_fd);
+
   SCM_ASRTGO (SCM_NIMP (v), badarg1);
   if (SCM_UNBNDP (port_or_fd))
     port_or_fd = scm_cur_outp;
   else
     SCM_ASSERT (SCM_INUMP (port_or_fd)
-		|| (SCM_NIMP (port_or_fd) && SCM_OPOUTFPORTP (port_or_fd)),
+		|| (SCM_NIMP (port_or_fd) && SCM_OPOUTPORTP (port_or_fd)),
 		port_or_fd, SCM_ARG2, s_uniform_array_write);
   vlen = SCM_LENGTH (v);
 
@@ -1652,9 +1692,10 @@ loop:
 
   if (SCM_NIMP (port_or_fd))
     {
-      SCM_SYSCALL (ans = fwrite (SCM_CHARS (v) + (cstart + offset) * sz,
-				 (scm_sizet) sz, (scm_sizet) (cend - offset),
-				 (FILE *)SCM_STREAM (port_or_fd)));
+      char *source = SCM_CHARS (v) + (cstart + offset) * sz;
+
+      ans = cend - offset;
+      scm_lfwrite (source, ans * sz, port_or_fd);
     }
   else /* file descriptor.  */
     {
@@ -2013,6 +2054,7 @@ scm_array_to_list (v)
       SCM_ASRTGO (SCM_ARRAYP (v), badarg1);
       return ra2l (v, SCM_ARRAY_BASE (v), 0);
     case scm_tc7_vector:
+    case scm_tc7_wvect:
       return scm_vector_to_list (v);
     case scm_tc7_string:
       return scm_string_to_list (v);
@@ -2097,7 +2139,7 @@ scm_array_to_list (v)
 }
 
 
-static char s_bad_ralst[] = "Bad scm_array contents scm_list";
+static char s_bad_ralst[] = "Bad scm_array contents list";
 
 static int l2ra SCM_P ((SCM lst, SCM ra, scm_sizet base, scm_sizet k));
 
@@ -2212,7 +2254,7 @@ tail:
 	    scm_iprin1 (ra, port, pstate);
 	  for (j += inc; n-- > 0; j += inc)
 	    {
-	      scm_gen_putc (' ', port);
+	      scm_putc (' ', port);
 	      SCM_ARRAY_BASE (ra) = j;
 	      scm_iprin1 (ra, port, pstate);
 	    }
@@ -2224,16 +2266,16 @@ tail:
 	  inc = SCM_ARRAY_DIMS (ra)[k].inc;
 	  for (i = SCM_ARRAY_DIMS (ra)[k].lbnd; i < SCM_ARRAY_DIMS (ra)[k].ubnd; i++)
 	    {
-	      scm_gen_putc ('(', port);
+	      scm_putc ('(', port);
 	      rapr1 (ra, j, k + 1, port, pstate);
-	      scm_gen_puts (scm_regular_string, ") ", port);
+	      scm_puts (") ", port);
 	      j += inc;
 	    }
 	  if (i == SCM_ARRAY_DIMS (ra)[k].ubnd)
 	    {			/* could be zero size. */
-	      scm_gen_putc ('(', port);
+	      scm_putc ('(', port);
 	      rapr1 (ra, j, k + 1, port, pstate);
-	      scm_gen_putc (')', port);
+	      scm_putc (')', port);
 	    }
 	  break;
 	}
@@ -2252,29 +2294,29 @@ tail:
 	scm_iprin1 (scm_uniform_vector_ref (ra, SCM_MAKINUM (j)), port, pstate);
       for (j += inc; n-- > 0; j += inc)
 	{
-	  scm_gen_putc (' ', port);
+	  scm_putc (' ', port);
 	  scm_iprin1 (scm_cvref (ra, j, SCM_UNDEFINED), port, pstate);
 	}
       break;
     case scm_tc7_string:
       if (n-- > 0)
-	scm_iprin1 (SCM_MAKICHR (SCM_CHARS (ra)[j]), port, pstate);
+	scm_iprin1 (SCM_MAKICHR (SCM_UCHARS (ra)[j]), port, pstate);
       if (SCM_WRITINGP (pstate))
 	for (j += inc; n-- > 0; j += inc)
 	  {
-	    scm_gen_putc (' ', port);
-	    scm_iprin1 (SCM_MAKICHR (SCM_CHARS (ra)[j]), port, pstate);
+	    scm_putc (' ', port);
+	    scm_iprin1 (SCM_MAKICHR (SCM_UCHARS (ra)[j]), port, pstate);
 	  }
       else
 	for (j += inc; n-- > 0; j += inc)
-	  scm_gen_putc (SCM_CHARS (ra)[j], port);
+	  scm_putc (SCM_CHARS (ra)[j], port);
       break;
     case scm_tc7_byvect:
       if (n-- > 0)
 	scm_intprint (((char *)SCM_CDR (ra))[j], 10, port);
       for (j += inc; n-- > 0; j += inc)
 	{
-	  scm_gen_putc (' ', port);
+	  scm_putc (' ', port);
 	  scm_intprint (((char *)SCM_CDR (ra))[j], 10, port);
 	}
       break;
@@ -2285,7 +2327,7 @@ tail:
 	scm_intprint (SCM_VELTS (ra)[j], 10, port);
       for (j += inc; n-- > 0; j += inc)
 	{
-	  scm_gen_putc (' ', port);
+	  scm_putc (' ', port);
 	  scm_intprint (SCM_VELTS (ra)[j], 10, port);
 	}
       break;
@@ -2295,7 +2337,7 @@ tail:
 	scm_intprint (((short *)SCM_CDR (ra))[j], 10, port);
       for (j += inc; n-- > 0; j += inc)
 	{
-	  scm_gen_putc (' ', port);
+	  scm_putc (' ', port);
 	  scm_intprint (((short *)SCM_CDR (ra))[j], 10, port);
 	}
       break;
@@ -2310,7 +2352,7 @@ tail:
 	  scm_floprint (z, port, pstate);
 	  for (j += inc; n-- > 0; j += inc)
 	    {
-	      scm_gen_putc (' ', port);
+	      scm_putc (' ', port);
 	      SCM_FLO (z) = ((float *) SCM_VELTS (ra))[j];
 	      scm_floprint (z, port, pstate);
 	    }
@@ -2325,7 +2367,7 @@ tail:
 	  scm_floprint (z, port, pstate);
 	  for (j += inc; n-- > 0; j += inc)
 	    {
-	      scm_gen_putc (' ', port);
+	      scm_putc (' ', port);
 	      SCM_REAL (z) = ((double *) SCM_VELTS (ra))[j];
 	      scm_floprint (z, port, pstate);
 	    }
@@ -2340,7 +2382,7 @@ tail:
 	  scm_floprint ((0.0 == SCM_IMAG (cz) ? z : cz), port, pstate);
 	  for (j += inc; n-- > 0; j += inc)
 	    {
-	      scm_gen_putc (' ', port);
+	      scm_putc (' ', port);
 	      SCM_REAL (z) = SCM_REAL (cz) = ((double *) SCM_VELTS (ra))[2 * j];
 	      SCM_IMAG (cz) = ((double *) SCM_VELTS (ra))[2 * j + 1];
 	      scm_floprint ((0.0 == SCM_IMAG (cz) ? z : cz), port, pstate);
@@ -2361,7 +2403,7 @@ scm_raprin1 (exp, port, pstate)
 {
   SCM v = exp;
   scm_sizet base = 0;
-  scm_gen_putc ('#', port);
+  scm_putc ('#', port);
 tail:
   switch SCM_TYP7
     (v)
@@ -2374,9 +2416,9 @@ tail:
 	if (SCM_ARRAYP (v))
 
 	  {
-	    scm_gen_puts (scm_regular_string, "<enclosed-array ", port);
+	    scm_puts ("<enclosed-array ", port);
 	    rapr1 (exp, base, 0, port, pstate);
-	    scm_gen_putc ('>', port);
+	    scm_putc ('>', port);
 	    return 1;
 	  }
 	else
@@ -2389,13 +2431,13 @@ tail:
       if (exp == v)
 	{			/* a uve, not an scm_array */
 	  register long i, j, w;
-	  scm_gen_putc ('*', port);
+	  scm_putc ('*', port);
 	  for (i = 0; i < (SCM_LENGTH (exp)) / SCM_LONG_BIT; i++)
 	    {
 	      w = SCM_VELTS (exp)[i];
 	      for (j = SCM_LONG_BIT; j; j--)
 		{
-		  scm_gen_putc (w & 1 ? '1' : '0', port);
+		  scm_putc (w & 1 ? '1' : '0', port);
 		  w >>= 1;
 		}
 	    }
@@ -2405,52 +2447,52 @@ tail:
 	      w = SCM_VELTS (exp)[SCM_LENGTH (exp) / SCM_LONG_BIT];
 	      for (; j; j--)
 		{
-		  scm_gen_putc (w & 1 ? '1' : '0', port);
+		  scm_putc (w & 1 ? '1' : '0', port);
 		  w >>= 1;
 		}
 	    }
 	  return 1;
 	}
       else
-	scm_gen_putc ('b', port);
+	scm_putc ('b', port);
       break;
     case scm_tc7_string:
-      scm_gen_putc ('a', port);
+      scm_putc ('a', port);
       break;
     case scm_tc7_byvect:
-      scm_gen_puts (scm_regular_string, "bytes", port);
+      scm_putc ('y', port);
       break;
     case scm_tc7_uvect:
-      scm_gen_putc ('u', port);
+      scm_putc ('u', port);
       break;
     case scm_tc7_ivect:
-      scm_gen_putc ('e', port);
+      scm_putc ('e', port);
       break;
     case scm_tc7_svect:
-      scm_gen_puts (scm_regular_string, "short", port);
+      scm_putc ('h', port);
       break;
 #ifdef LONGLONGS
     case scm_tc7_llvect:
-      scm_gen_puts (scm_regular_string, "long_long", port);
+      scm_puts ("long_long", port);
       break;
 #endif
 #ifdef SCM_FLOATS
 #ifdef SCM_SINGLES
     case scm_tc7_fvect:
-      scm_gen_putc ('s', port);
+      scm_putc ('s', port);
       break;
 #endif /*SCM_SINGLES*/
     case scm_tc7_dvect:
-      scm_gen_putc ('i', port);
+      scm_putc ('i', port);
       break;
     case scm_tc7_cvect:
-      scm_gen_putc ('c', port);
+      scm_putc ('c', port);
       break;
 #endif /*SCM_FLOATS*/
     }
-  scm_gen_putc ('(', port);
+  scm_putc ('(', port);
   rapr1 (exp, base, 0, port, pstate);
-  scm_gen_putc (')', port);
+  scm_putc (')', port);
   return 1;
 }
 
@@ -2475,6 +2517,7 @@ loop:
       ra = SCM_ARRAY_V (ra);
       goto loop;
     case scm_tc7_vector:
+    case scm_tc7_wvect:
       return SCM_EOL;
     case scm_tc7_bvect:
       return SCM_BOOL_T;
@@ -2512,9 +2555,6 @@ static SCM
 markra (ptr)
      SCM ptr;
 {
-  if SCM_GC8MARKP
-    (ptr) return SCM_BOOL_F;
-  SCM_SETGC8MARK (ptr);
   return SCM_ARRAY_V (ptr);
 }
 
@@ -2529,18 +2569,18 @@ freera (ptr)
   return sizeof (scm_array) + SCM_ARRAY_NDIM (ptr) * sizeof (scm_array_dim);
 }
 
-static scm_smobfuns rasmob =
-{markra, freera, scm_raprin1, scm_array_equal_p};
-
-
 /* This must be done after scm_init_scl() */
 
 void
 scm_init_unif ()
 {
-#include "unif.x"
-  scm_tc16_array = scm_newsmob (&rasmob);
+  scm_tc16_array = scm_make_smob_type_mfpe ("array", 0,
+					    markra,
+					    freera,
+					    scm_raprin1,
+					    scm_array_equal_p);
   scm_add_feature ("array");
+#include "unif.x"
 }
 
 #else /* ARRAYS */
