@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996,1998,1999 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -59,107 +59,69 @@
  */
 
 
-
-static int prinsfpt SCM_P ((SCM exp, SCM port, scm_print_state *pstate));
-
-static int 
-prinsfpt (exp, port, pstate)
-     SCM exp;
-     SCM port;
-     scm_print_state *pstate;
+static void
+sf_flush (SCM port)
 {
-  scm_prinport (exp, port, "soft");
-  return !0;
+  scm_port *pt = SCM_PTAB_ENTRY (port);
+  SCM stream = pt->stream;
+
+  if (pt->write_pos > pt->write_buf)
+    {
+      /* write the byte. */
+      scm_apply (SCM_VELTS (stream)[0], SCM_MAKICHR (*pt->write_buf),
+		 scm_listofnull);
+      pt->write_pos = pt->write_buf;
+  
+      /* flush the output.  */
+      {
+	SCM f = SCM_VELTS (stream)[2];
+
+	if (f != SCM_BOOL_F)
+	  scm_apply (f, SCM_EOL, SCM_EOL);
+      }
+    }
 }
 
-/* sfputc sfwrite sfputs sfclose 
- * are called within a SCM_SYSCALL.  
- *
- * So we need to set errno to 0 before returning.  sfflush
- * may be called within a SCM_SYSCALL.  So we need to set errno to 0
- * before returning.
- */
-
-
-static int sfputc SCM_P ((int c, SCM p));
-
-static int 
-sfputc (c, p)
-     int c;
-     SCM p;
+static void
+sf_write (SCM port, void *data, size_t size)
 {
-  scm_apply (SCM_VELTS (p)[0], SCM_MAKICHR (c), scm_listofnull);
-  errno = 0;
-  return c;
+  SCM p = SCM_STREAM (port);
+
+  scm_apply (SCM_VELTS (p)[1], scm_cons (scm_makfrom0str ((char *) data),
+					 SCM_EOL),
+	     SCM_EOL);
 }
 
+/* calling the flush proc (element 2) is in case old code needs it,
+   but perhaps softports could the use port buffer in the same way as
+   fports.  */
 
-static scm_sizet sfwrite SCM_P ((char *str, scm_sizet siz, scm_sizet num, SCM p));
-
-static scm_sizet 
-sfwrite (str, siz, num, p)
-     char *str;
-     scm_sizet siz;
-     scm_sizet num;
-     SCM p;
-{
-  SCM sstr;
-  sstr = scm_makfromstr (str, siz * num, 0);
-  scm_apply (SCM_VELTS (p)[1], sstr, scm_listofnull);
-  errno = 0;
-  return num;
-}
-
-
-static int sfputs SCM_P ((char *s, SCM p));
-
+/* places a single char in the input buffer.  */
 static int 
-sfputs (s, p)
-     char *s;
-     SCM p;
-{
-  sfwrite (s, 1, strlen (s), p);
-  return 0;
-}
-
-
-static int sfflush SCM_P ((SCM stream));
-
-static int 
-sfflush (stream)
-     SCM stream;
-{
-  SCM f = SCM_VELTS (stream)[2];
-  if (SCM_BOOL_F == f)
-    return 0;
-  f = scm_apply (f, SCM_EOL, SCM_EOL);
-  errno = 0;
-  return SCM_BOOL_F == f ? EOF : 0;
-}
-
-
-static int sfgetc SCM_P ((SCM p));
-
-static int 
-sfgetc (p)
-     SCM p;
+sf_fill_input (SCM port)
 {
+  SCM p = SCM_STREAM (port);
   SCM ans;
-  ans = scm_apply (SCM_VELTS (p)[3], SCM_EOL, SCM_EOL);
-  errno = 0;
-  if (SCM_FALSEP (ans) || SCM_EOF_VAL == ans)
+
+  ans = scm_apply (SCM_VELTS (p)[3], SCM_EOL, SCM_EOL); /* get char.  */
+  if (SCM_FALSEP (ans) || SCM_EOF_OBJECT_P (ans))
     return EOF;
-  SCM_ASSERT (SCM_ICHRP (ans), ans, SCM_ARG1, "getc");
-  return SCM_ICHR (ans);
+  SCM_ASSERT (SCM_ICHRP (ans), ans, SCM_ARG1, "sf_fill_input");
+  {
+    scm_port *pt = SCM_PTAB_ENTRY (port);    
+
+    *pt->read_buf = SCM_ICHR (ans);
+    pt->read_pos = pt->read_buf;
+    pt->read_end = pt->read_buf + 1;
+    return *pt->read_buf;
+  }
 }
 
 
-static int sfclose SCM_P ((SCM p));
-
 static int 
-sfclose (p)
-     SCM p;
+sf_close (SCM port)
 {
+  SCM p = SCM_STREAM (port);
   SCM f = SCM_VELTS (p)[4];
   if (SCM_BOOL_F == f)
     return 0;
@@ -177,7 +139,7 @@ scm_make_soft_port (pv, modes)
      SCM pv;
      SCM modes;
 {
-  struct scm_port_table * pt;
+  scm_port *pt;
   SCM z;
   SCM_ASSERT (SCM_NIMP (pv) && SCM_VECTORP (pv) && 5 == SCM_LENGTH (pv), pv, SCM_ARG1, s_make_soft_port);
   SCM_ASSERT (SCM_NIMP (modes) && SCM_ROSTRINGP (modes), modes, SCM_ARG2, s_make_soft_port);
@@ -188,40 +150,29 @@ scm_make_soft_port (pv, modes)
   SCM_SETCAR (z, scm_tc16_sfport | scm_mode_bits (SCM_ROCHARS (modes)));
   SCM_SETPTAB_ENTRY (z, pt);
   SCM_SETSTREAM (z, pv);
+  pt->read_pos = pt->read_buf = pt->read_end = &pt->shortbuf;
+  pt->write_buf = pt->write_pos = &pt->shortbuf;
+  pt->read_buf_size = pt->write_buf_size = 1;
+  pt->write_end = pt->write_buf + pt->write_buf_size;
+  pt->rw_random = 0;
   SCM_ALLOW_INTS;
   return z;
 }
 
 
-static int noop0 SCM_P ((SCM stream));
+void scm_make_sfptob (void); /* Called from ports.c */
 
-static int 
-noop0 (stream)
-     SCM stream;
+void
+scm_make_sfptob ()
 {
-  return 0;
+  long tc = scm_make_port_type ("soft", sf_fill_input, sf_write);
+  scm_set_port_mark (tc, scm_markstream);
+  scm_set_port_flush (tc, sf_flush);
+  scm_set_port_close (tc, sf_close);
 }
 
-
-scm_ptobfuns scm_sfptob =
-{
-  scm_markstream,
-  noop0,
-  prinsfpt,
-  0,
-  sfputc,
-  sfputs,
-  sfwrite,
-  sfflush,
-  sfgetc,
-  sfclose
-};
-
-
-
 void
 scm_init_vports ()
 {
 #include "vports.x"
 }
-
