@@ -16,7 +16,8 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -40,8 +41,7 @@
  *
  * If you write modifications of your own for GUILE, it is your choice
  * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.  
- */
+ * If you do not wish that, delete this exception notice.  */
 
 
 #include "libguile/__scm.h"
@@ -54,6 +54,7 @@
 #define SCM_IS_WHVEC_V(X) (SCM_VELTS(X)[-1] == 2)
 #define SCM_IS_WHVEC_B(X) (SCM_VELTS(X)[-1] == 3)
 #define SCM_IS_WHVEC_ANY(X) (SCM_VELTS(X)[-1])
+#define SCM_WVECT_GC_CHAIN(X) (SCM_VELTS(X)[-2])
 
 
 
